@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -38,16 +36,18 @@ charpoly0(GEN x, long v, long flag)
     case 2: return carhess(x,v);
     case 3: return carberkowitz(x,v);
     case 4:
-      if (typ(x) != t_MAT) pari_err(typeer,"charpoly");
+      if (typ(x) != t_MAT) pari_err_TYPE("charpoly",x);
       RgM_check_ZM(x, "charpoly");
       x = ZM_charpoly(x); setvarn(x, v); return x;
+    case 5:
+      return charpoly(x, v);
   }
-  pari_err(flagerr,"charpoly"); return NULL; /* not reached */
+  pari_err_FLAG("charpoly"); return NULL; /* not reached */
 }
 
 /* characteristic pol. Easy cases. Return NULL in case it's not so easy. */
 static GEN
-easychar(GEN x, long v, GEN *py)
+easychar(GEN x, long v)
 {
   pari_sp av;
   long lx;
@@ -60,11 +60,9 @@ easychar(GEN x, long v, GEN *py)
       p1=cgetg(4,t_POL);
       p1[1]=evalsigne(1) | evalvarn(v);
       gel(p1,2) = gneg(x); gel(p1,3) = gen_1;
-      if (py) { *py = cgetg(2, t_MAT); gel(*py,1) = mkcolcopy(x); }
       return p1;
 
     case t_COMPLEX: case t_QUAD:
-      if (py) pari_err(typeer,"easychar");
       p1 = cgetg(5,t_POL);
       p1[1] = evalsigne(1) | evalvarn(v);
       gel(p1,2) = gnorm(x); av = avma;
@@ -73,28 +71,59 @@ easychar(GEN x, long v, GEN *py)
 
     case t_FFELT: {
       pari_sp ltop=avma;
-      if (py) pari_err(typeer,"easychar");
       p1 = FpX_to_mod(FF_charpoly(x), FF_p_i(x));
       setvarn(p1,v); return gerepileupto(ltop,p1);
     }
 
     case t_POLMOD:
-      if (py) pari_err(typeer,"easychar");
       return RgXQ_charpoly(gel(x,2), gel(x,1), v);
 
     case t_MAT:
       lx=lg(x);
-      if (lx==1)
-      {
-        if (py) *py = cgetg(1,t_MAT);
-        return pol_1(v);
-      }
-      if (lg(x[1]) != lx) break;
+      if (lx==1) return pol_1(v);
+      if (lgcols(x) != lx) break;
       return NULL;
   }
-  pari_err(mattype1,"easychar");
+  pari_err_TYPE("easychar",x);
   return NULL; /* not reached */
 }
+/* compute charpoly by mapping to Fp first, return lift to Z */
+static GEN
+RgM_Fp_charpoly(GEN x, GEN p, long v)
+{
+  GEN T;
+  if (lgefint(p) == 3)
+  {
+    ulong pp = itou(p);
+    T = Flm_charpoly(RgM_to_Flm(x, pp), pp);
+    T = Flx_to_ZX(T);
+  }
+  else
+    T = FpM_charpoly(RgM_to_FpM(x, p), p);
+  setvarn(T, v); return T;
+}
+GEN
+charpoly(GEN x, long v)
+{
+  GEN T, p = NULL;
+  if ((T = easychar(x,v))) return T;
+  if (RgM_is_ZM(x))
+  {
+    T = ZM_charpoly(x);
+    setvarn(T, v);
+  }
+  else if (RgM_is_FpM(x, &p) && BPSW_psp(p))
+  {
+    pari_sp av = avma;
+    T = RgM_Fp_charpoly(x,p,v);
+    T = gerepileupto(av, FpX_to_mod(T,p));
+  }
+  else if (isinexact(x))
+    T = carhess(x, v);
+  else
+    T = carberkowitz(x, v);
+  return T;
+}
 
 /* We possibly worked with an "invalid" polynomial p, satisfying
  * varn(p) > gvar2(p). Fix this. */
@@ -102,7 +131,7 @@ static GEN
 fix_pol(pari_sp av, GEN p)
 {
   long w = gvar2(p), v = varn(p);
-  if (w == v) pari_err(talker,"incorrect variable in charpoly");
+  if (w == v) pari_err_PRIORITY("charpoly", p, "=", w);
   if (varncmp(w,v) < 0) p = gerepileupto(av, poleval(p, pol_x(v)));
   return p;
 }
@@ -113,7 +142,7 @@ caract(GEN x, long v)
   GEN  T, C, x_k, Q;
   long k, n;
 
-  if ((T = easychar(x,v,NULL))) return T;
+  if ((T = easychar(x,v))) return T;
 
   n = lg(x)-1;
   if (n == 1) return fix_pol(av, deg1pol(gen_1, gneg(gcoeff(x,1,1)), v));
@@ -134,6 +163,23 @@ caract(GEN x, long v)
   return fix_pol(av, RgX_Rg_div(T, mpfact(n)));
 }
 
+/* C = charpoly(x, v) */
+static GEN
+RgM_adj_from_char(GEN x, long v, GEN C)
+{
+  if (varn(C) != v) /* problem with variable priorities */
+  {
+    C = gdiv(gsub(C, gsubst(C, v, gen_0)), pol_x(v));
+    if (odd(lg(x))) C = RgX_neg(C); /* even dimension */
+    return gsubst(C, v, x);
+  }
+  else
+  {
+    C = RgX_shift_shallow(C, -1);
+    if (odd(lg(x))) C = RgX_neg(C); /* even dimension */
+    return RgX_RgM_eval(C, x);
+  }
+}
 /* assume x square matrice */
 static GEN
 mattrace(GEN x)
@@ -145,81 +191,106 @@ mattrace(GEN x)
   for (i = 2; i < lx; i++) t = gadd(t, gcoeff(x,i,i));
   return t;
 }
+static int
+bad_char(GEN q, long n)
+{
+  forprime_t S;
+  ulong p;
+  if (!signe(q)) return 0;
+  (void)u_forprime_init(&S, 2, n);
+  while ((p = u_forprime_next(&S)))
+    if (!umodiu(q, p)) return 1;
+  return 0;
+}
 /* Using traces: return the characteristic polynomial of x (in variable v).
  * If py != NULL, the adjoint matrix is put there. */
 GEN
 caradj(GEN x, long v, GEN *py)
 {
   pari_sp av, av0;
-  long i, k, l;
-  GEN p, y, t;
+  long i, k, n;
+  GEN T, y, t;
 
-  if ((p = easychar(x, v, py))) return p;
-
-  l = lg(x); av0 = avma;
-  p = cgetg(l+2,t_POL); p[1] = evalsigne(1) | evalvarn(v);
-  gel(p,l+1) = gen_1;
-  if (l == 1) { if (py) *py = cgetg(1,t_MAT); return p; }
-  av = avma; t = gerepileupto(av, gneg(mattrace(x)));
-  gel(p,l) = t;
-  if (l == 2) {
-    p = fix_pol(av0, p);
-    if (py) *py = matid(1); return p;
+  if ((T = easychar(x, v)))
+  {
+    if (py)
+    {
+      if (typ(x) != t_MAT) pari_err_TYPE("matadjoint",x);
+      *py = cgetg(1,t_MAT);
+    }
+    return T;
   }
-  if (l == 3) {
+
+  n = lg(x)-1; av0 = avma;
+  T = cgetg(n+3,t_POL); T[1] = evalsigne(1) | evalvarn(v);
+  gel(T,n+2) = gen_1;
+  if (!n) { if (py) *py = cgetg(1,t_MAT); return T; }
+  av = avma; t = gerepileupto(av, gneg(mattrace(x)));
+  gel(T,n+1) = t;
+  if (n == 1) {
+    T = fix_pol(av0, T);
+    if (py) *py = matid(1); return T;
+  }
+  if (n == 2) {
     GEN a = gcoeff(x,1,1), b = gcoeff(x,1,2);
     GEN c = gcoeff(x,2,1), d = gcoeff(x,2,2);
     av = avma;
-    gel(p,2) = gerepileupto(av, gsub(gmul(a,d), gmul(b,c)));
-    p = fix_pol(av0, p);
+    gel(T,2) = gerepileupto(av, gsub(gmul(a,d), gmul(b,c)));
+    T = fix_pol(av0, T);
     if (py) {
       y = cgetg(3, t_MAT);
       gel(y,1) = mkcol2(gcopy(d), gneg(c));
       gel(y,2) = mkcol2(gneg(b), gcopy(a));
       *py = y;
     }
-    return p;
+    return T;
   }
   /* l > 3 */
+  if (bad_char(residual_characteristic(x), n))
+  { /* n! not invertible in base ring */
+    T = charpoly(x, v);
+    if (!py) return gerepileupto(av, T);
+    *py = RgM_adj_from_char(x, v, T);
+    gerepileall(av, 2, &T,py);
+    return T;
+  }
   av = avma; y = RgM_shallowcopy(x);
-  for (i = 1; i < l; i++) gcoeff(y,i,i) = gadd(gcoeff(y,i,i), t);
-  for (k = 2; k < l-1; k++)
+  for (i = 1; i <= n; i++) gcoeff(y,i,i) = gadd(gcoeff(y,i,i), t);
+  for (k = 2; k < n; k++)
   {
     GEN y0 = y;
     y = RgM_mul(y, x);
     t = gdivgs(mattrace(y), -k);
-    for (i = 1; i < l; i++) gcoeff(y,i,i) = gadd(gcoeff(y,i,i), t);
+    for (i = 1; i <= n; i++) gcoeff(y,i,i) = gadd(gcoeff(y,i,i), t);
     y = gclone(y);
-    gel(p,l-k+1) = gerepilecopy(av, t); av = avma;
+    gel(T,n-k+2) = gerepilecopy(av, t); av = avma;
     if (k > 2) gunclone(y0);
   }
   t = gmul(gcoeff(x,1,1),gcoeff(y,1,1));
-  for (i=2; i<l; i++) t = gadd(t, gmul(gcoeff(x,1,i),gcoeff(y,i,1)));
-  gel(p,2) = gerepileupto(av, gneg(t));
-  p = fix_pol(av0, p);
-  if (py) *py = (l & 1)? RgM_neg(y): gcopy(y);
-  gunclone(y); return p;
+  for (i=2; i<=n; i++) t = gadd(t, gmul(gcoeff(x,1,i),gcoeff(y,i,1)));
+  gel(T,2) = gerepileupto(av, gneg(t));
+  T = fix_pol(av0, T);
+  if (py) *py = odd(n)? gcopy(y): RgM_neg(y);
+  gunclone(y); return T;
 }
 
 GEN
 adj(GEN x)
 {
   GEN y;
-  (void)caradj(x,MAXVARN,&y); return y;
+  (void)caradj(x, MAXVARN, &y); return y;
 }
 
 GEN
 adjsafe(GEN x)
 {
-  long n = lg(x)-1;
+  const long v = MAXVARN;
   pari_sp av = avma;
   GEN C;
-  if (typ(x) != t_MAT) pari_err(typeer, "matadjoint");
-  if (n <= 1) return gcopy(x);
-  C = carberkowitz(x, 0);
-  C = RgX_shift_shallow(C, -1);
-  if (!odd(n)) C = RgX_neg(C);
-  return gerepileupto(av, RgX_RgM_eval(C, x));
+  if (typ(x) != t_MAT) pari_err_TYPE("matadjoint",x);
+  if (lg(x) < 3) return gcopy(x);
+  C = charpoly(x,v);
+  return gerepileupto(av, RgM_adj_from_char(x, v, C));
 }
 
 GEN
@@ -230,7 +301,7 @@ matadjoint0(GEN x, long flag)
     case 0: return adj(x);
     case 1: return adjsafe(x);
   }
-  pari_err(flagerr,"matadjoint"); return NULL; /* not reached */
+  pari_err_FLAG("matadjoint"); return NULL; /* not reached */
 }
 
 /*******************************************************************/
@@ -246,7 +317,7 @@ easymin(GEN x, long v)
   GEN G, R, dR;
   if (typ(x)==t_POLMOD && !issquarefree(gel(x,1)))
     return NULL;
-  R=easychar(x, v, NULL);
+  R = easychar(x, v);
   if (!R) return R;
   dR=RgX_deriv(R);
   if (!lgpol(dR)) {avma=ltop; return NULL;}
@@ -275,7 +346,7 @@ minpoly(GEN x, long v)
     setvarn(P,v);
     return gerepileupto(ltop,P);
   }
-  if (typ(x)!=t_MAT) pari_err(typeer,"minpoly");
+  if (typ(x)!=t_MAT) pari_err_TYPE("minpoly",x);
   if (lg(x) == 1) return pol_1(v);
   return gerepilecopy(ltop,gel(matfrobenius(x,1,v),1));
 }
@@ -291,9 +362,9 @@ hess(GEN x)
   pari_sp av = avma, lim;
   long lx = lg(x), m, i, j;
 
-  if (typ(x) != t_MAT) pari_err(mattype1,"hess");
+  if (typ(x) != t_MAT) pari_err_TYPE("hess",x);
   if (lx == 1) return cgetg(1,t_MAT);
-  if (lg(x[1]) != lx) pari_err(mattype1,"hess");
+  if (lgcols(x) != lx) pari_err_DIM("hess");
 
   x = RgM_shallowcopy(x); lim = stack_lim(av,2);
   for (m=2; m<lx-1; m++)
@@ -327,12 +398,9 @@ hess(GEN x)
 GEN
 Flm_hess(GEN x, ulong p)
 {
-  pari_sp av = avma;
   long lx = lg(x), m, i, j;
-
-  if (typ(x) != t_MAT) pari_err(mattype1,"hess");
   if (lx == 1) return cgetg(1,t_MAT);
-  if (lg(x[1]) != lx) pari_err(mattype1,"hess");
+  if (lgcols(x) != lx) pari_err_DIM("hess");
 
   x = Flm_copy(x);
   for (m=2; m<lx-1; m++)
@@ -355,9 +423,49 @@ Flm_hess(GEN x, ulong p)
         ucoeff(x,j,m) = Fl_add(ucoeff(x,j,m), Fl_mul(c,ucoeff(x,j,i), p), p);
     }
   }
+  return x;
+}
+GEN
+FpM_hess(GEN x, GEN p)
+{
+  pari_sp av = avma, lim;
+  long lx = lg(x), m, i, j;
+  if (lx == 1) return cgetg(1,t_MAT);
+  if (lgcols(x) != lx) pari_err_DIM("hess");
+  if (lgefint(p) == 3)
+  {
+    ulong pp = p[2];
+    x = Flm_hess(ZM_to_Flm(x, pp), pp);
+    return gerepileupto(av, Flm_to_ZM(x));
+  }
+  x = RgM_shallowcopy(x); lim = stack_lim(av,2);
+  for (m=2; m<lx-1; m++)
+  {
+    GEN t = NULL;
+    for (i=m+1; i<lx; i++) { t = gcoeff(x,i,m-1); if (signe(t)) break; }
+    if (i == lx) continue;
+    for (j=m-1; j<lx; j++) swap(gcoeff(x,i,j), gcoeff(x,m,j));
+    swap(gel(x,i), gel(x,m)); t = Fp_inv(t, p);
+
+    for (i=m+1; i<lx; i++)
+    {
+      GEN c = gcoeff(x,i,m-1);
+      if (!signe(c)) continue;
+
+      c = Fp_mul(c,t, p); gcoeff(x,i,m-1) = gen_0;
+      for (j=m; j<lx; j++)
+        gcoeff(x,i,j) = Fp_sub(gcoeff(x,i,j), Fp_mul(c,gcoeff(x,m,j),p), p);
+      for (j=1; j<lx; j++)
+        gcoeff(x,j,m) = Fp_add(gcoeff(x,j,m), Fp_mul(c,gcoeff(x,j,i),p), p);
+      if (low_stack(lim, stack_lim(av,2)))
+      {
+        if (DEBUGMEM>1) pari_warn(warnmem,"hess, m = %ld", m);
+        gerepileall(av,2, &x, &t);
+      }
+    }
+  }
   return gerepilecopy(av,x);
 }
-
 GEN
 carhess(GEN x, long v)
 {
@@ -365,7 +473,7 @@ carhess(GEN x, long v)
   long lx, r, i;
   GEN y, H;
 
-  if ((H = easychar(x,v,NULL))) return H;
+  if ((H = easychar(x,v))) return H;
 
   lx = lg(x); av = avma; y = cgetg(lx+1, t_VEC);
   gel(y,1) = pol_1(v); H = hess(x);
@@ -385,6 +493,37 @@ carhess(GEN x, long v)
   }
   return fix_pol(av, gel(y,lx));
 }
+
+GEN
+FpM_charpoly(GEN x, GEN p)
+{
+  pari_sp av;
+  long lx, r, i;
+  GEN y, H;
+
+  /* Flm_charpoly left on stack */
+  if (lgefint(p) == 3) return Flx_to_ZX(Flm_charpoly(x, p[2]));
+  lx = lg(x); av = avma; y = cgetg(lx+1, t_VEC);
+  gel(y,1) = pol_1(0); H = FpM_hess(x, p);
+  for (r = 1; r < lx; r++)
+  {
+    pari_sp av2 = avma;
+    GEN z, a = gen_1, b = pol_0(0);
+    for (i = r-1; i; i--)
+    {
+      a = Fp_mul(a, gcoeff(H,i+1,i), p);
+      if (!signe(a)) break;
+      b = ZX_add(b, ZX_Z_mul(gel(y,i), Fp_mul(a,gcoeff(H,i,r),p)));
+    }
+    b = FpX_red(b, p);
+    z = FpX_sub(RgX_shift_shallow(gel(y,r), 1),
+                FpX_Fp_mul(gel(y,r), gcoeff(H,r,r), p), p);
+    z = FpX_sub(z,b,p);
+    if (r+1 == lx) { gel(y,lx) = z; break; }
+    gel(y,r+1) = gerepileupto(av2, z); /* (X - H[r,r])y[r] - b */
+  }
+  return gerepileupto(av, gel(y,lx));
+}
 GEN
 Flm_charpoly(GEN x, long p)
 {
@@ -418,7 +557,7 @@ static double
 charpoly_bound(GEN M)
 {
   pari_sp av = avma;
-  GEN s = real_0(3), bin, B2 = itor(sqri(ZM_supnorm(M)), 3);
+  GEN s = real_0(LOWDEFAULTPREC), bin, B2 = itor(sqri(ZM_supnorm(M)), LOWDEFAULTPREC);
   long n = lg(M)-1, k;
   double d;
   bin = gen_1;
@@ -438,8 +577,8 @@ ZM_charpoly(GEN M)
   pari_sp av = avma;
   long l = lg(M), n = l-1, bit;
   GEN q = NULL, H = NULL, Hp;
+  forprime_t S;
   ulong p;
-  byteptr d;
   if (!n) return pol_1(0);
 
   bit = (long)charpoly_bound(M) + 1;
@@ -447,10 +586,9 @@ ZM_charpoly(GEN M)
     err_printf("ZM_charpoly: bit-bound 2^%ld\n", bit);
     timer_start(&T);
   }
-  d = init_modular(&p);
-  for(;;)
+  init_modular(&S);
+  while ((p = u_forprime_next(&S)))
   {
-    NEXT_PRIME_VIADIFF_CHECK(p,d);
     Hp = Flm_charpoly(ZM_to_Flm(M, p), p);
     if (!H)
     {
@@ -462,15 +600,14 @@ ZM_charpoly(GEN M)
     }
     else
     {
-      GEN qp = muliu(q, p);
-      int stable = ZX_incremental_CRT(&H, Hp, q,qp, p);
+      int stable = ZX_incremental_CRT(&H, Hp, &q,p);
       if (DEBUGLEVEL>5)
         timer_printf(&T, "charpoly mod %lu (stable=%ld), bound = 2^%ld",
-                     p, stable, expi(qp));
-      if (stable && expi(qp) > bit) break;
-      q = qp;
+                     p, stable, expi(q));
+      if (stable && expi(q) > bit) break;
     }
   }
+  if (!p) pari_err_OVERFLOW("ZM_charpoly [ran out of primes]");
   return gerepilecopy(av, H);
 }
 
@@ -485,8 +622,7 @@ carberkowitz(GEN x, long v)
   long lx, i, j, k, r;
   GEN V, S, C, Q;
   pari_sp av0, av, lim;
-  if ((V = easychar(x,v,NULL))) return V;
-
+  if ((V = easychar(x,v))) return V;
   lx = lg(x); av0 = avma; lim = stack_lim(av0,1);
   V = cgetg(lx+1, t_VEC);
   S = cgetg(lx+1, t_VEC);
@@ -495,10 +631,8 @@ carberkowitz(GEN x, long v)
   av = avma;
   gel(C,1) = gen_m1;
   gel(V,1) = gen_m1;
+  for (i=2;i<=lx; i++) gel(C,i) = gel(Q,i) = gel(S,i) = gel(V,i) = gen_0;
   gel(V,2) = gcoeff(x,1,1);
-  for (i=2;i<=lx; i++) gel(S,i) = gen_0;
-  for (i=2;i<=lx; i++) gel(C,i) = gen_0;
-  for (i=2;i<=lx; i++) gel(Q,i) = gen_0;
   for (r = 2; r < lx; r++)
   {
     pari_sp av2;
@@ -570,19 +704,32 @@ gnorm(GEN x)
       gel(y,2) = FF_norm(x); return y;
 
     case t_POLMOD:
-      return RgXQ_norm(gel(x,2), gel(x,1));
-
+    {
+      GEN T = gel(x,1), a = gel(x,2);
+      if (typ(a) != t_POL || varn(a) != varn(T)) return gpowgs(a, degpol(T));
+      return RgXQ_norm(a, T);
+    }
     case t_VEC: case t_COL: case t_MAT:
       y = cgetg_copy(x, &lx);
       for (i=1; i<lx; i++) gel(y,i) = gnorm(gel(x,i));
       return y;
   }
-  pari_err(typeer,"gnorm");
+  pari_err_TYPE("gnorm",x);
   return NULL; /* not reached */
 }
 
-GEN
-gnorml2(GEN x)
+/* return |q|^2, complex modulus */
+static GEN
+cxquadnorm(GEN q, long prec)
+{
+  GEN X = gel(q,1), c = gel(X,2); /* (1-D)/4, -D/4 */
+  if (signe(c) > 0) return quadnorm(q); /* imaginary */
+  if (!prec) pari_err_TYPE("gnorml2", q);
+  return sqrr(quadtofp(q, prec));
+}
+
+static GEN
+gnorml2_i(GEN x, long prec)
 {
   pari_sp av, lim;
   long i, lx;
@@ -594,7 +741,7 @@ gnorml2(GEN x)
     case t_REAL: return sqrr(x);
     case t_FRAC: return sqrfrac(x);
     case t_COMPLEX: av = avma; return gerepileupto(av, cxnorm(x));
-    case t_QUAD:    av = avma; return gerepileupto(av, quadnorm(x));
+    case t_QUAD:    av = avma; return gerepileupto(av, cxquadnorm(x,prec));
 
     case t_POL: lx = lg(x)-1; x++; break;
 
@@ -602,7 +749,7 @@ gnorml2(GEN x)
     case t_COL:
     case t_MAT: lx = lg(x); break;
 
-    default: pari_err(typeer,"gnorml2");
+    default: pari_err_TYPE("gnorml2",x);
       return NULL; /* not reached */
   }
   if (lx == 1) return gen_0;
@@ -619,6 +766,68 @@ gnorml2(GEN x)
   }
   return gerepileupto(av,s);
 }
+GEN
+gnorml2(GEN x) { return gnorml2_i(x, 0); }
+
+static GEN pnormlp(GEN,GEN,long);
+static GEN
+pnormlpvec(long i0, GEN x, GEN p, long prec)
+{
+  pari_sp av = avma, lim = stack_lim(av,1);
+  long i, lx = lg(x);
+  GEN s = gen_0;
+  for (i=i0; i<lx; i++)
+  {
+    s = gadd(s, pnormlp(gel(x,i),p,prec));
+    if (low_stack(lim, stack_lim(av,1)))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"gnormlp, i = %ld", i);
+      s = gerepileupto(av, s);
+    }
+  }
+  return s;
+}
+/* (||x||_p)^p */
+static GEN
+pnormlp(GEN x, GEN p, long prec)
+{
+  switch(typ(x))
+  {
+    case t_INT: case t_REAL: x = mpabs(x); break;
+    case t_FRAC: x = absfrac(x); break;
+    case t_COMPLEX: case t_QUAD: x = gabs(x,prec); break;
+    case t_POL: return pnormlpvec(2, x, p, prec);
+    case t_VEC: case t_COL: case t_MAT: return pnormlpvec(1, x, p, prec);
+    default: pari_err_TYPE("gnormlp",x);
+  }
+  return gpow(x, p, prec);
+}
+
+GEN
+gnormlp(GEN x, GEN p, long prec)
+{
+  pari_sp av = avma;
+  if (!p) return gsupnorm(x, prec);
+  if (gsigne(p) <= 0) pari_err_DOMAIN("normlp", "p", "<=", gen_0, p);
+  if (is_scalar_t(typ(x))) return gabs(x, prec);
+  if (typ(p) == t_INT)
+  {
+    ulong pp = itou_or_0(p);
+    switch(pp)
+    {
+      case 1: return gnorml1(x, prec);
+      case 2: x = gnorml2_i(x, prec); break;
+      default: x = pnormlp(x, p, prec); break;
+    }
+    if (pp && typ(x) == t_INT && Z_ispowerall(x, pp, &x))
+      return gerepileuptoleaf(av, x);
+    if (pp == 2) return gerepileupto(av, gsqrt(x, prec));
+  }
+  else
+    x = pnormlp(x, p, prec);
+  x = gpow(x, ginv(p), prec);
+  return gerepileupto(av, x);
+}
 
 GEN
 gnorml1(GEN x,long prec)
@@ -644,7 +853,7 @@ gnorml1(GEN x,long prec)
       for (i=1; i<lx; i++) s = gadd(s, gnorml1(gel(x,i),prec));
       break;
 
-    default: pari_err(typeer,"gnorml1");
+    default: pari_err_TYPE("gnorml1",x);
       return NULL; /* not reached */
   }
   return gerepileupto(av, s);
@@ -680,7 +889,7 @@ gnorml1_fake(GEN x)
       for (i=1; i<lx; i++) s = gadd(s, gnorml1_fake(gel(x,i)));
       break;
 
-    default: pari_err(typeer,"gnorml1_fake");
+    default: pari_err_TYPE("gnorml1_fake",x);
       return NULL; /* not reached */
   }
   return gerepileupto(av, s);
@@ -691,14 +900,14 @@ store(GEN z, GEN *m) { if (!*m || gcmp(z, *m) > 0) *m = z; }
 /* compare |x| to *m or |x|^2 to *msq, whichever is easiest, and update
  * the pointed value if x is larger */
 void
-gsupnorm_aux(GEN x, GEN *m, GEN *msq)
+gsupnorm_aux(GEN x, GEN *m, GEN *msq, long prec)
 {
   long i, lx;
   GEN z;
   switch(typ(x))
   {
     case t_COMPLEX: z = cxnorm(x); store(z, msq); return;
-    case t_QUAD:  z = quadnorm(x); store(z, msq); return;
+    case t_QUAD:  z = cxquadnorm(x,prec); store(z, msq); return;
     case t_INT: case t_REAL: z = mpabs(x); store(z,m); return;
     case t_FRAC: z = absfrac(x); store(z,m); return;
 
@@ -708,17 +917,17 @@ gsupnorm_aux(GEN x, GEN *m, GEN *msq)
     case t_COL:
     case t_MAT: lx = lg(x); break;
 
-    default: pari_err(typeer,"gsupnorm");
+    default: pari_err_TYPE("gsupnorm",x);
       return; /* not reached */
   }
-  for (i=1; i<lx; i++) gsupnorm_aux(gel(x,i), m, msq);
+  for (i=1; i<lx; i++) gsupnorm_aux(gel(x,i), m, msq, prec);
 }
 GEN
 gsupnorm(GEN x, long prec)
 {
   GEN m = NULL, msq = NULL;
   pari_sp av = avma;
-  gsupnorm_aux(x, &m, &msq);
+  gsupnorm_aux(x, &m, &msq, prec);
   /* now set m = max (m, sqrt(msq)) */
   if (msq) {
     msq = gsqrt(msq, prec);
@@ -738,9 +947,9 @@ matcompanion(GEN x)
   long n = degpol(x), j;
   GEN y, c;
 
-  if (typ(x)!=t_POL) pari_err(notpoler,"matcompanion");
+  if (typ(x)!=t_POL) pari_err_TYPE("matcompanion",x);
+  if (!signe(x)) pari_err_DOMAIN("matcompanion","polynomial","=",gen_0,x);
   if (n == 0) return cgetg(1, t_MAT);
-  if (gequal0(x)) pari_err(zeropoler,"matcompanion");
 
   y = cgetg(n+1,t_MAT);
   for (j=1; j < n; j++) gel(y,j) = col_ei(n, j+1);
@@ -816,14 +1025,15 @@ gtrace(GEN x)
     case t_MAT:
       lx = lg(x); if (lx == 1) return gen_0;
       /*now lx >= 2*/
-      if (lx != lg(x[1])) pari_err(mattype1,"gtrace");
+      if (lx != lgcols(x)) pari_err_DIM("gtrace");
       av = avma; return gerepileupto(av, mattrace(x));
   }
-  pari_err(typeer,"gtrace");
+  pari_err_TYPE("gtrace",x);
   return NULL; /* not reached */
 }
 
-/* Gauss decomposition for positive definite matrix a [matrix Q, Algo 2.7.6]
+/* Cholesky decomposition for positive definite matrix a
+ * [GTM138, Algo 2.7.6, matrix Q]
  * If a is not positive definite return NULL. */
 GEN
 qfgaussred_positive(GEN a)
@@ -832,16 +1042,16 @@ qfgaussred_positive(GEN a)
   GEN b;
   long i,j,k, n = lg(a);
 
-  if (typ(a)!=t_MAT) pari_err(typeer,"qfgaussred_positive");
+  if (typ(a)!=t_MAT) pari_err_TYPE("qfgaussred_positive",a);
   if (n == 1) return cgetg(1, t_MAT);
-  if (lg(a[1])!=n) pari_err(mattype1,"qfgaussred_positive");
+  if (lgcols(a)!=n) pari_err_DIM("qfgaussred_positive");
   b = cgetg(n,t_MAT);
   for (j=1; j<n; j++)
   {
     GEN p1=cgetg(n,t_COL), p2=gel(a,j);
 
     gel(b,j) = p1;
-    for (i=1; i<=j; i++) p1[i] = p2[i];
+    for (i=1; i<=j; i++) gel(p1,i) = gel(p2,i);
     for (   ; i<n ; i++) gel(p1,i) = gen_0;
   }
   for (k=1; k<n; k++)
@@ -866,19 +1076,39 @@ qfgaussred_positive(GEN a)
   return gerepilecopy(av,b);
 }
 
+/* Maximal pivot strategy: x is a suitable pivot if it is non zero and either
+ * - an exact type, or
+ * - it is maximal among remaining non-zero (t_REAL) pivots */
+static int
+suitable(GEN x, long k, GEN *pp, long *pi)
+{
+  long t = typ(x);
+  switch(t)
+  {
+    case t_INT: return signe(x) != 0;
+    case t_FRAC: return 1;
+    case t_REAL: {
+      GEN p = *pp;
+      if (signe(x) && (!p || absr_cmp(p, x) < 0)) { *pp = x; *pi = k; }
+      return 0;
+    }
+    default: return !gequal0(x);
+  }
+}
+
 /* Gauss reduction (arbitrary symetric matrix, only the part above the
  * diagonal is considered). If signature is non-zero, return only the
  * signature, in which case gsigne() should be defined for elements of a. */
 static GEN
 gaussred(GEN a, long signature)
 {
-  GEN r, p, invp, ak, al;
+  GEN r, ak, al;
   pari_sp av, av1, lim;
   long n = lg(a), i, j, k, l, sp, sn, t;
 
-  if (typ(a) != t_MAT) pari_err(typeer,"gaussred");
+  if (typ(a) != t_MAT) pari_err_TYPE("gaussred",a);
   if (n == 1) return signature? mkvec2(gen_0, gen_0): cgetg(1, t_MAT);
-  if (lg(a[1]) != n) pari_err(mattype1,"gaussred");
+  if (lgcols(a) != n) pari_err_DIM("gaussred");
   n--;
 
   av = avma;
@@ -888,7 +1118,10 @@ gaussred(GEN a, long signature)
   t = n; sp = sn = 0;
   while (t)
   {
-    k=1; while (k<=n && (!r[k] || gequal0(gcoeff(a,k,k)))) k++;
+    long pind = 0;
+    GEN invp, p = NULL;
+    k=1; while (k<=n && (!r[k] || !suitable(gcoeff(a,k,k), k, &p, &pind))) k++;
+    if (k > n && p) k = pind;
     if (k <= n)
     {
       p = gcoeff(a,k,k); invp = ginv(p); /* != 0 */
@@ -913,7 +1146,8 @@ gaussred(GEN a, long signature)
     { /* all remaining diagonal coeffs are currently 0 */
       for (k=1; k<=n; k++) if (r[k])
       {
-        l=k+1; while (l<=n && (!r[l] || gequal0(gcoeff(a,k,l)))) l++;
+        l=k+1; while (l<=n && (!r[l] || !suitable(gcoeff(a,k,l), l, &p, &pind))) l++;
+        if (l > n && p) l = pind;
         if (l > n) continue;
 
         p = gcoeff(a,k,l); invp = ginv(p);
@@ -988,12 +1222,12 @@ jacobi(GEN a, long prec)
   long de, e, e1, e2, i, j, p, q, l = lg(a);
   GEN c, ja, L, r, L2, r2, unr;
 
-  if (typ(a) != t_MAT) pari_err(mattype1,"jacobi");
+  if (typ(a) != t_MAT) pari_err_TYPE("jacobi",a);
   ja = cgetg(3,t_VEC);
   L = cgetg(l,t_COL); gel(ja,1) = L;
   r = cgetg(l,t_MAT); gel(ja,2) = r;
   if (l == 1) return ja;
-  if (lg(a[1]) != l) pari_err(mattype1,"jacobi");
+  if (lgcols(a) != l) pari_err_DIM("jacobi");
 
   e1 = HIGHEXPOBIT-1;
   for (j=1; j<l; j++)
@@ -1023,7 +1257,7 @@ jacobi(GEN a, long prec)
     }
   }
   a = c; unr = real_1(prec);
-  de = bit_accuracy(prec);
+  de = prec2nbits(prec);
 
  /* e1 = min expo(a[i,i])
   * e2 = max expo(a[i,j]), i != j */
@@ -1052,7 +1286,7 @@ jacobi(GEN a, long prec)
     for (i=p+1; i<q; i++) rot(gcoeff(a,p,i), gcoeff(a,i,q), s,u);
     for (i=q+1; i<l; i++) rot(gcoeff(a,p,i), gcoeff(a,q,i), s,u);
     y = gcoeff(a,p,q);
-    t = mulrr(t, y); setexpo(y, expo(y)-de-1);
+    t = mulrr(t, y); shiftr_inplace(y, -de - 1);
     x = gel(L,p); subrrz(x,t, x);
     y = gel(L,q); addrrz(y,t, y);
     for (i=1; i<l; i++) rot(gcoeff(r,i,p), gcoeff(r,i,q), s,u);
@@ -1091,13 +1325,13 @@ jacobi(GEN a, long prec)
 GEN
 matrixqz0(GEN x,GEN p)
 {
-  if (typ(x) != t_MAT) pari_err(typeer,"QM_minors_coprime");
+  if (typ(x) != t_MAT) pari_err_TYPE("QM_minors_coprime",x);
   if (!p) return QM_minors_coprime(x,NULL);
-  if (typ(p) != t_INT) pari_err(typeer,"QM_minors_coprime");
+  if (typ(p) != t_INT) pari_err_TYPE("QM_minors_coprime",p);
   if (signe(p)>=0) return QM_minors_coprime(x,p);
   if (equaliu(p,1)) return QM_ImZ_hnf(x); /* p = -1 */
   if (equaliu(p,2)) return QM_ImQ_hnf(x); /* p = -2 */
-  pari_err(flagerr,"QM_minors_coprime"); return NULL; /* not reached */
+  pari_err_FLAG("QM_minors_coprime"); return NULL; /* not reached */
 }
 
 GEN
@@ -1108,23 +1342,22 @@ QM_minors_coprime(GEN x, GEN D)
   GEN P, y;
 
   n = lg(x)-1; if (!n) return gcopy(x);
-  m = lg(x[1])-1;
-  if (n > m) pari_err(talker,"need more rows than columns in QM_minors_coprime");
-  if (n==m)
-  {
-    D = det(x);
-    if (gequal0(D)) pari_err(talker,"matrix of non-maximal rank in QM_minors_coprime");
-    avma = av; return matid(n);
-  }
-  /* m > n */
+  m = nbrows(x);
+  if (n > m) pari_err_DOMAIN("QM_minors_coprime","n",">",strtoGENstr("m"),x);
   y = x; x = cgetg(n+1,t_MAT);
   for (j=1; j<=n; j++)
   {
     gel(x,j) = Q_primpart(gel(y,j));
     RgV_check_ZV(gel(x,j), "QM_minors_coprime");
   }
-
   /* x now a ZM */
+  if (n==m)
+  {
+    if (gequal0(ZM_det(x)))
+      pari_err_DOMAIN("QM_minors_coprime", "rank(A)", "<",stoi(n),x);
+    avma = av; return matid(n);
+  }
+  /* m > n */
   if (!D || gequal0(D))
   {
     pari_sp av2 = avma;
@@ -1151,7 +1384,7 @@ QM_minors_coprime(GEN x, GEN D)
       }
       if (low_stack(lim, stack_lim(av1,1)))
       {
-        if (DEBUGMEM>1) pari_warn(warnmem,"QM_minors_coprime, prime p = %Ps", p);
+        if (DEBUGMEM>1) pari_warn(warnmem,"QM_minors_coprime, p = %Ps", p);
         x = gerepilecopy(av1, x); pov2 = shifti(p, -1);
       }
     }
@@ -1236,7 +1469,7 @@ QM_imZ_hnf_aux(GEN A)
     if (!c) A = ZM_copy(A); else if ( isintzero(c) ) A = cgetg(1,t_MAT);
     return A;
   }
-  m = lg(A[1]);
+  m = lgcols(A);
   for (i=1; i<m; i++)
   {
     for (j = k = 1; j<n; j++)
@@ -1278,8 +1511,8 @@ QM_ImQ_hnf(GEN x)
   GEN c;
 
   n = lg(x); if (n==1) return gcopy(x);
-  m = lg(x[1]); x = RgM_shallowcopy(x);
-  c = const_vecsmall(n-1, 0);
+  m = lgcols(x); x = RgM_shallowcopy(x);
+  c = zero_zv(n-1);
   av1 = avma; lim = stack_lim(av1,1);
   for (k=1; k<m; k++)
   {
@@ -1309,7 +1542,8 @@ intersect(GEN x, GEN y)
   pari_sp av;
   GEN z;
 
-  if (typ(x)!=t_MAT || typ(y)!=t_MAT) pari_err(typeer,"intersect");
+  if (typ(x)!=t_MAT) pari_err_TYPE("intersect",x);
+  if (typ(y)!=t_MAT) pari_err_TYPE("intersect",y);
   if (lx==1 || lg(y)==1) return cgetg(1,t_MAT);
 
   av = avma; z = ker(shallowconcat(x,y));
