@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -22,32 +20,36 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
+static ulong _maxprime = 0;
+static ulong diffptrlen;
+
 /* Building/Rebuilding the diffptr table. The actual work is done by the
  * following two subroutines;  the user entry point is the function
  * initprimes() below.  initprimes1() is the old algorithm, called when
- * maxnum (size) is moderate. */
-static byteptr
-initprimes1(ulong size, long *lenp, long *lastp)
+ * maxnum (size) is moderate. Must be called after pari_init_stack() )*/
+static void
+initprimes1(ulong size, long *lenp, ulong *lastp, byteptr p1)
 {
+  pari_sp av = avma;
   long k;
-  byteptr q, r, s, p = (byteptr)pari_calloc(size+2), fin = p + size;
+  byteptr q, r, s, p = (byteptr)stack_calloc(size+2), fin = p + size;
 
   for (r=q=p,k=1; r<=fin; )
   {
     do { r+=k; k+=2; r+=k; } while (*++q);
     for (s=r; s<=fin; s+=k) *s = 1;
   }
-  r = p; *r++ = 2; *r++ = 1; /* 2 and 3 */
-  for (s=q=r-1; ; s=q)
+  r = p1; *r++ = 2; *r++ = 1; /* 2 and 3 */
+  for (s=q=p+1; ; s=q)
   {
     do q++; while (*q);
     if (q > fin) break;
     *r++ = (unsigned char) ((q-s) << 1);
   }
   *r++ = 0;
-  *lenp = r - p;
+  *lenp = r - p1;
   *lastp = ((s - p) << 1) + 1;
-  return (byteptr) pari_realloc(p,r-p);
+  avma = av;
 }
 
 /*  Timing in ms (Athlon/850; reports 512K of secondary cache; looks
@@ -140,7 +142,7 @@ initprimes1(ulong size, long *lenp, long *lastp)
   /* SLOW2_IN_ROOTS below 3: some slowdown starts to be noticable
    * when things fit into the cache on Sparc.
    * The choice of 2.6 gives a slowdown of 1-2% on UltraSparcII,
-   * but makes calculations for "maximum" of 436273009 (not applicable anymore)
+   * but makes calculations for "maximum" of 436273009
    * fit into 256K cache (still common for some architectures).
    *
    * One may change it when small caches become uncommon, but the gain
@@ -176,7 +178,7 @@ static cache_model_t cache_model = { CACHE_ARENA, CACHE_ALPHA, CACHE_CUTOFF };
 /* Assume that some calculation requires a chunk of memory to be
    accessed often in more or less random fashion (as in sieving).
    Assume that the calculation can be done in steps by subdividing the
-   chunk into smaller subchunks (arenas) and treathing them
+   chunk into smaller subchunks (arenas) and treating them
    separately.  Assume that the overhead of subdivision is equivalent
    to the number of arenas.
 
@@ -193,7 +195,7 @@ static cache_model_t cache_model = { CACHE_ARENA, CACHE_ALPHA, CACHE_CUTOFF };
  */
 static ulong
 good_arena_size(ulong slow2_size, ulong total, ulong fixed_to_cache,
-                cache_model_t *cache_model, long model_type)
+                cache_model_t *cache_model)
 {
   ulong asize, cache_arena = cache_model->arena;
   double Xmin, Xmax, A, B, C1, C2, D, V;
@@ -209,21 +211,16 @@ good_arena_size(ulong slow2_size, ulong total, ulong fixed_to_cache,
      [The latter is hard to substantiate theoretically, but this
      function describes benchmarks pretty close; it does not hurt that
      one can minimize it explicitly too ;-).  The switch between
-     diffenent choices of max() happens whe overhead=0.018.]
+     different choices of max() happens when overhead=0.018.]
 
      Thus the problem is minimizing (1 + slow2_size/arena)*overhead**0.29.
      This boils down to F=((X+A)/(X+B))X^alpha, X=overhead,
      B = (1 - fixed_to_cache/cache_arena), A = B +
-     slow2_size/cache_arena, alpha = 0.38, and X>=0.018, X>-B.  It
-     turns out that the minimization problem is not very nasty.  (As
-     usual with minimization problems which depend on parameters,
-     different values of A,B lead to different algorithms.  However,
-     the boundaries between the domains in (A,B) plane where different
-     algorithms work are explicitly calculatable.)
+     slow2_size/cache_arena, alpha = 0.38, and X>=0.018, X>-B.
 
-     Thus we need to find the rightmost root of (X+A)*(X+B) -
-     alpha(A-B)X to the right of 0.018 (if such exists and is below
-     Xmax).  Then we manually check the remaining region [0, 0.018].
+     We need to find the rightmost root of (X+A)*(X+B) - alpha(A-B)X to the
+     right of 0.018 (if such exists and is below Xmax).  Then we manually
+     check the remaining region [0, 0.018].
 
      Since we cannot trust the purely-experimental cache-hit slowdown
      function, as a sanity check always prefer fitting into the
@@ -232,9 +229,6 @@ good_arena_size(ulong slow2_size, ulong total, ulong fixed_to_cache,
 
    */
 
-  if (model_type != 0)
-      pari_err(talker, "unsupported type of cache model"); /* Future expansion */
-
   /* The simplest case: we fit into cache */
   if (total + fixed_to_cache <= cache_arena)
       return total;
@@ -324,7 +318,7 @@ set_optimize(long what, GEN g)
     ret = (long)(cache_model.cutoff * 1000);
     break;
   default:
-    pari_err(talker, "panic: set_optimize");
+    pari_err_BUG("set_optimize");
     break;
   }
   if (g != NULL) {
@@ -340,188 +334,170 @@ set_optimize(long what, GEN g)
   return ret;
 }
 
+/* s is odd; prime differences (starting from 5-3=2) start at known_primes[2],
+  terminated by a 0 byte. Checks n odd numbers starting at 'start', setting
+  bytes starting at data to 0 (composite) or 1 (prime) */
 static void
-sieve_chunk(byteptr known_primes, ulong s, byteptr data, ulong count)
-{   /* start must be odd;
-       prime differences (starting from (5-3)=2) start at known_primes[2],
-       are terminated by a 0 byte;
+sieve_chunk(byteptr known_primes, ulong s, byteptr data, ulong n)
+{
+  ulong p, cnt = n-1, start = s, delta = 1;
+  byteptr q;
 
-       Checks cnt odd numbers starting at 'start', setting bytes
-       starting at data to 0 or 1 depending on whether the
-       corresponding odd number is prime or not */
-    ulong p;
-    byteptr q;
-    register byteptr write_to = data;   /* Better code with gcc 2.8.1 */
-    register ulong   cnt      = count;  /* Better code with gcc 2.8.1 */
-    register ulong   start    = s;      /* Better code with gcc 2.8.1 */
-    register ulong   delta    = 1;      /* Better code with gcc 2.8.1 */
-
-    memset(data, 0, cnt);
-    start >>= 1;                        /* (start - 1)/2 */
-    start += cnt;                       /* Corresponds to the end */
-    cnt -= 1;
-    /* data corresponds to start.  q runs over primediffs.  */
-    /* Don't care about DIFFPTR_SKIP: false positives provide no problem */
-    for (q = known_primes + 1, p = 3; delta; delta = *++q, p += delta) {
-        /* first odd number which is >= start > p and divisible by p
-           = last odd number which is <= start + 2p - 1 and 0 (mod p)
-           = p + the last even number which is <= start + p - 1 and 0 (mod p)
-           = p + the last even number which is <= start + p - 2 and 0 (mod p)
-           = p + start + p - 2 - (start + p - 2) % 2p
-           = start + 2(p - 1 - ((start-1)/2 + (p-1)/2) % p). */
-      long off = cnt - ((start+(p>>1)) % p);
-
-      while (off >= 0) {
-          write_to[off] = 1;
-          off -= p;
-      }
-    }
+  memset(data, 0, n);
+  start >>= 1;  /* (start - 1)/2 */
+  start += n; /* Corresponds to the end */
+  /* data corresponds to start, q runs over primediffs */
+  for (q = known_primes + 1, p = 3; delta; delta = *++q, p += delta)
+  { /* first odd number >= start > p and divisible by p
+       = last odd number <= start + 2p - 2 and 0 (mod p)
+       = p + last number <= start + p - 2 and 0 (mod 2p)
+       = p + start+p-2 - (start+p-2) % 2p
+       = start + 2(p - 1 - ((start-1)/2 + (p-1)/2) % p). */
+    long off = cnt - ((start+(p>>1)) % p);
+    while (off >= 0) { data[off] = 1; off -= p; }
+  }
 }
 
-/* Here's the workhorse.  This is recursive, although normally the first
-   recursive call will bottom out and invoke initprimes1() at once.
-   (Not static;  might conceivably be useful to someone in library mode) */
-byteptr
-initprimes0(ulong maxnum, long *lenp, ulong *lastp)
+/* assume maxnum <= 436273290 < 2^29 */
+static void
+initprimes0(ulong maxnum, long *lenp, ulong *lastp, byteptr p1)
 {
-  long size, alloced, psize;
-  byteptr q, fin, p, p1, fin1, plast, curdiff;
+  pari_sp av = avma;
+  long alloced, psize;
+  byteptr q, end, p, end1, plast, curdiff;
   ulong last, remains, curlow, rootnum, asize;
-  ulong prime_above = 3;
+  ulong prime_above;
   byteptr p_prime_above;
 
-  if (maxnum <= 1ul<<17)        /* Arbitrary. */
-    return initprimes1(maxnum>>1, lenp, (long*)lastp); /* Break recursion */
-
-  maxnum |= 1;                  /* make it odd. */
+  maxnum |= 1; /* make it odd. */
+  /* base case */
+  if (maxnum < 1ul<<17) { initprimes1(maxnum>>1, lenp, lastp, p1); return; }
 
   /* Checked to be enough up to 40e6, attained at 155893 */
-  /* Due to multibyte representation of large gaps, this estimate will
-     be broken by large enough maxnum.  However, assuming exponential
-     distribution of gaps with the average log(n), we are safe up to
-     circa exp(-256/log(1/0.09)) = 1.5e46.  OK with LONG_BITS <= 128. ;-) */
-  size = (long) (1.09 * maxnum/log((double)maxnum)) + 146;
-  p1 = (byteptr) pari_malloc(size);
-  rootnum = (ulong) sqrt((double)maxnum); /* cast it back to a long */
-  rootnum |= 1;
-  {
-    byteptr p2 = initprimes0(rootnum, &psize, &last); /* recursive call */
-    memcpy(p1, p2, psize); pari_free(p2);
-  }
-  fin1 = p1 + psize - 1;
-  remains = (maxnum - rootnum) >> 1; /* number of odd numbers to check */
+  rootnum = usqrt(maxnum) | 1;
+  initprimes1(rootnum>>1, &psize, &last, p1);
+  end1 = p1 + psize - 1;
+  remains = (maxnum - last) >> 1; /* number of odd numbers to check */
 
-  /* Actually, we access primes array of psize too; but we access it
-     consequently, thus we do not include it in fixed_to_cache */
-  asize = good_arena_size((ulong)(rootnum * slow2_in_roots), remains + 1, 0,
-                          &cache_model, 0) - 1;
+  /* we access primes array of psize too; but we access it consecutively,
+   * thus we do not include it in fixed_to_cache */
+  asize = good_arena_size((ulong)(rootnum * slow2_in_roots), remains+1, 0,
+                          &cache_model) - 1;
   /* enough room on the stack ? */
   alloced = (((byteptr)avma) <= ((byteptr)bot) + asize);
   if (alloced)
-    p = (byteptr) pari_malloc(asize + 1);
+    p = (byteptr)pari_malloc(asize+1);
   else
-    p = (byteptr) bot;
-  fin = p + asize;              /* the 0 sentinel goes at fin. */
-  curlow = rootnum + 2; /* First candidate: know primes up to rootnum (odd). */
-  curdiff = fin1;
+    p = (byteptr)stack_malloc(asize+1);
+  end = p + asize; /* the 0 sentinel goes at end. */
+  curlow = last + 2; /* First candidate: know primes up to last (odd). */
+  curdiff = end1;
 
-  /* During each iteration p..fin-1 represents a range of odd
+  /* During each iteration p..end-1 represents a range of odd
      numbers.  plast is a pointer which represents the last prime seen,
-     it may point before p..fin-1. */
-  plast = p - ((rootnum - last) >> 1) - 1;
+     it may point before p..end-1. */
+  plast = p - 1;
   p_prime_above = p1 + 2;
-  while (remains)       /* Cycle over arenas.  Performance is not crucial */
-  {
+  prime_above = 3;
+  while (remains)
+  { /* cycle over arenas; performance not crucial */
     unsigned char was_delta;
-
-    if (asize > remains) {
-      asize = remains;
-      fin = p + asize;
-    }
+    if (asize > remains) { asize = remains; end = p + asize; }
     /* Fake the upper limit appropriate for the given arena */
     while (prime_above*prime_above <= curlow + (asize << 1) && *p_prime_above)
-        prime_above += *p_prime_above++;
+      prime_above += *p_prime_above++;
     was_delta = *p_prime_above;
-    *p_prime_above = 0;                 /* Put a 0 sentinel for sieve_chunk */
-
+    *p_prime_above = 0; /* sentinel for sieve_chunk */
     sieve_chunk(p1, curlow, p, asize);
+    *p_prime_above = was_delta; /* restore */
 
-    *p_prime_above = was_delta;         /* Restore */
-    p[asize] = 0;                       /* Put a 0 sentinel for ZZZ */
-    /* now q runs over addresses corresponding to primes */
+    p[asize] = 0; /* sentinel */
     for (q = p; ; plast = q++)
-    {
-      long d;
-
-      while (*q) q++;                   /* use ZZZ 0-sentinel at end */
-      if (q >= fin) break;
-      d = (q - plast) << 1;
-      while (d >= DIFFPTR_SKIP)
-        *curdiff++ = DIFFPTR_SKIP, d -= DIFFPTR_SKIP;
-      *curdiff++ = (unsigned char)d;
+    { /* q runs over addresses corresponding to primes */
+      while (*q) q++; /* use sentinel at end */
+      if (q >= end) break;
+      *curdiff++ = (unsigned char)(q-plast) << 1; /* < 255 for q < 436273291 */
     }
     plast -= asize;
     remains -= asize;
     curlow += (asize<<1);
-  } /* while (remains) */
+  }
   last = curlow - ((p - plast) << 1);
-  *curdiff++ = 0;               /* sentinel */
+  *curdiff++ = 0; /* sentinel */
   *lenp = curdiff - p1;
   *lastp = last;
-  if (alloced) pari_free(p);
-  return (byteptr) pari_realloc(p1, *lenp);
+  if (alloced) pari_free(p); else avma = av;
 }
-#if 0 /* not yet... GN */
-/* The diffptr table will contain at least 6548 entries (up to and including
-   the 6547th prime, 65557, and the terminal null byte), because the isprime/
-   small-factor-extraction machinery wants to depend on everything up to 65539
-   being in the table, and we might as well go to a multiple of 4 Bytes.--GN */
-
-void
-init_tinyprimes_tridiv(byteptr p);      /* in ifactor2.c */
-#endif
-
-static ulong _maxprime = 0;
 
 ulong
-maxprime(void) { return _maxprime; }
+maxprime(void) { return diffptr ? _maxprime : 0; }
 
 void
-maxprime_check(ulong c)
-{
-  if (_maxprime < c) pari_err(primer1, c);
-}
-
-/* assume ptr is the address of a diffptr containing the succesive
- * differences between primes, and p = current prime (up to *ptr excluded)
- * return smallest prime >= a, update ptr */
-ulong
-init_primepointer(ulong a, ulong p, byteptr *ptr)
-{
-  byteptr diff = *ptr;
-  if (a <= 0) a = 2;
-  maxprime_check((ulong)a);
-  while (a > p) NEXT_PRIME_VIADIFF(p,diff);
-  *ptr = diff; return p;
-}
+maxprime_check(ulong c) { if (_maxprime < c) pari_err_MAXPRIME(c); }
 
+/* We ensure 65302 <= maxnum <= 436273290: the LHS ensures modular function
+ * have enough fast primes to work, the RHS ensures that p_{n+1} - p_n < 255 */
 byteptr
-initprimes(ulong maxnum)
+initprimes(ulong maxnum, long *lenp, ulong *lastp)
+{
+  byteptr t;
+  if (maxnum < 65537)
+    maxnum = 65537;
+  else if (maxnum > 436273290)
+    maxnum = 436273290;
+  t = (byteptr)pari_malloc((size_t) (1.09 * maxnum/log((double)maxnum)) + 146);
+  initprimes0(maxnum, lenp, lastp, t);
+  return (byteptr)pari_realloc(t, *lenp);
+}
+
+void
+initprimetable(ulong maxnum)
 {
   long len;
   ulong last;
-  byteptr p;
-  /* The algorithm must see the next prime beyond maxnum, whence the +512. */
-  ulong maxnum1 = ((maxnum<65302?65302:maxnum)+512ul);
+  byteptr p = initprimes(maxnum, &len, &last), old = diffptr;
+  diffptrlen = minss(diffptrlen, len);
+  _maxprime  = minss(_maxprime,last); /*Protect against ^C*/
+  diffptr = p; diffptrlen = len; _maxprime = last;
+  if (old) free(old);
+}
 
-  if ((maxnum>>1) > LONG_MAX - 1024)
-      pari_err(talker, "Too large primelimit");
-  p = initprimes0(maxnum1, &len, &last);
-#if 0 /* not yet... GN */
-  static int build_the_tables = 1;
-  if (build_the_tables) { init_tinyprimes_tridiv(p); build_the_tables=0; }
-#endif
-  _maxprime = last; return p;
+/* all init_primepointer_xx routines set *ptr to the corresponding place
+ * in prime table */
+/* smallest p >= a */
+ulong
+init_primepointer_geq(ulong a, byteptr *pd)
+{
+  ulong n, p;
+  prime_table_next_p(a, pd, &p, &n);
+  return p;
+}
+/* largest p < a */
+ulong
+init_primepointer_lt(ulong a, byteptr *pd)
+{
+  ulong n, p;
+  prime_table_next_p(a, pd, &p, &n);
+  PREC_PRIME_VIADIFF(p, *pd);
+  return p;
+}
+/* largest p <= a */
+ulong
+init_primepointer_leq(ulong a, byteptr *pd)
+{
+  ulong n, p;
+  prime_table_next_p(a, pd, &p, &n);
+  if (p != a) PREC_PRIME_VIADIFF(p, *pd);
+  return p;
+}
+/* smallest p > a */
+ulong
+init_primepointer_gt(ulong a, byteptr *pd)
+{
+  ulong n, p;
+  prime_table_next_p(a, pd, &p, &n);
+  if (p == a) NEXT_PRIME_VIADIFF(p, *pd);
+  return p;
 }
 
 GEN
@@ -538,7 +514,7 @@ boundfact(GEN n, ulong lim)
       return gerepilecopy(av, merge_factor_i(a,b));
     }
   }
-  pari_err(arither1);
+  pari_err_TYPE("boundfact",n);
   return NULL; /* not reached */
 }
 
@@ -562,31 +538,6 @@ Z_smoothen(GEN N, GEN L, GEN *pP, GEN *pe)
 /**                    SIMPLE FACTORISATIONS                          **/
 /**                                                                   **/
 /***********************************************************************/
-
-/* Factor n and output [p,e] where
- * p, e are vecsmall with n = prod{p[i]^e[i]} */
-GEN
-factoru(ulong n)
-{
-  GEN f = cgetg(3,t_VEC);
-  pari_sp av = avma;
-  GEN F, P, E, p, e;
-  long i, l;
-  /* enough room to store <= 15 primes and exponents (OK if n < 2^64) */
-  (void)new_chunk((15 + 1)*2);
-  F = Z_factor(utoi(n));
-  P = gel(F,1);
-  E = gel(F,2); l = lg(P);
-  avma = av;
-  p = cgetg(l,t_VECSMALL); gel(f,1) = p;
-  e = cgetg(l,t_VECSMALL); gel(f,2) = e;
-  for (i = 1; i < l; i++)
-  {
-    p[i] = itou(gel(P,i));
-    e[i] = itou(gel(E,i));
-  }
-  return f;
-}
 /* Factor n and output [p,e,c] where
  * p, e and c are vecsmall with n = prod{p[i]^e[i]} and c[i] = p[i]^e[i] */
 GEN
@@ -598,7 +549,7 @@ factoru_pow(ulong n)
   long i, l;
   /* enough room to store <= 15 * [p,e,p^e] (OK if n < 2^64) */
   (void)new_chunk((15 + 1)*3);
-  F = Z_factor(utoi(n));
+  F = factoru(n);
   P = gel(F,1);
   E = gel(F,2); l = lg(P);
   avma = av;
@@ -607,19 +558,23 @@ factoru_pow(ulong n)
   gel(f,3) = c = cgetg(l,t_VECSMALL);
   for(i = 1; i < l; i++)
   {
-    p[i] = itou(gel(P,i));
-    e[i] = itou(gel(E,i));
+    p[i] = P[i];
+    e[i] = E[i];
     c[i] = upowuu(p[i], e[i]);
   }
   return f;
 }
 
-/* factor p^n - 1, assuming p prime */
+static GEN
+factorlim(GEN n, ulong lim)
+{ return lim? Z_factor_limit(n, lim): Z_factor(n); }
+/* factor p^n - 1, assuming p prime. If lim != 0, limit factorization to
+ * primes <= lim */
 GEN
-factor_pn_1(GEN p, long n)
+factor_pn_1_limit(GEN p, long n, ulong lim)
 {
   pari_sp av = avma;
-  GEN A = Z_factor(subis(p,1)), d = divisorsu(n);
+  GEN A = factorlim(subiu(p,1), lim), d = divisorsu(n);
   long i, pp = itos_or_0(p);
   for(i=2; i<lg(d); i++)
   {
@@ -630,16 +585,19 @@ factor_pn_1(GEN p, long n)
        (pp==2 && (d[i]&7)==4)))
     {
       GEN f=factor_Aurifeuille_prime(p,d[i]);
-      B = Z_factor(f);
+      B = factorlim(f, lim);
       A = merge_factor(A, B, (void*)&cmpii, cmp_nodata);
-      B = Z_factor(diviiexact(polcyclo_eval(d[i],p), f));
+      B = factorlim(diviiexact(polcyclo_eval(d[i],p), f), lim);
     }
     else
-      B = Z_factor(polcyclo_eval(d[i],p));
+      B = factorlim(polcyclo_eval(d[i],p), lim);
     A = merge_factor(A, B, (void*)&cmpii, cmp_nodata);
   }
   return gerepilecopy(av, A);
 }
+GEN
+factor_pn_1(GEN p, ulong n)
+{ return factor_pn_1_limit(p, n, 0); }
 
 #if 0
 static GEN
@@ -681,7 +639,7 @@ factor_eulerphi(GEN n)
     }
   }
   avma = av;
-  if (!B) return v? to_mat(gen_2, v): trivfact();
+  if (!B) return v? to_mat(gen_2, v): trivial_fact();
   A = cgetg(3, t_MAT);
   P = gel(B,1); E = gel(B,2); l = lg(P);
   AP = cgetg(l+1, t_COL); gel(A,1) = AP; AP++;
@@ -698,6 +656,127 @@ factor_eulerphi(GEN n)
 }
 #endif
 
+/***********************************************************************/
+/**                                                                   **/
+/**         CHECK FACTORIZATION FOR ARITHMETIC FUNCTIONS              **/
+/**                                                                   **/
+/***********************************************************************/
+static int
+RgV_is_ZVpos(GEN v)
+{
+  long i, l = lg(v);
+  for (i = 1; i < l; i++)
+  {
+    GEN c = gel(v,i);
+    if (typ(c) != t_INT || signe(c) <= 0) return 0;
+  }
+  return 1;
+}
+/* check whether v is a ZV with non-0 entries */
+static int
+RgV_is_ZVnon0(GEN v)
+{
+  long i, l = lg(v);
+  for (i = 1; i < l; i++)
+  {
+    GEN c = gel(v,i);
+    if (typ(c) != t_INT || !signe(c)) return 0;
+  }
+  return 1;
+}
+/* check whether v is a ZV with non-zero entries OR exactly [0] */
+static int
+RgV_is_ZV0(GEN v)
+{
+  long i, l = lg(v);
+  for (i = 1; i < l; i++)
+  {
+    GEN c = gel(v,i);
+    long s;
+    if (typ(c) != t_INT) return 0;
+    s = signe(c);
+    if (!s) return (l == 2);
+  }
+  return 1;
+}
+
+static int
+is_Z_factor_i(GEN f)
+{ return typ(f) == t_MAT && lg(f) == 3 && RgV_is_ZVpos(gel(f,2)); }
+int
+is_Z_factorpos(GEN f)
+{ return is_Z_factor_i(f) && RgV_is_ZVpos(gel(f,1)); }
+int
+is_Z_factor(GEN f)
+{ return is_Z_factor_i(f) && RgV_is_ZV0(gel(f,1)); }
+/* as is_Z_factorpos, also allow factor(0) */
+int
+is_Z_factornon0(GEN f)
+{ return is_Z_factor_i(f) && RgV_is_ZVnon0(gel(f,1)); }
+GEN
+clean_Z_factor(GEN f)
+{
+  GEN P = gel(f,1);
+  long n = lg(P)-1;
+  if (n && equalim1(gel(P,1)))
+    return mkmat2(vecslice(P,2,n), vecslice(gel(f,2),2,n));
+  return f;
+}
+
+/* n associated to a factorization of a positive integer: either N (t_INT)
+ * a factorization matrix faN, or a t_VEC: [N, faN] */
+GEN
+check_arith_pos(GEN n, const char *f) {
+  switch(typ(n))
+  {
+    case t_INT:
+      if (signe(n) <= 0 ) pari_err_DOMAIN(f, "argument", "<=", gen_0, gen_0);
+      return NULL;
+    case t_VEC:
+      if (lg(n) != 3 || typ(gel(n,1)) != t_INT) break;
+      n = gel(n,2); /* fall through */
+    case t_MAT:
+      if (!is_Z_factorpos(n)) break;
+      return n;
+  }
+  pari_err_TYPE(f,n);
+  return NULL;
+}
+/* n associated to a factorization of a non-0 integer */
+GEN
+check_arith_non0(GEN n, const char *f) {
+  switch(typ(n))
+  {
+    case t_INT:
+      if (!signe(n)) pari_err_DOMAIN(f, "argument", "=", gen_0, gen_0);
+      return NULL;
+    case t_VEC:
+      if (lg(n) != 3 || typ(gel(n,1)) != t_INT) break;
+      n = gel(n,2); /* fall through */
+    case t_MAT:
+      if (!is_Z_factornon0(n)) break;
+      return n;
+  }
+  pari_err_TYPE(f,n);
+  return NULL;
+}
+/* n associated to a factorization of an integer */
+GEN
+check_arith_all(GEN n, const char *f) {
+  switch(typ(n))
+  {
+    case t_INT:
+      return NULL;
+    case t_VEC:
+      if (lg(n) != 3 || typ(gel(n,1)) != t_INT) break;
+      n = gel(n,2); /* fall through */
+    case t_MAT:
+      if (!is_Z_factor(n)) break;
+      return n;
+  }
+  pari_err_TYPE(f,n);
+  return NULL;
+}
 
 /***********************************************************************/
 /**                                                                   **/
@@ -705,56 +784,90 @@ factor_eulerphi(GEN n)
 /**                (ultimately depend on Z_factor())                  **/
 /**                                                                   **/
 /***********************************************************************/
+/* set P,E from F. Check whether F is an integer and kill "factor" -1 */
+static void
+set_fact_check(GEN F, GEN *pP, GEN *pE, int *isint)
+{
+  GEN E, P;
+  if (lg(F) != 3) pari_err_TYPE("divisors",F);
+  P = gel(F,1);
+  E = gel(F,2);
+  RgV_check_ZV(E, "divisors");
+  *isint = RgV_is_ZV(P);
+  if (*isint)
+  {
+    long i, l = lg(P);
+    /* skip -1 */
+    if (l>1 && signe(gel(P,1)) < 0) { E++; P = vecslice(P,2,--l); }
+    /* test for 0 */
+    for (i = 1; i < l; i++)
+      if (!signe(gel(P,i)) && signe(gel(E,i)))
+        pari_err_DOMAIN("divisors", "argument", "=", gen_0, F);
+  }
+  *pP = P;
+  *pE = E;
+}
+static void
+set_fact(GEN F, GEN *pP, GEN *pE) { *pP = gel(F,1); *pE = gel(F,2); }
+
+int
+divisors_init(GEN n, GEN *pP, GEN *pE)
+{
+  long i,l;
+  GEN E, P, e;
+  int isint;
+
+  switch(typ(n))
+  {
+    case t_INT:
+      if (!signe(n)) pari_err_DOMAIN("divisors", "argument", "=", gen_0, gen_0);
+      set_fact(absi_factor(n), &P,&E);
+      isint = 1; break;
+    case t_VEC:
+      if (lg(n) != 3) pari_err_TYPE("divisors",n);
+      set_fact_check(gel(n,2), &P,&E, &isint);
+      break;
+    case t_MAT:
+      set_fact_check(n, &P,&E, &isint);
+      break;
+    default:
+      set_fact(factor(n), &P,&E);
+      isint = 0; break;
+  }
+  l = lg(P);
+  e = cgetg(l, t_VECSMALL);
+  for (i=1; i<l; i++)
+  {
+    e[i] = itos(gel(E,i));
+    if (e[i] < 0) pari_err_TYPE("divisors [denominator]",n);
+  }
+  *pP = P; *pE = e; return isint;
+}
 
 GEN
 divisors(GEN n)
 {
   pari_sp av = avma;
-  long i, j, l, tn = typ(n);
-  ulong nbdiv;
-  int isint = 1;
+  long i, j, l;
+  ulong ndiv;
   GEN *d, *t, *t1, *t2, *t3, P, E, e;
+  int isint = divisors_init(n, &P, &E);
 
-  if (tn == t_MAT && lg(n) == 3)
-  {
-    P = gel(n,1); l = lg(P);
-    for (i = 1; i < l; i++)
-      if (typ(gel(P,i)) != t_INT) { isint = 0; break; }
-  }
-  else
-  {
-    if (tn == t_INT)
-      n = Z_factor(n);
-    else {
-      if (is_matvec_t(tn)) pari_err(typeer,"divisors");
-      isint = 0;
-      n = factor(n);
-    }
-    P = gel(n,1); l = lg(P);
-  }
-  E = gel(n,2);
-  if (isint && l>1 && signe(P[1]) < 0) { E++; P++; l--; } /* skip -1 */
-  e = cgetg(l, t_VECSMALL);
-  nbdiv = 1;
-  for (i=1; i<l; i++)
-  {
-    e[i] = itos(gel(E,i));
-    if (e[i] < 0) pari_err(talker, "denominators not allowed in divisors");
-    nbdiv = itou_or_0( muluu(nbdiv, 1+e[i]) );
-  }
-  if (!nbdiv || nbdiv & ~LGBITS)
-    pari_err(talker, "too many divisors (more than %ld)", LGBITS - 1);
-  d = t = (GEN*) cgetg(nbdiv+1,t_VEC);
+  l = lg(E); e = cgetg(l, t_VECSMALL);
+  for (i=1; i<l; i++) e[i] = E[i]+1;
+  ndiv = itou_or_0( zv_prod_Z(e) );
+  if (!ndiv || ndiv & ~LGBITS) pari_err_OVERFLOW("divisors");
+  d = t = (GEN*) cgetg(ndiv+1,t_VEC);
   *++d = gen_1;
   if (isint)
   {
     for (i=1; i<l; i++)
-      for (t1=t,j=e[i]; j; j--,t1=t2)
+      for (t1=t,j=E[i]; j; j--,t1=t2)
         for (t2=d, t3=t1; t3<t2; ) *++d = mulii(*++t3, gel(P,i));
-    e = sort((GEN)t);
+    e = ZV_sort((GEN)t);
   } else {
     for (i=1; i<l; i++)
-      for (t1=t,j=e[i]; j; j--,t1=t2)
+      for (t1=t,j=E[i]; j; j--,t1=t2)
         for (t2=d, t3=t1; t3<t2; ) *++d = gmul(*++t3, gel(P,i));
     e = (GEN)t;
   }
@@ -808,30 +921,30 @@ GEN
 corepartial(GEN n, long all)
 {
   pari_sp av = avma;
-  if (typ(n) != t_INT) pari_err(typeer,"corepartial");
+  if (typ(n) != t_INT) pari_err_TYPE("corepartial",n);
   return gerepileuptoint(av, corefa(Z_factor_limit(n,all)));
 }
 GEN
 core2partial(GEN n, long all)
 {
   pari_sp av = avma;
-  if (typ(n) != t_INT) pari_err(typeer,"core2partial");
+  if (typ(n) != t_INT) pari_err_TYPE("core2partial",n);
   return gerepilecopy(av, core2fa(Z_factor_limit(n,all)));
 }
-GEN
-core(GEN n)
+static GEN
+core2_i(GEN n)
 {
-  pari_sp av = avma;
-  if (typ(n) != t_INT) pari_err(typeer,"core");
-  return gerepileuptoint(av, corefa(Z_factor(n)));
+  GEN f = core(n);
+  if (!signe(f)) return mkvec2(gen_0, gen_1);
+  switch(typ(n))
+  {
+    case t_VEC: n = gel(n,1); break;
+    case t_MAT: n = factorback(n); break;
+  }
+  return mkvec2(f, sqrtint(diviiexact(n, f)));
 }
 GEN
-core2(GEN n)
-{
-  pari_sp av = avma;
-  if (typ(n) != t_INT) pari_err(typeer,"core");
-  return gerepilecopy(av, core2fa(Z_factor(n)));
-}
+core2(GEN n) { pari_sp av = avma; return gerepilecopy(av, core2_i(n)); }
 
 GEN
 core0(GEN n,long flag) { return flag? core2(n): core(n); }
@@ -857,9 +970,9 @@ GEN
 coredisc2(GEN n)
 {
   pari_sp av = avma;
-  GEN y = core2(n);
+  GEN y = core2_i(n);
   GEN c = gel(y,1), f = gel(y,2);
-  if (_mod4(c)<=1) return y;
+  if (_mod4(c)<=1) return gerepilecopy(av, y);
   y = cgetg(3,t_VEC);
   gel(y,1) = shifti(c,2);
   gel(y,2) = gmul2n(f,-1); return gerepileupto(av, y);
@@ -868,3 +981,458 @@ coredisc2(GEN n)
 GEN
 coredisc0(GEN n,long flag) { return flag? coredisc2(n): coredisc(n); }
 
+long
+omega(GEN n)
+{
+  pari_sp av = avma;
+  GEN F, P;
+  if ((F = check_arith_non0(n,"omega"))) {
+    long n;
+    P = gel(F,1); n = lg(P)-1;
+    if (n && equalim1(gel(P,1))) n--;
+    return n;
+  }
+  else if (lgefint(n) == 3)
+  {
+    if (n[2] == 1) return 0;
+    F = factoru(n[2]);
+  }
+  else
+    F = absi_factor(n);
+  P = gel(F,1); avma = av; return lg(P)-1;
+}
+
+long
+bigomega(GEN n)
+{
+  pari_sp av = avma;
+  GEN F, E;
+  if ((F = check_arith_non0(n,"bigomega")))
+  {
+    GEN P = gel(F,1);
+    long n = lg(P)-1;
+    E = gel(F,2);
+    if (n && equalim1(gel(P,1))) E = vecslice(E,2,n);
+    E = ZV_to_zv(E);
+  }
+  else if (lgefint(n) == 3)
+  {
+    if (n[2] == 1) return 0;
+    F = factoru(n[2]);
+    E = gel(F,2);
+  }
+  else
+    E = ZV_to_zv(gel(absi_factor(n), 2));
+  avma = av; return zv_sum(E);
+}
+
+/* assume f = factoru(n), possibly with 0 exponents. Return phi(n) */
+ulong
+eulerphiu_fact(GEN f)
+{
+  GEN P = gel(f,1), E = gel(f,2);
+  long i, m = 1, l = lg(P);
+  for (i = 1; i < l; i++)
+  {
+    ulong p = P[i], e = E[i];
+    if (!e) continue;
+    if (p == 2)
+    { if (e > 1) m <<= e-1; }
+    else
+    {
+      m *= (p-1);
+      if (e > 1) m *= upowuu(p, e-1);
+    }
+  }
+  return m;
+}
+ulong
+eulerphiu(ulong n)
+{
+  pari_sp av = avma;
+  GEN F;
+  if (!n) return 2;
+  F = factoru(n);
+  avma = av; return eulerphiu_fact(F);
+}
+GEN
+eulerphi(GEN n)
+{
+  pari_sp av = avma;
+  GEN Q, F, P, E;
+  long i, l;
+
+  if ((F = check_arith_all(n,"eulerphi")))
+  {
+    F = clean_Z_factor(F);
+    n = (typ(n) == t_VEC)? gel(n,1): factorback(F);
+    if (lgefint(n) == 3)
+    {
+      ulong e;
+      F = mkmat2(ZV_to_nv(gel(F,1)), ZV_to_nv(gel(F,2)));
+      e = eulerphiu_fact(F);
+      avma = av; return utoipos(e);
+    }
+  }
+  else if (lgefint(n) == 3) return utoipos(eulerphiu((ulong)n[2]));
+  else
+    F = absi_factor(n);
+  if (!signe(n)) return gen_2;
+  P = gel(F,1);
+  E = gel(F,2); l = lg(P);
+  Q = cgetg(l, t_VEC);
+  for (i = 1; i < l; i++)
+  {
+    GEN p = gel(P,i), q;
+    ulong v = itou(gel(E,i));
+    q = subiu(p,1);
+    if (v != 1) q = mulii(q, v == 2? p: powiu(p, v-1));
+    gel(Q,i) = q;
+  }
+  return gerepileuptoint(av, ZV_prod(Q));
+}
+
+static GEN
+numdiv_aux(GEN F)
+{
+  GEN x, E = gel(F,2);
+  long i, l = lg(E);
+  x = cgetg(l, t_VECSMALL);
+  for (i=1; i<l; i++) x[i] = itou(gel(E,i))+1;
+  return x;
+}
+GEN
+numdiv(GEN n)
+{
+  pari_sp av = avma;
+  GEN F, E;
+  long i, l;
+  if ((F = check_arith_non0(n,"numdiv")))
+  {
+    F = clean_Z_factor(F);
+    E = numdiv_aux(F);
+  }
+  else if (lgefint(n) == 3)
+  {
+    if (n[2] == 1) return gen_1;
+    F = factoru(n[2]);
+    E = gel(F,2); l = lg(E);
+    for (i=1; i<l; i++) E[i]++;
+  }
+  else
+    E = numdiv_aux(absi_factor(n));
+  return gerepileuptoint(av, zv_prod_Z(E));
+}
+
+/* 1 + p + ... + p^v, p != 2^BIL - 1 */
+static GEN
+u_euler_sumdiv(ulong p, long v)
+{
+  GEN u = utoipos(1 + p); /* can't overflow */
+  for (; v > 1; v--) u = addsi(1, mului(p, u));
+  return u;
+}
+/* 1 + q + ... + q^v */
+static GEN
+euler_sumdiv(GEN q, long v)
+{
+  GEN u = addui(1, q);
+  for (; v > 1; v--) u = addui(1, mulii(q, u));
+  return u;
+}
+static GEN
+u_euler_sumdivk(ulong p, long v, long k) { return euler_sumdiv(powuu(p,k), v); }
+static GEN
+euler_sumdivk(GEN p, long v, long k) { return euler_sumdiv(powiu(p,k), v); }
+
+static GEN
+sumdiv_aux(GEN F)
+{
+  GEN x, P = gel(F,1), E = gel(F,2);
+  long i, l = lg(P);
+  x = cgetg(l, t_VEC);
+  for (i=1; i<l; i++) gel(x,i) = euler_sumdiv(gel(P,i), itou(gel(E,i)));
+  return x;
+}
+GEN
+sumdiv(GEN n)
+{
+  pari_sp av = avma;
+  GEN F, P, E;
+  long i, l;
+
+  if ((F = check_arith_non0(n,"sumdiv")))
+  {
+    F = clean_Z_factor(F);
+    P = sumdiv_aux(F);
+  }
+  else if (lgefint(n) == 3)
+  {
+    if (n[2] == 1) return gen_1;
+    F = factoru(n[2]);
+    P = gel(F,1);
+    E = gel(F,2); l = lg(P);
+    for (i=1; i<l; i++) gel(P,i) = u_euler_sumdiv(P[i], E[i]);
+  }
+  else
+    P = sumdiv_aux(absi_factor(n));
+  return gerepileuptoint(av, ZV_prod(P));
+}
+
+static GEN
+sumdivk_aux(GEN F, long k)
+{
+  GEN x, P = gel(F,1), E = gel(F,2);
+  long i, l = lg(P);
+  x = cgetg(l, t_VEC);
+  for (i=1; i<l; i++) gel(x,i) = euler_sumdivk(gel(P,i), gel(E,i)[2], k);
+  return x;
+}
+GEN
+sumdivk(GEN n, long k)
+{
+  pari_sp av = avma;
+  GEN E, F, P;
+  long i, l, k1;
+
+  if (!k) return numdiv(n);
+  if (k == 1) return sumdiv(n);
+  if (k ==-1) return gerepileupto(av, gdiv(sumdiv(n), n));
+  k1 = k;
+  if (k < 0)  k = -k;
+  if ((F = check_arith_non0(n,"sumdivk")))
+  {
+    F = clean_Z_factor(F);
+    P = sumdivk_aux(F,k);
+  }
+  else if (lgefint(n) == 3)
+  {
+    if (n[2] == 1) return gen_1;
+    F = factoru(n[2]);
+    P = gel(F,1);
+    E = gel(F,2); l = lg(P);
+    for (i=1; i<l; i++) gel(P,i) = u_euler_sumdivk(P[i], E[i], k);
+  }
+  else
+    P = sumdivk_aux(absi_factor(n), k);
+  P = ZV_prod(P);
+  if (k1 > 0) return gerepileuptoint(av, P);
+  return gerepileupto(av, gdiv(P, powiu(n,k)));
+}
+
+/* K t_VECSMALL of k >= 0 */
+GEN
+usumdivkvec(ulong n, GEN K)
+{
+  pari_sp av = avma;
+  GEN F = factoru(n), P = gel(F,1), E = gel(F,2), Z, S;
+  long i,j, l = lg(P), lK = lg(K);
+  Z = cgetg(l, t_VEC);
+  S = cgetg(lK, t_VEC);
+  for (j=1; j<lK; j++)
+  {
+    long k = K[j];
+    for (i=1; i<l; i++) gel(Z,i) = u_euler_sumdivk(P[i], E[i], k);
+    gel(S,j) = ZV_prod(Z);
+  }
+  return gerepilecopy(av, S);
+}
+
+long
+uissquarefree_fact(GEN f)
+{
+  GEN E = gel(f,2);
+  long i, l = lg(E);
+  for (i = 1; i < l; i++)
+    if (E[i] > 1) return 0;
+  return 1;
+}
+long
+uissquarefree(ulong n)
+{
+  if (!n) return 0;
+  return moebiusu(n)? 1: 0;
+}
+long
+Z_issquarefree(GEN n)
+{
+  switch(lgefint(n))
+  {
+    case 2: return 0;
+    case 3: return uissquarefree(n[2]);
+  }
+  return moebius(n)? 1: 0;
+}
+long
+issquarefree(GEN x)
+{
+  pari_sp av;
+  GEN d;
+  switch(typ(x))
+  {
+    case t_INT: return Z_issquarefree(x);
+    case t_POL:
+      if (!signe(x)) return 0;
+      av = avma; d = RgX_gcd(x, RgX_deriv(x));
+      avma = av; return (lg(d) == 3);
+    default: pari_err_TYPE("issquarefree",x);
+      return 0; /* not reached */
+  }
+}
+
+/*********************************************************************/
+/**                                                                 **/
+/**                    DIGITS / SUM OF DIGITS                       **/
+/**                                                                 **/
+/*********************************************************************/
+
+static ulong DS[] ={
+  0,1,2,3,4,5,6,7,8,9,1,2,3,4,5,6,7,8,9,10,2,3,4,5,6,7,8,9,10,11,3,4,5,6,7,8,
+  9,10,11,12,4,5,6,7,8,9,10,11,12,13,5,6,7,8,9,10,11,12,13,14,6,7,8,9,10,11,
+  12,13,14,15,7,8,9,10,11,12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,9,10,11,
+  12,13,14,15,16,17,18,1,2,3,4,5,6,7,8,9,10,2,3,4,5,6,7,8,9,10,11,3,4,5,6,7,8,
+  9,10,11,12,4,5,6,7,8,9,10,11,12,13,5,6,7,8,9,10,11,12,13,14,6,7,8,9,10,11,
+  12,13,14,15,7,8,9,10,11,12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,9,10,11,
+  12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,19,2,3,4,5,6,7,8,9,10,11,3,
+  4,5,6,7,8,9,10,11,12,4,5,6,7,8,9,10,11,12,13,5,6,7,8,9,10,11,12,13,14,6,7,8,
+  9,10,11,12,13,14,15,7,8,9,10,11,12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,
+  9,10,11,12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,19,11,12,13,14,15,
+  16,17,18,19,20,3,4,5,6,7,8,9,10,11,12,4,5,6,7,8,9,10,11,12,13,5,6,7,8,9,10,
+  11,12,13,14,6,7,8,9,10,11,12,13,14,15,7,8,9,10,11,12,13,14,15,16,8,9,10,11,
+  12,13,14,15,16,17,9,10,11,12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,
+  19,11,12,13,14,15,16,17,18,19,20,12,13,14,15,16,17,18,19,20,21,4,5,6,7,8,9,
+  10,11,12,13,5,6,7,8,9,10,11,12,13,14,6,7,8,9,10,11,12,13,14,15,7,8,9,10,11,
+  12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,9,10,11,12,13,14,15,16,17,18,10,
+  11,12,13,14,15,16,17,18,19,11,12,13,14,15,16,17,18,19,20,12,13,14,15,16,17,
+  18,19,20,21,13,14,15,16,17,18,19,20,21,22,5,6,7,8,9,10,11,12,13,14,6,7,8,9,
+  10,11,12,13,14,15,7,8,9,10,11,12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,9,
+  10,11,12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,19,11,12,13,14,15,16,
+  17,18,19,20,12,13,14,15,16,17,18,19,20,21,13,14,15,16,17,18,19,20,21,22,14,
+  15,16,17,18,19,20,21,22,23,6,7,8,9,10,11,12,13,14,15,7,8,9,10,11,12,13,14,
+  15,16,8,9,10,11,12,13,14,15,16,17,9,10,11,12,13,14,15,16,17,18,10,11,12,13,
+  14,15,16,17,18,19,11,12,13,14,15,16,17,18,19,20,12,13,14,15,16,17,18,19,20,
+  21,13,14,15,16,17,18,19,20,21,22,14,15,16,17,18,19,20,21,22,23,15,16,17,18,
+  19,20,21,22,23,24,7,8,9,10,11,12,13,14,15,16,8,9,10,11,12,13,14,15,16,17,9,
+  10,11,12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,19,11,12,13,14,15,16,
+  17,18,19,20,12,13,14,15,16,17,18,19,20,21,13,14,15,16,17,18,19,20,21,22,14,
+  15,16,17,18,19,20,21,22,23,15,16,17,18,19,20,21,22,23,24,16,17,18,19,20,21,
+  22,23,24,25,8,9,10,11,12,13,14,15,16,17,9,10,11,12,13,14,15,16,17,18,10,11,
+  12,13,14,15,16,17,18,19,11,12,13,14,15,16,17,18,19,20,12,13,14,15,16,17,18,
+  19,20,21,13,14,15,16,17,18,19,20,21,22,14,15,16,17,18,19,20,21,22,23,15,16,
+  17,18,19,20,21,22,23,24,16,17,18,19,20,21,22,23,24,25,17,18,19,20,21,22,23,
+  24,25,26,9,10,11,12,13,14,15,16,17,18,10,11,12,13,14,15,16,17,18,19,11,12,
+  13,14,15,16,17,18,19,20,12,13,14,15,16,17,18,19,20,21,13,14,15,16,17,18,19,
+  20,21,22,14,15,16,17,18,19,20,21,22,23,15,16,17,18,19,20,21,22,23,24,16,17,
+  18,19,20,21,22,23,24,25,17,18,19,20,21,22,23,24,25,26,18,19,20,21,22,23,24,
+  25,26,27
+};
+
+ulong
+sumdigitsu(ulong n)
+{
+  ulong s = 0;
+  while (n) { s += DS[n % 1000]; n /= 1000; }
+  return s;
+}
+
+/* res=array of 9-digits integers, return \sum_{0 <= i < l} sumdigits(res[i]) */
+static ulong
+sumdigits_block(ulong *res, long l)
+{
+  long s = sumdigitsu(*--res);
+  while (--l > 0) s += sumdigitsu(*--res);
+  return s;
+}
+
+GEN
+sumdigits(GEN n)
+{
+  pari_sp av = avma;
+  ulong s, *res;
+  long l;
+
+  if (typ(n) != t_INT) pari_err_TYPE("sumdigits", n);
+  l = lgefint(n);
+  switch(l)
+  {
+    case 2: return gen_0;
+    case 3: return utoipos(sumdigitsu(n[2]));
+  }
+  res = convi(n, &l);
+  if ((ulong)l < ULONG_MAX / 81)
+  {
+    s = sumdigits_block(res, l);
+    avma = av; return utoipos(s);
+  }
+  else /* Huge. Overflows ulong */
+  {
+    const long L = (long)(ULONG_MAX / 81);
+    GEN S = gen_0;
+    while (l > L)
+    {
+      S = addiu(S, sumdigits_block(res, L));
+      res += L; l -= L;
+    }
+    if (l)
+      S = addiu(S, sumdigits_block(res, l));
+    return gerepileuptoint(av, S);
+  }
+}
+
+static void
+digits_dac(GEN x, GEN B, long l, GEN* z)
+{
+  GEN q,r;
+  long m;
+  if (l==1) { *z=x; return; }
+  m=l>>1;
+  q=dvmdii(x,powiu(B,m),&r);
+  digits_dac(q,B,l-m,z);
+  digits_dac(r,B,m,z+l-m);
+}
+
+static void
+digits_dacsmall(GEN x, ulong B, long l, ulong* z)
+{
+  pari_sp av = avma;
+  GEN q,r;
+  long m;
+  if (l==1) { *z=itou(x); return; }
+  m=l>>1;
+  q=dvmdii(x,powuu(B,m),&r);
+  digits_dacsmall(q,B,l-m,z);
+  digits_dacsmall(r,B,m,z+l-m);
+  avma = av;
+}
+
+GEN
+digits(GEN x, GEN B)
+{
+  pari_sp av=avma;
+  long lz;
+  GEN z;
+  if (typ(x)!=t_INT) pari_err_TYPE("digits",x);
+  if (!B)
+    B = utoi(10);
+  else {
+    if (typ(B)!=t_INT) pari_err_TYPE("digits",B);
+    if (cmpis(B,2)<0) pari_err_DOMAIN("digits","B","<",gen_2,B);
+  }
+  if (equalis(B,2))    {avma = av; return binaire(x); }
+  if (!signe(x))       {avma = av; return cgetg(1,t_VEC); }
+  if (absi_cmp(x,B)<0) {avma = av; retmkvec(absi(x)); }
+  x = absi(x); lz = logint(x,B,NULL);
+  if (lgefint(B)>3)
+  {
+    z = zerovec(lz);
+    digits_dac(x,B,lz,(GEN*)(z+1));
+    return gerepilecopy(av,z);
+  }
+  else
+  {
+    ulong b = B[2];
+    z = zero_zv(lz);
+    digits_dacsmall(x,b,lz,(ulong*)(z+1));
+    return gerepileupto(av,vecsmall_to_vec(z));
+  }
+}
