@@ -1,7 +1,5 @@
 #line 2 "../src/kernel/none/mp.c"
-/* $Id$
-
-Copyright (C) 2000-2003 The PARI group.
+/* Copyright (C) 2000-2003 The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -23,7 +21,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "paripriv.h"
 #include "../src/kernel/none/tune-gen.h"
 
-int pari_kernel_init(void) { return 0; } /*nothing to do*/
+void pari_kernel_init(void) { }
+void pari_kernel_close(void) { }
 
 /* NOTE: arguments of "spec" routines (muliispec, addiispec, etc.) aren't
  * GENs but pairs (long *a, long na) representing a list of digits (in basis
@@ -93,7 +92,7 @@ incpos(GEN a)
 static GEN
 incneg(GEN a)
 {
-  long l = lgefint(a)-1;
+  long i, l = lgefint(a)-1;
   if (a[l]--)
   {
     if (l == 2 && !a[2])
@@ -104,11 +103,14 @@ incneg(GEN a)
     }
     return a;
   }
-  for (l--; l>1; l--)
-    if (a[l]--) break;
-  l++; a++; /* save one cell */
-  a[0] = evaltyp(t_INT) | _evallg(l);
-  a[1] = evalsigne(-1) | evallgefint(l);
+  for (i = l-1;; i--) /* finishes since a[2] != 0 */
+    if (a[i]--) break;
+  if (!a[2])
+  {
+    a++; /* save one cell */
+    a[0] = evaltyp(t_INT) | _evallg(l);
+    a[1] = evalsigne(-1) | evallgefint(l);
+  }
   return a;
 }
 
@@ -164,7 +166,7 @@ static GEN
 addiispec(GEN x, GEN y, long nx, long ny)
 {
   GEN xd, yd, zd;
-  long lz, i;
+  long lz, i = -2;
   LOCAL_OVERFLOW;
 
   if (nx < ny) swapspec(x,y, nx,ny);
@@ -174,7 +176,11 @@ addiispec(GEN x, GEN y, long nx, long ny)
   xd = x + nx;
   yd = y + ny;
   zd[-1] = addll(xd[-1], yd[-1]);
-  for (i = -2; i >= -ny; i--) zd[i] = addllx(xd[i], yd[i]);
+#ifdef addllx8
+  for (  ; i-8 > -ny; i-=8)
+    addllx8(xd+i, yd+i, zd+i, overflow);
+#endif
+  for (  ; i >= -ny; i--) zd[i] = addllx(xd[i], yd[i]);
   if (overflow)
     for(;;)
     {
@@ -224,7 +230,7 @@ static GEN
 subiispec(GEN x, GEN y, long nx, long ny)
 {
   GEN xd,yd,zd;
-  long lz;
+  long lz, i = -2;
   LOCAL_OVERFLOW;
 
   if (ny==1) return subiuspec(x,*y,nx);
@@ -232,18 +238,23 @@ subiispec(GEN x, GEN y, long nx, long ny)
   lz = nx+2; (void)new_chunk(lz);
   xd = x + nx;
   yd = y + ny;
-  *--zd = subll(*--xd, *--yd);
-  while (yd > y) *--zd = subllx(*--xd, *--yd);
+  zd[-1] = subll(xd[-1], yd[-1]);
+#ifdef subllx8
+  for (  ; i-8 > -ny; i-=8)
+    subllx8(xd+i, yd+i, zd+i, overflow);
+#endif
+  for (  ; i >= -ny; i--) zd[i] = subllx(xd[i], yd[i]);
   if (overflow)
     for(;;)
     {
-      *--zd = ((ulong)*--xd) - 1;
-      if (*xd) break;
+      zd[i] = ((ulong)xd[i]) - 1;
+      if (xd[i--]) break;
     }
-  if (xd == x)
-    while (*zd == 0) { zd++; lz--; } /* shorten z */
+  if (i>=-nx)
+    for (; i >= -nx; i--) zd[i] = xd[i];
   else
-    do  *--zd = *--xd; while (xd > x);
+    while (zd[i+1] == 0) { i++; lz--; } /* shorten z */
+  zd += i+1;
   *--zd = evalsigne(1) | evallgefint(lz);
   *--zd = evaltyp(t_INT) | evallg(lz);
   avma=(pari_sp)zd; return zd;
@@ -255,8 +266,8 @@ roundr_up_ip(GEN x, long l)
   long i = l;
   for(;;)
   {
-    if (++x[--i]) break;
-    if (i == 2) { x[2] = (long)HIGHBIT; setexpo(x, expo(x)+1); break; }
+    if (++((ulong*)x)[--i]) break;
+    if (i == 2) { x[2] = (long)HIGHBIT; shiftr_inplace(x, 1); break; }
   }
 }
 
@@ -298,66 +309,60 @@ affir(GEN x, GEN y)
   }
 }
 
-static GEN
-shifti_spec(GEN x, long lx, long n)
+INLINE GEN
+shiftispec(GEN x, long nx, long n)
 {
-  long ly, i, m, s = signe(x);
-  GEN y;
-  if (!s) return gen_0;
-  if (!n)
-  {
-    y = cgeti(lx);
-    y[1] = evalsigne(s) | evallgefint(lx);
-    while (--lx > 1) y[lx]=x[lx];
-    return y;
-  }
+  long ny, i, m;
+  GEN y, yd;
+  if (!n)  return icopyspec(x, nx);
+
   if (n > 0)
   {
     GEN z = (GEN)avma;
     long d = dvmdsBIL(n, &m);
 
-    ly = lx+d; y = new_chunk(ly);
+    ny = nx+d; y = new_chunk(ny + 2); yd = y + 2;
     for ( ; d; d--) *--z = 0;
-    if (!m) for (i=2; i<lx; i++) y[i]=x[i];
+    if (!m) for (i=0; i<nx; i++) yd[i]=x[i];
     else
     {
       register const ulong sh = BITS_IN_LONG - m;
-      shift_left(y,x, 2,lx-1, 0,m);
-      i = ((ulong)x[2]) >> sh;
+      shift_left(yd,x, 0,nx-1, 0,m);
+      i = ((ulong)x[0]) >> sh;
       /* Extend y on the left? */
-      if (i) { ly++; y = new_chunk(1); y[2] = i; }
+      if (i) { ny++; y = new_chunk(1); y[2] = i; }
     }
   }
   else
   {
-    ly = lx - dvmdsBIL(-n, &m);
-    if (ly<3) return gen_0;
-    y = new_chunk(ly);
+    ny = nx - dvmdsBIL(-n, &m);
+    if (ny<1) return gen_0;
+    y = new_chunk(ny + 2); yd = y + 2;
     if (m) {
-      shift_right(y,x, 2,ly, 0,m);
-      if (y[2] == 0)
+      shift_right(yd,x, 0,ny, 0,m);
+      if (yd[0] == 0)
       {
-        if (ly==3) { avma = (pari_sp)(y+3); return gen_0; }
-        ly--; avma = (pari_sp)(++y);
+        if (ny==1) { avma = (pari_sp)(y+3); return gen_0; }
+        ny--; avma = (pari_sp)(++y);
       }
     } else {
-      for (i=2; i<ly; i++) y[i]=x[i];
+      for (i=0; i<ny; i++) yd[i]=x[i];
     }
   }
-  y[1] = evalsigne(s)|evallgefint(ly);
-  y[0] = evaltyp(t_INT)|evallg(ly); return y;
+  y[1] = evalsigne(1)|evallgefint(ny + 2);
+  y[0] = evaltyp(t_INT)|evallg(ny + 2); return y;
 }
 
 GEN
-shifti(GEN x, long n)
-{
-  return shifti_spec(x, lgefint(x), n);
-}
-
-GEN
-trunc2nr_lg(GEN x, long lx, long n)
+mantissa2nr(GEN x, long n)
 { /*This is a kludge since x is not an integer*/
-  return shifti_spec(x, lx, n);
+  long s = signe(x);
+  GEN y;
+
+  if(s == 0) return gen_0;
+  y = shiftispec(x + 2, lg(x) - 2, n);
+  if (signe(y)) setsigne(y, s);
+  return y;
 }
 
 GEN
@@ -368,7 +373,7 @@ truncr(GEN x)
 
   if ((s=signe(x)) == 0 || (e=expo(x)) < 0) return gen_0;
   d = nbits2prec(e+1); m = remsBIL(e);
-  if (d > lg(x)) pari_err(precer, "truncr (precision loss in truncation)");
+  if (d > lg(x)) pari_err_PREC( "truncr (precision loss in truncation)");
 
   y=cgeti(d); y[1] = evalsigne(s) | evallgefint(d);
   if (++m == BITS_IN_LONG)
@@ -388,7 +393,7 @@ floorr(GEN x)
   if (signe(x) >= 0) return truncr(x);
   if ((e=expo(x)) < 0) return gen_m1;
   d = nbits2prec(e+1); m = remsBIL(e);
-  lx=lg(x); if (d>lx) pari_err(precer, "floorr (precision loss in truncation)");
+  lx=lg(x); if (d>lx) pari_err_PREC( "floorr (precision loss in truncation)");
   y = new_chunk(d);
   if (++m == BITS_IN_LONG)
   {
@@ -414,19 +419,23 @@ END:
 }
 
 INLINE int
-absi_cmp_lg(GEN x, GEN y, long l)
+cmpiispec(GEN x, GEN y, long lx, long ly)
 {
-  long i=2;
-  while (i<l && x[i]==y[i]) i++;
-  if (i==l) return 0;
+  long i;
+  if (lx < ly) return -1;
+  if (lx > ly) return  1;
+  i = 0; while (i<lx && x[i]==y[i]) i++;
+  if (i==lx) return 0;
   return ((ulong)x[i] > (ulong)y[i])? 1: -1;
 }
 
 INLINE int
-absi_equal_lg(GEN x, GEN y, long l)
+equaliispec(GEN x, GEN y, long lx, long ly)
 {
-  long i = l-1; while (i>1 && x[i]==y[i]) i--;
-  return i==1;
+  long i;
+  if (lx != ly) return 0;
+  i = ly-1; while (i>=0 && x[i]==y[i]) i--;
+  return i < 0;
 }
 
 /***********************************************************************/
@@ -476,10 +485,6 @@ addumului(ulong a, ulong b, GEN Y)
   avma=(pari_sp)z; return z;
 }
 
-GEN muliispec(GEN a, GEN b, long na, long nb);
-/*#define KARAMULR_VARIANT*/
-#define muliispec_mirror muliispec
-
 /***********************************************************************/
 /**                                                                   **/
 /**                          DIVISION                                 **/
@@ -492,7 +497,7 @@ umodiu(GEN y, ulong x)
   long sy=signe(y),ly,i;
   LOCAL_HIREMAINDER;
 
-  if (!x) pari_err(gdiver);
+  if (!x) pari_err_INV("umodiu",gen_0);
   if (!sy) return 0;
   ly = lgefint(y);
   if (x <= (ulong)y[2]) hiremainder=0;
@@ -514,7 +519,7 @@ diviu_rem(GEN y, ulong x, ulong *rem)
   GEN z;
   LOCAL_HIREMAINDER;
 
-  if (!x) pari_err(gdiver);
+  if (!x) pari_err_INV("diviu_rem",gen_0);
   if (!signe(y)) { *rem = 0; return gen_0; }
 
   ly = lgefint(y);
@@ -536,7 +541,7 @@ divis_rem(GEN y, long x, long *rem)
   GEN z;
   LOCAL_HIREMAINDER;
 
-  if (!x) pari_err(gdiver);
+  if (!x) pari_err_INV("divis_rem",gen_0);
   if (!sy) { *rem=0; return gen_0; }
   if (x<0) { s = -sy; x = -x; } else s = sy;
 
@@ -560,7 +565,7 @@ divis(GEN y, long x)
   GEN z;
   LOCAL_HIREMAINDER;
 
-  if (!x) pari_err(gdiver);
+  if (!x) pari_err_INV("divis",gen_0);
   if (!sy) return gen_0;
   if (x<0) { s = -sy; x = -x; } else s = sy;
 
@@ -583,7 +588,7 @@ divrr(GEN x, GEN y)
   ulong y0,y1;
   GEN r, r1;
 
-  if (!sy) pari_err(gdiver);
+  if (!x) pari_err_INV("divrr",y);
   e = expo(x) - expo(y);
   if (!sx) return real_0_bit(e);
   if (sy<0) sx = -sx;
@@ -625,7 +630,7 @@ divrr(GEN x, GEN y)
         GEN y1 = y+1;
         j = lr-i; r1[j] = subll(r1[j],y1[j]);
         for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
-        j=i; do r[--j]++; while (j && !r[j]);
+        j=i; do ((ulong*)r)[--j]++; while (j && !r[j]);
       }
       hiremainder = r1[1]; overflow = 0;
       qp = divll(r1[2],y0); k = hiremainder;
@@ -663,7 +668,7 @@ divrr(GEN x, GEN y)
         r1[1] -= hiremainder;
         while (r1[1])
         {
-          qp++; if (!qp) { j=i; do r[--j]++; while (j && !r[j]); }
+          qp++; if (!qp) { j=i; do ((ulong*)r)[--j]++; while (j && !r[j]); }
           j = lr-i-(lr-i>=ly); r1[j] = subll(r1[j],y[j]);
           for (j--; j>1; j--) r1[j] = subllx(r1[j],y[j]);
           r1[1] -= overflow;
@@ -676,7 +681,7 @@ divrr(GEN x, GEN y)
   /* round correctly */
   if ((ulong)r1[1] > (y0>>1))
   {
-    j=i; do r[--j]++; while (j && !r[j]);
+    j=i; do ((ulong*)r)[--j]++; while (j && !r[j]);
   }
   r1 = r-1; for (j=i; j>=2; j--) r[j]=r1[j];
   if (r[0] == 0) e--;
@@ -696,7 +701,7 @@ divri(GEN x, GEN y)
   pari_sp av;
   GEN z;
 
-  if (!s) pari_err(gdiver);
+  if (!s) pari_err_INV("divri",y);
   if (!signe(x)) return real_0_bit(expo(x) - expi(y));
   if (!is_bigint(y)) {
     GEN z = divru(x, y[2]);
@@ -724,7 +729,11 @@ dvmdii(GEN x, GEN y, GEN *z)
   ulong y0,y1, *xd,*rd,*qd;
   GEN q, r, r1;
 
-  if (!sy) { if (z == ONLY_REM && !sx) return gen_0; pari_err(gdiver); }
+  if (!sy)
+  {
+    if (z == ONLY_REM && !sx) return gen_0;
+    if (!sy) pari_err_INV("dvmdii",gen_0);
+  }
   if (!sx)
   {
     if (!z || z == ONLY_REM) return gen_0;
@@ -931,7 +940,7 @@ red_montgomery(GEN T, GEN N, ulong inv)
   d = NLIMBS(T); /* <= 2*k */
   if (d == 0) return gen_0;
 #ifdef DEBUG
-  if (d > 2*k) pari_err(bugparier,"red_montgomery");
+  if (d > 2*k) pari_err_BUG("red_montgomery");
 #endif
   if (k == 1)
   { /* as below, special cased for efficiency */
@@ -1005,7 +1014,7 @@ red_montgomery(GEN T, GEN N, ulong inv)
   GEN res = remii(mulii(T, Fp_inv(R, N)), N);
   if (k > lgefint(N)
     || !equalii(remii(Td,N),res)
-    || cmpii(Td, addii(shifti(T, -s), N)) >= 0) pari_err(bugparier,"red_montgomery");
+    || cmpii(Td, addii(shifti(T, -s), N)) >= 0) pari_err_BUG("red_montgomery");
 }
 #endif
   avma = (pari_sp)Td; return Td;
@@ -1100,7 +1109,7 @@ diviiexact(GEN x, GEN y)
   ulong y0inv,q;
   GEN z;
 
-  if (!sy) pari_err(gdiver);
+  if (!sy) pari_err_INV("diviiexact",gen_0);
   if (!sx) return gen_0;
   lx = lgefint(x);
   if (lx == 3) {
@@ -1166,6 +1175,21 @@ diviiexact(GEN x, GEN y)
   avma = (pari_sp)z; return z;
 }
 
+/* assume yz != and yz | x */
+GEN
+diviuuexact(GEN x, ulong y, ulong z)
+{
+  long tmp[4];
+  ulong t;
+  LOCAL_HIREMAINDER;
+  t = mulll(y, z);
+  if (!hiremainder) return diviuexact(x, t);
+  tmp[0] = evaltyp(t_INT)|_evallg(4);
+  tmp[1] = evalsigne(1)|evallgefint(4);
+  tmp[2] = hiremainder;
+  tmp[3] = t;
+  return diviiexact(x, tmp);
+}
 
 /********************************************************************/
 /**                                                                **/
@@ -1635,6 +1659,29 @@ muliispec(GEN a, GEN b, long na, long nb)
   }
   return gerepileuptoint(av, addshiftw(c,c0, n0));
 }
+GEN
+muluui(ulong x, ulong y, GEN z)
+{
+  long t, s = signe(z);
+  GEN r;
+  LOCAL_HIREMAINDER;
+
+  if (!x || !y || !signe(z)) return gen_0;
+  t = mulll(x,y);
+  if (!hiremainder)
+    r = muluispec(t, z+2, lgefint(z)-2);
+  else
+  {
+    long tmp[2];
+    tmp[0] = hiremainder;
+    tmp[1] = t;
+    r = muliispec(z+2,tmp,lgefint(z)-2,2);
+  }
+  setsigne(r,s); return r;
+}
+
+#define sqrispec_mirror sqrispec
+#define muliispec_mirror muliispec
 
 /* x % (2^n), assuming n >= 0 */
 GEN
@@ -2052,10 +2099,10 @@ sqrtr_abs(GEN x)
     l1 <<= 1; if (l1 > l) l1 = l;
     setlg(a, l1 + 2);
     setlg(t, l1 + 2);
-    affrr(addrr(t, divrr(a,t)), t); setexpo(t, expo(t)-1);
+    affrr(addrr(t, divrr(a,t)), t); shiftr_inplace(t, -1);
     avma = av;
   }
-  affrr(t,y); setexpo(y, expo(y) + (ex>>1));
+  affrr(t,y); shiftr_inplace(y, (ex>>1));
   avma = av0; return y;
 }
 
