@@ -1,7 +1,5 @@
 #line 2 "../src/kernel/none/mp_indep.c"
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -51,20 +49,18 @@ affrr(GEN x, GEN y)
 GEN
 trunc2nr(GEN x, long n)
 {
-  long ex, lx;
+  long ex;
   if (!signe(x)) return gen_0;
   ex = expo(x) + n; if (ex < 0) return gen_0;
-  lx = lg(x);
-  return trunc2nr_lg(x, lx, ex - bit_accuracy(lx) + 1);
+  return mantissa2nr(x, ex - bit_prec(x) + 1);
 }
 
 /* x a t_REAL, x = i/2^e, i a t_INT */
 GEN
 mantissa_real(GEN x, long *e)
 {
-  long lx = lg(x);
-  *e = bit_accuracy(lx)-1-expo(x);
-  return trunc2nr_lg(x, lx, 0);
+  *e = bit_prec(x)-1-expo(x);
+  return mantissa2nr(x, 0);
 }
 
 GEN
@@ -147,10 +143,11 @@ mulur_2(ulong x, GEN y, long s)
   y--; garde = mulll(x,y[lx]);
   for (i=lx-1; i>=3; i--) z[i]=addmul(x,y[i]);
   z[2]=hiremainder; /* != 0 since y normalized and |x| > 1 */
-
   sh = bfffo(hiremainder); m = BITS_IN_LONG-sh;
   if (sh) shift_left(z,z, 2,lx-1, garde,sh);
-  z[1] = evalsigne(s) | evalexpo(m+e); return z;
+  z[1] = evalsigne(s) | evalexpo(m+e);
+  if ((garde << sh) & HIGHBIT) roundr_up_ip(z, lx);
+  return z;
 }
 
 INLINE GEN
@@ -199,97 +196,158 @@ mulur(ulong x, GEN y)
   return mulur_2(x, y, s);
 }
 
-#ifdef KARAMULR_VARIANT
-static GEN addshiftw(GEN x, GEN y, long d);
-static GEN
-karamulrr1(GEN y, GEN x, long ly, long lz)
-{
-  long i, l, lz2 = (lz+2)>>1, lz3 = lz-lz2;
-  GEN lo1, lo2, hi;
-
-  hi = muliispec_mirror(x,y, lz2,lz2);
-  i = lz2; while (i<lz && !x[i]) i++;
-  lo1 = muliispec_mirror(y,x+i, lz2,lz-i);
-  i = lz2; while (i<ly && !y[i]) i++;
-  lo2 = muliispec_mirror(x,y+i, lz2,ly-i);
-  if (signe(lo1))
-  {
-    if (ly!=lz) { lo2 = addshiftw(lo1,lo2,1); lz3++; }
-    else lo2 = addii(lo1,lo2);
-  }
-  l = lgefint(lo2)-(lz3+2);
-  if (l > 0) hi = addiispec(hi+2,lo2+2, lgefint(hi)-2,l);
-  return hi;
-}
-#endif
-
-/* set z <-- x*y, floating point multiplication.
- * lz = lg(z) = lg(x) <= ly <= lg(y), sz = signe(z). flag = lg(x) < lg(y) */
 INLINE void
-mulrrz_i(GEN z, GEN x, GEN y, long lz, long flag, long sz)
+mulrrz_end(GEN z, GEN hi, long lz, long sz, long ez, ulong garde)
 {
-  long ez = expo(x) + expo(y);
+  long i;
+  if (hi[2] < 0)
+  {
+    if (z != hi)
+      for (i=2; i<lz ; i++) z[i] = hi[i];
+    ez++;
+  }
+  else
+  {
+    shift_left(z,hi,2,lz-1, garde, 1);
+    garde <<= 1;
+  }
+  if (garde & HIGHBIT)
+  { /* round to nearest */
+    i = lz; do ((ulong*)z)[--i]++; while (i>1 && z[i]==0);
+    if (i == 1) { z[2] = (long)HIGHBIT; ez++; }
+  }
+  z[1] = evalsigne(sz)|evalexpo(ez);
+}
+/* mulrrz_end for lz = 3, minor simplifications. z[2]=hiremainder from mulll */
+INLINE void
+mulrrz_3end(GEN z, long sz, long ez, ulong garde)
+{
+  if (z[2] < 0)
+  { /* z2 < (2^BIL-1)^2 / 2^BIL, hence z2+1 != 0 */
+    if (garde & HIGHBIT) z[2]++; /* round properly */
+    ez++;
+  }
+  else
+  {
+    z[2] = (z[2]<<1) | (garde>>(BITS_IN_LONG-1));
+    if (garde & (HIGHBIT-1))
+    {
+      z[2]++; /* round properly, z2+1 can overflow */
+      if (!z[2]) { z[2] = (long)HIGHBIT; ez++; }
+    }
+  }
+  z[1] = evalsigne(sz)|evalexpo(ez);
+}
+
+/* set z <-- x^2 != 0, floating point multiplication.
+ * lz = lg(z) = lg(x) */
+INLINE void
+sqrz_i(GEN z, GEN x, long lz)
+{
+  long ez = expo(x) << 1;
   long i, j, lzz, p1;
   ulong garde;
-  GEN y1;
+  GEN x1;
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
   if (lz > MULRR_MULII_LIMIT)
   {
     pari_sp av = avma;
-#ifdef KARAMULR_VARIANT
-    GEN hi = karamulrr1(y+2, x+2, lz+flag-2, lz-2);
-#else
-    GEN hi = muliispec_mirror(y+2, x+2, lz+flag-2, lz-2);
-#endif
-    garde = hi[lz];
-    if (hi[2] < 0)
-    {
-      ez++;
-      for (i=2; i<lz ; i++) z[i] = hi[i];
-    }
-    else
-    {
-      shift_left(z,hi,2,lz-1, garde, 1);
-      garde <<= 1;
-    }
-    if (garde & HIGHBIT)
-    { /* round to nearest */
-      i = lz; do z[--i]++; while (i>1 && z[i]==0);
-      if (i == 1) { z[2] = (long)HIGHBIT; ez++; }
-    }
-    z[1] = evalsigne(sz)|evalexpo(ez);
+    GEN hi = sqrispec_mirror(x+2, lz-2);
+    mulrrz_end(z, hi, lz, 1, ez, hi[lz]);
     avma = av; return;
   }
   if (lz == 3)
   {
-    if (flag)
-    {
-      (void)mulll(x[2],y[3]);
-      garde = addmul(x[2],y[2]);
-    }
-    else
-      garde = mulll(x[2],y[2]);
-    if (hiremainder & HIGHBIT)
-    {
-      ez++;
-      /* hiremainder < (2^BIL-1)^2 / 2^BIL, hence hiremainder+1 != 0 */
-      if (garde & HIGHBIT) hiremainder++; /* round properly */
-    }
-    else
-    {
-      hiremainder = (hiremainder<<1) | (garde>>(BITS_IN_LONG-1));
-      if (garde & (HIGHBIT-1))
-      {
-        hiremainder++; /* round properly */
-        if (!hiremainder) { hiremainder = HIGHBIT; ez++; }
-      }
-    }
-    z[1] = evalsigne(sz) | evalexpo(ez);
-    z[2] = hiremainder; return;
+    garde = mulll(x[2],x[2]);
+    z[2] = hiremainder;
+    mulrrz_3end(z, 1, ez, garde);
+    return;
   }
 
+  lzz = lz-1; p1 = x[lzz];
+  if (p1)
+  {
+    (void)mulll(p1,x[3]);
+    garde = addmul(p1,x[2]);
+    z[lzz] = hiremainder;
+  }
+  else
+  {
+    garde = 0;
+    z[lzz] = 0;
+  }
+  for (j=lz-2, x1=x-j; j>=3; j--)
+  {
+    p1 = x[j]; x1++;
+    if (p1)
+    {
+      (void)mulll(p1,x1[lz+1]);
+      garde = addll(addmul(p1,x1[lz]), garde);
+      for (i=lzz; i>j; i--)
+      {
+        hiremainder += overflow;
+        z[i] = addll(addmul(p1,x1[i]), z[i]);
+      }
+      z[j] = hiremainder+overflow;
+    }
+    else z[j]=0;
+  }
+  p1 = x[2]; x1++;
+  garde = addll(mulll(p1,x1[lz]), garde);
+  for (i=lzz; i>2; i--)
+  {
+    hiremainder += overflow;
+    z[i] = addll(addmul(p1,x1[i]), z[i]);
+  }
+  z[2] = hiremainder+overflow;
+  mulrrz_end(z, z, lz, 1, ez, garde);
+}
+
+/* lz "large" = lg(y) = lg(z), lg(x) > lz if flag = 1 and >= if flag = 0 */
+INLINE void
+mulrrz_int(GEN z, GEN x, GEN y, long lz, long flag, long sz)
+{
+  pari_sp av = avma;
+  GEN hi = muliispec_mirror(y+2, x+2, lz+flag-2, lz-2);
+  mulrrz_end(z, hi, lz, sz, expo(x)+expo(y), hi[lz]);
+  avma = av;
+}
+
+/* lz = 3 */
+INLINE void
+mulrrz_3(GEN z, GEN x, GEN y, long flag, long sz)
+{
+  ulong garde;
+  LOCAL_HIREMAINDER;
+  if (flag)
+  {
+    (void)mulll(x[2],y[3]);
+    garde = addmul(x[2],y[2]);
+  }
+  else
+    garde = mulll(x[2],y[2]);
+  z[2] = hiremainder;
+  mulrrz_3end(z, sz, expo(x)+expo(y), garde);
+}
+
+/* set z <-- x*y, floating point multiplication. Trailing 0s for x are
+ * treated efficiently (important application: mulir).
+ * lz = lg(z) = lg(x) <= ly <= lg(y), sz = signe(z). flag = lg(x) < lg(y) */
+INLINE void
+mulrrz_i(GEN z, GEN x, GEN y, long lz, long flag, long sz)
+{
+  long ez, i, j, lzz, p1;
+  ulong garde;
+  GEN y1;
+  LOCAL_HIREMAINDER;
+  LOCAL_OVERFLOW;
+
+  if (x == y) { sqrz_i(z,x,lz); return; }
+  if (lz > MULRR_MULII_LIMIT) { mulrrz_int(z,x,y,lz,flag,sz); return; }
+  if (lz == 3) { mulrrz_3(z,x,y,flag,sz); return; }
+  ez = expo(x) + expo(y);
   if (flag) { (void)mulll(x[2],y[lz]); garde = hiremainder; } else garde = 0;
   lzz=lz-1; p1=x[lzz];
   if (p1)
@@ -323,29 +381,18 @@ mulrrz_i(GEN z, GEN x, GEN y, long lz, long flag, long sz)
     z[i] = addll(addmul(p1,y1[i]), z[i]);
   }
   z[2] = hiremainder+overflow;
-
-  if (z[2] < 0)
-    ez++;
-  else
-  {
-    shift_left(z,z,2,lzz, garde, 1);
-    garde <<= 1;
-  }
-  if (garde & HIGHBIT)
-  { /* round to nearest */
-    i = lz; do z[--i]++; while (i>2 && z[i]==0);
-    if (z[i] == 0) { z[2] = (long)HIGHBIT; ez++; }
-  }
-  z[1] = evalsigne(sz) | evalexpo(ez);
+  mulrrz_end(z, z, lz, sz, ez, garde);
 }
 
 GEN
 mulrr(GEN x, GEN y)
 {
-  long flag, ly, lz, sx = signe(x), sy = signe(y);
+  long flag, ly, lz, sx, sy;
   GEN z;
 
-  if (!sx || !sy) return real_0_bit(expo(x) + expo(y));
+  if (x == y) return sqrr(x);
+  sx = signe(x); if (!sx) return real_0_bit(expo(x) + expo(y));
+  sy = signe(y); if (!sy) return real_0_bit(expo(x) + expo(y));
   if (sy < 0) sx = -sx;
   lz = lg(x);
   ly = lg(y);
@@ -363,26 +410,104 @@ sqrr(GEN x)
 
   if (!sx) return real_0_bit(2*expo(x));
   lz = lg(x); z = cgetr(lz);
-  mulrrz_i(z, x,x, lz,0, 1);
+  sqrz_i(z, x, lz);
   return z;
 }
 
 GEN
 mulir(GEN x, GEN y)
 {
-  long sx = signe(x), sy, lz;
-  GEN z;
-
+  long sx = signe(x), sy;
   if (!sx) return mul0r(y);
   if (lgefint(x) == 3) {
-    z = mulur((ulong)x[2], y); if (sx < 0) togglesign(z); return z;
+    GEN z = mulur((ulong)x[2], y);
+    if (sx < 0) togglesign(z);
+    return z;
   }
   sy = signe(y);
   if (!sy) return real_0_bit(expi(x) + expo(y));
   if (sy < 0) sx = -sx;
-  lz = lg(y); z = cgetr(lz);
-  mulrrz_i(z, itor(x,lz),y, lz,0, sx);
-  avma = (pari_sp)z; return z;
+  {
+    long lz = lg(y), lx = lgefint(x);
+    GEN hi, z = cgetr(lz);
+    pari_sp av = avma;
+    if (lx < (lz>>1) || (lx < lz && lz > MULRR_MULII_LIMIT))
+    { /* size mantissa of x < half size of mantissa z, or lx < lz so large
+       * that mulrr will call mulii anyway: mulii */
+      x = itor(x, lx);
+      hi = muliispec_mirror(y+2, x+2, lz-2, lx-2);
+      mulrrz_end(z, hi, lz, sx, expo(x)+expo(y), hi[lz]);
+    }
+    else /* dubious: complete x with 0s and call mulrr */
+      mulrrz_i(z, itor(x,lz), y, lz, 0, sx);
+    avma = av; return z;
+  }
+}
+
+/* x + y*z, generic. If lgefint(z) <= 3, caller should use faster variants  */
+static GEN
+addmulii_gen(GEN x, GEN y, GEN z, long lz)
+{
+  long lx = lgefint(x), ly;
+  pari_sp av;
+  GEN t;
+  if (lx == 2) return mulii(z,y);
+  ly = lgefint(y);
+  if (ly == 2) return icopy(x); /* y = 0, wasteful copy */
+  av = avma; (void)new_chunk(lx+ly+lz); /*HACK*/
+  t = mulii(z, y);
+  avma = av; return addii(t,x);
+}
+/* x + y*z, lgefint(z) == 3 */
+static GEN
+addmulii_lg3(GEN x, GEN y, GEN z)
+{
+  long s = signe(z), lx, ly;
+  ulong w = z[2];
+  pari_sp av;
+  GEN t;
+  if (w == 1) return (s > 0)? addii(x,y): subii(x,y); /* z = +- 1 */
+  lx = lgefint(x);
+  ly = lgefint(y);
+  if (lx == 2)
+  { /* x = 0 */
+    if (ly == 2) return gen_0;
+    t = muluispec(w, y+2, ly-2);
+    if (signe(y) < 0) s = -s;
+    setsigne(t, s); return t;
+  }
+  if (ly == 2) return icopy(x); /* y = 0, wasteful copy */
+  av = avma; (void)new_chunk(1+lx+ly);/*HACK*/
+  t = muluispec(w, y+2, ly-2);
+  if (signe(y) < 0) s = -s;
+  setsigne(t, s);
+  avma = av; return addii(x,t);
+}
+/* x + y*z */
+GEN
+addmulii(GEN x, GEN y, GEN z)
+{
+  long lz = lgefint(z);
+  switch(lz)
+  {
+    case 2: return icopy(x); /* z = 0, wasteful copy */
+    case 3: return addmulii_lg3(x, y, z);
+    default:return addmulii_gen(x, y, z, lz);
+  }
+}
+/* x + y*z, returns x itself and not a copy when y*z = 0 */
+GEN
+addmulii_inplace(GEN x, GEN y, GEN z)
+{
+  long lz;
+  if (lgefint(y) == 2) return x;
+  lz = lgefint(z);
+  switch(lz)
+  {
+    case 2: return x;
+    case 3: return addmulii_lg3(x, y, z);
+    default:return addmulii_gen(x, y, z, lz);
+  }
 }
 
 /* written by Bruno Haible following an idea of Robert Harley */
@@ -415,7 +540,7 @@ divsi(long x, GEN y)
   long p1, s = signe(y);
   LOCAL_HIREMAINDER;
 
-  if (!s) pari_err(gdiver);
+  if (!s) pari_err_INV("divsi",gen_0);
   if (!x || lgefint(y)>3 || ((long)y[2])<0) return gen_0;
   hiremainder=0; p1=divll(labs(x),y[2]);
   if (x<0) { hiremainder = -((long)hiremainder); p1 = -p1; }
@@ -430,7 +555,7 @@ divir(GEN x, GEN y)
   long ly = lg(y), lx = lgefint(x);
   pari_sp av;
 
-  if (ly == 2) pari_err(gdiver);
+  if (ly == 2) pari_err_INV("divir",y);
   if (lx == 2) return div0r(y);
   if (lx == 3) {
     z = divur(x[2], y);
@@ -449,7 +574,7 @@ divur(ulong x, GEN y)
   long ly = lg(y);
   GEN z;
 
-  if (ly == 2) pari_err(gdiver);
+  if (ly == 2) pari_err_INV("divur",y);
   if (!x) return div0r(y);
   if (ly > INVNEWTON_LIMIT) {
     av = avma; z = invr(y);
@@ -468,7 +593,7 @@ divsr(long x, GEN y)
   long ly = lg(y);
   GEN z;
 
-  if (ly == 2) pari_err(gdiver);
+  if (ly == 2) pari_err_INV("divsr",y);
   if (!x) return div0r(y);
   if (ly > INVNEWTON_LIMIT) {
     av = avma; z = invr(y);
@@ -501,7 +626,7 @@ invr(GEN b)
   ulong mask;
 
   if (l <= maxss(INVNEWTON_LIMIT, (1L<<s) + 2)) {
-    if (l == 2) pari_err(gdiver);
+    if (l == 2) pari_err_INV("invr",b);
     return invr_basecase(b);
   }
   mask = quadratic_prec_mask(l-2);
@@ -556,7 +681,7 @@ divrs(GEN x, long y)
   GEN z;
   LOCAL_HIREMAINDER;
 
-  if (!y) pari_err(gdiver);
+  if (!y) pari_err_INV("divrs",gen_0);
   if (y<0) { s = -s; y = -y; }
   if (!s) return real_0_bit(expo(x) - expu(y));
   if (y==1) { z = rcopy(x); setsigne(z,s); return z; }
@@ -580,7 +705,7 @@ divru(GEN x, ulong y)
   GEN z;
   LOCAL_HIREMAINDER;
 
-  if (!y) pari_err(gdiver);
+  if (!y) pari_err_INV("divru",gen_0);
   if (!s) return real_0_bit(expo(x) - expu(y));
   if (y==1) return rcopy(x);
   if (y==2) return shiftr(x, -1);
@@ -617,28 +742,20 @@ truedvmdii(GEN x, GEN y, GEN *z)
   pari_sp av;
   GEN r, q, *gptr[2];
   if (!is_bigint(y)) return truedvmdis(x, itos(y), z);
+  if (z == ONLY_REM) return modii(x,y);
 
   av = avma;
   q = dvmdii(x,y,&r); /* assume that r is last on stack */
-
   switch(signe(r))
   {
     case 0:
-      if (z == ONLY_REM) { avma = av; return gen_0; }
       if (z) *z = gen_0;
       return q;
     case 1:
-      if (z == ONLY_REM) return gerepileuptoint(av,r);
       if (z) *z = r; else cgiv(r);
       return q;
     case -1: break;
   }
-
-  if (z == ONLY_REM)
-  { /* r += sign(y) * y, that is |y| */
-    r = subiispec(y+2,r+2, lgefint(y)-2,lgefint(r)-2);
-    return gerepileuptoint(av, r);
-  }
   q = addis(q, -signe(y));
   if (!z) return gerepileuptoint(av, q);
 
@@ -651,15 +768,16 @@ truedvmdis(GEN x, long y, GEN *z)
 {
   pari_sp av = avma;
   long r;
-  GEN q = divis_rem(x,y,&r);
+  GEN q;
+
+  if (z == ONLY_REM) return modis(x, y);
+  q = divis_rem(x,y,&r);
 
   if (r >= 0)
   {
-    if (z == ONLY_REM) { avma = av; return utoi(r); }
     if (z) *z = utoi(r);
     return q;
   }
-  if (z == ONLY_REM) { avma = av; return utoi(r + labs(y)); }
   q = gerepileuptoint(av, addis(q, (y < 0)? 1: -1));
   if (z) *z = utoi(r + labs(y));
   return q;
@@ -668,14 +786,12 @@ GEN
 truedvmdsi(long x, GEN y, GEN *z)
 {
   long q, r;
+  if (z == ONLY_REM) return modsi(x, y);
   q = sdivsi_rem(x,y,&r);
   if (r >= 0) {
-    if (z == ONLY_REM) return utoi(r);
     if (z) *z = utoi(r);
     return stoi(q);
   }
-  if (z == ONLY_REM) /* r += sign(y) * y, that is |y| */
-    return subiuspec(y+2,(ulong)-r, lgefint(y)-2);
   q = q - signe(y);
   if (!z) return stoi(q);
 
@@ -710,6 +826,18 @@ int2u(ulong n) {
   *int_MSW(z) = 1L << m; return z;
 }
 
+GEN
+shifti(GEN x, long n)
+{
+  long s = signe(x);
+  GEN y;
+
+  if(s == 0) return gen_0;
+  y = shiftispec(x + 2, lgefint(x) - 2, n);
+  if (signe(y)) setsigne(y, s);
+  return y;
+}
+
 /* actual operations will take place on a+2 and b+2: we strip the codewords */
 GEN
 mulii(GEN a,GEN b)
@@ -730,7 +858,7 @@ sqri(GEN a) { return sqrispec(a+2, lgefint(a)-2); }
 /* sqrt()'s result may be off by 1 when a is not representable exactly as a
  * double [64bit machine] */
 ulong
-usqrtsafe(ulong a)
+usqrt(ulong a)
 {
   ulong x = (ulong)sqrt((double)a);
 #ifdef LONG_IS_64BIT
@@ -785,7 +913,7 @@ dbltor(double x)
     const ulong a = fi.i;
     ulong A;
     e = ((a & (HIGHBIT-1)) >> mant_len) - exp_mid;
-    if (e == exp_mid+1) pari_err(talker, "NaN or Infinity in dbltor");
+    if (e == exp_mid+1) pari_err_OVERFLOW("dbltor [NaN or Infinity]");
     A = a << expo_len;
     if (e == -exp_mid)
     { /* unnormalized values */
@@ -810,14 +938,12 @@ rtodbl(GEN x)
   const int exp_mid = 0x3ff;/* exponent bias */
   const int expo_len = 11; /* number of bits of exponent */
 
-  if (typ(x)==t_INT && !s) return 0.0;
-  if (typ(x)!=t_REAL) pari_err(typeer,"rtodbl");
   if (!s || (ex=expo(x)) < - exp_mid) return 0.0;
 
   /* start by rounding to closest */
   a = (x[2] & (HIGHBIT-1)) + 0x400;
   if (a & HIGHBIT) { ex++; a=0; }
-  if (ex >= exp_mid) pari_err(overflower,"t_REAL->double conversion");
+  if (ex >= exp_mid) pari_err_OVERFLOW("t_REAL->double conversion");
   fi.i = ((ex + exp_mid) << mant_len) | (a >> expo_len);
   if (s<0) fi.i |= HIGHBIT;
   return fi.f;
@@ -882,7 +1008,7 @@ dbltor(double x)
     const ulong b = fi.i[INDEX1];
     ulong A, B;
     e = ((a & (HIGHBIT-1)) >> shift) - exp_mid;
-    if (e == exp_mid+1) pari_err(talker, "NaN or Infinity in dbltor");
+    if (e == exp_mid+1) pari_err_OVERFLOW("dbltor [NaN or Infinity]");
     A = b >> (BITS_IN_LONG-expo_len) | (a << expo_len);
     B = b << expo_len;
     if (e == -exp_mid)
@@ -924,8 +1050,6 @@ rtodbl(GEN x)
   const int expo_len = 11; /* number of bits of exponent */
   const int shift = mant_len-32;
 
-  if (typ(x)==t_INT && !s) return 0.0;
-  if (typ(x)!=t_REAL) pari_err(typeer,"rtodbl");
   if (!s || (ex=expo(x)) < - exp_mid) return 0.0;
 
   /* start by rounding to closest */
@@ -936,7 +1060,7 @@ rtodbl(GEN x)
     if (a & HIGHBIT) { ex++; a=0; }
   }
   else b = 0;
-  if (ex >= exp_mid) pari_err(overflower,"t_REAL->double conversion");
+  if (ex >= exp_mid) pari_err_OVERFLOW("t_REAL->double conversion");
   ex += exp_mid;
   k = (a >> expo_len) | (ex << shift);
   if (s<0) k |= HIGHBIT;
