@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -53,7 +51,7 @@ eval_mnemonic(GEN str, const char *tmplate)
   const char *arg;
 
   if (typ(str)==t_INT) return itos(str);
-  if (typ(str)!=t_STR) pari_err(talker,"invalid flag");
+  if (typ(str)!=t_STR) pari_err_TYPE("eval_mnemonic",str);
 
   arg=GSTR(str);
   etmplate = strchr(tmplate, '\n');
@@ -78,9 +76,9 @@ eval_mnemonic(GEN str, const char *tmplate)
     /* Now the ID is whatever is between arg and e. */
     l = e - arg;
     if (l >= sizeof(b))
-      pari_err(talker,"id too long in a stringified flag");
+      pari_err(e_MISC,"id too long in a stringified flag");
     if (!l)                             /* Garbage after whitespace? */
-      pari_err(talker,"a stringified flag does not start with an id");
+      pari_err(e_MISC,"a stringified flag does not start with an id");
     strncpy(b, arg, l);
     b[l] = 0;
     arg = e;
@@ -88,7 +86,7 @@ eval_mnemonic(GEN str, const char *tmplate)
     while (('0' <= *e) && (*e <= '9'))
       e++;
     if (*e == 0)
-      pari_err(talker,"numeric id in a stringified flag");
+      pari_err(e_MISC,"numeric id in a stringified flag");
     negate = 0;
     negated = NULL;
 find:
@@ -137,14 +135,14 @@ find:
       negate = 1;
     }
     if (!id)
-      pari_err(talker,"Unrecognized id '%s' in a stringified flag", inibuf);
+      pari_err(e_MISC,"Unrecognized id '%s' in a stringified flag", inibuf);
     if (singleton && !first)
-      pari_err(talker,"Singleton id non-single in a stringified flag");
+      pari_err(e_MISC,"Singleton id non-single in a stringified flag");
     if (id[0] == '=') {
       if (negate)
-        pari_err(talker,"Cannot negate id=value in a stringified flag");
+        pari_err(e_MISC,"Cannot negate id=value in a stringified flag");
       if (!first)
-        pari_err(talker,"Assign action should be first in a stringified flag");
+        pari_err(e_MISC,"Assign action should be first in a stringified flag");
       action = A_ACTION_ASSIGN;
       id++;
       if (id[0] == '=') {
@@ -153,7 +151,7 @@ find:
       }
     } else if (id[0] == '^') {
       if (id[1] != '~')
-        pari_err(talker, "Unrecognized action in a template");
+        pari_err(e_MISC, "Unrecognized action in a template");
       id += 2;
       if (negate)
         action = A_ACTION_SET;
@@ -173,7 +171,7 @@ find:
     while (isspace((int)*e))
       e++;
     if (*e && (*e != ';') && (*e != ','))
-      pari_err(talker, "Non-numeric argument of an action in a template");
+      pari_err(e_MISC, "Non-numeric argument of an action in a template");
     numarg = atol(id);          /* Now it is safe to get it... */
     switch (action) {
     case A_ACTION_SET:
@@ -186,13 +184,13 @@ find:
       retval = numarg;
       break;
     default:
-      pari_err(talker,"error in parse_option_string");
+      pari_err(e_MISC,"error in parse_option_string");
     }
     first = 0;
     while (isspace((int)*arg))
       arg++;
     if (*arg && !(ispunct((int)*arg) && *arg != '-'))
-      pari_err(talker,"Junk after an id in a stringified flag");
+      pari_err(e_MISC,"Junk after an id in a stringified flag");
     /* Skip punctuation */
     if (*arg)
       arg++;
@@ -241,7 +239,8 @@ check_proto(const char *code)
     case '&':
     case 'C':
     case 'G':
-    case 'I': case 'E':
+    case 'I':
+    case 'J':
     case 'L':
     case 'M':
     case 'P':
@@ -253,6 +252,7 @@ check_proto(const char *code)
     case 'x':
       arity++;
       break;
+    case 'E':
     case 's':
       if (*s == '*') s++;
       arity++;
@@ -265,7 +265,7 @@ check_proto(const char *code)
         s++; break;
       }
       old = s; while (*s && *s != ',') s++;
-      if (*s != ',') pari_err(syntaxer, "missing comma", old, code);
+      if (*s != ',') pari_err(e_SYNTAX, "missing comma", old, code);
       break;
     case 'V':
     case '=':
@@ -275,10 +275,10 @@ check_proto(const char *code)
     case 'm':
     case 'l':
     case 'i':
-    case 'v': pari_err(syntaxer, "this code has to come first", s-1, code);
-    default: pari_err(syntaxer, "unknown parser code", s-1, code);
+    case 'v': pari_err(e_SYNTAX, "this code has to come first", s-1, code);
+    default: pari_err(e_SYNTAX, "unknown parser code", s-1, code);
   }
-  if (arity > 20) pari_err(impl,"functions with more than 20 parameters");
+  if (arity > 20) pari_err_IMPL("functions with more than 20 parameters");
   return arity;
 }
 
@@ -302,7 +302,7 @@ installep(const char *name, long len, entree **table)
 }
 
 entree *
-install(void *f, char *name, char *code)
+install(void *f, const char *name, const char *code)
 {
   long hash, arity;
   entree *ep = is_entry_intern(name, functions_hash, &hash);
@@ -311,16 +311,16 @@ install(void *f, char *name, char *code)
   if (ep && ep->valence != EpNEW)
   {
     if (ep->valence != EpINSTALL)
-      pari_err(talker,"[install] identifier '%s' already in use", name);
+      pari_err(e_MISC,"[install] identifier '%s' already in use", name);
     pari_warn(warner, "[install] updating '%s' prototype; module not reloaded", name);
     if (ep->code) pari_free((void*)ep->code);
   }
   else
   {
-    char *s = name;
+    const char *s = name;
     if (isalpha((int)*s))
       while (is_keyword_char(*++s)) /* empty */;
-    if (*s) pari_err(syntaxer,"not a valid identifier", s, name);
+    if (*s) pari_err(e_SYNTAX,"not a valid identifier", s, name);
     if (!ep) ep = installep(name, strlen(name), functions_hash + hash);
     ep->value=f; ep->valence=EpINSTALL;
   }
@@ -334,7 +334,7 @@ void
 kill0(const char *e)
 {
   entree *ep = is_entry(e);
-  if (!ep || EpSTATIC(ep)) pari_err(talker,"can't kill that");
+  if (!ep || EpSTATIC(ep)) pari_err(e_MISC,"can't kill that");
   freeep(ep);
   ep->valence = EpNEW;
   ep->value   = NULL;
@@ -362,11 +362,17 @@ type0(GEN x)
 /*                                                                 */
 /*******************************************************************/
 
+#ifdef LONG_IS_64BIT
+static const long MAX_DIGITS = 19;
+#else
+static const long MAX_DIGITS = 9;
+#endif
+
 static ulong
 number(int *n, const char **s)
 {
   ulong m = 0;
-  for (*n = 0; *n < 9 && isdigit((int)**s); (*n)++,(*s)++)
+  for (*n = 0; *n < MAX_DIGITS && isdigit((int)**s); (*n)++,(*s)++)
     m = 10*m + (**s - '0');
   return m;
 }
@@ -374,8 +380,30 @@ number(int *n, const char **s)
 ulong
 u_pow10(int n)
 {
-  const ulong pw10[] = { 1UL, 10UL, 100UL, 1000UL, 10000UL, 100000UL,
-                        1000000UL, 10000000UL, 100000000UL, 1000000000UL };
+  const ulong pw10[] = {
+    1UL
+    ,10UL
+    ,100UL
+    ,1000UL
+    ,10000UL
+    ,100000UL
+    ,1000000UL
+    ,10000000UL
+    ,100000000UL
+    ,1000000000UL
+#ifdef LONG_IS_64BIT
+    ,10000000000UL
+    ,100000000000UL
+    ,1000000000000UL
+    ,10000000000000UL
+    ,100000000000000UL
+    ,1000000000000000UL
+    ,10000000000000000UL
+    ,100000000000000000UL
+    ,1000000000000000000UL
+    ,10000000000000000000UL
+#endif
+  };
   return pw10[n];
 }
 
@@ -415,7 +443,7 @@ real_0_digits(long n) {
 }
 
 static GEN
-real_read(pari_sp av, const char **s, GEN y, long PREC)
+real_read(pari_sp av, const char **s, GEN y, long prec)
 {
   long l, n = 0;
   switch(**s)
@@ -424,7 +452,7 @@ real_read(pari_sp av, const char **s, GEN y, long PREC)
     case '.':
     {
       const char *old = ++*s;
-      if (isalpha((int)**s))
+      if (isalpha((int)**s) || **s=='.')
       {
         if (**s == 'E' || **s == 'e') {
           n = exponent(s);
@@ -437,7 +465,7 @@ real_read(pari_sp av, const char **s, GEN y, long PREC)
       n = old - *s;
       if (**s != 'E' && **s != 'e')
       {
-        if (!signe(y)) { avma = av; return real_0(PREC); }
+        if (!signe(y)) { avma = av; return real_0(prec); }
         break;
       }
     }
@@ -446,14 +474,16 @@ real_read(pari_sp av, const char **s, GEN y, long PREC)
       n += exponent(s);
       if (!signe(y)) { avma = av; return real_0_digits(n); }
   }
-  l = lgefint(y); if (l < (long)PREC) l = (long)PREC;
-  if (!n) return itor(y, l);
-  y = itor(y, l+1);
+  l = nbits2prec(bit_accuracy(lgefint(y)));
+  if (l < prec) l = prec; else prec = l;
+  if (!n) return itor(y, prec);
+  incrprec(l);
+  y = itor(y, l);
   if (n > 0)
-    y = mulrr(y, rpowuu(10UL, (ulong)n, l+1));
+    y = mulrr(y, rpowuu(10UL, (ulong)n, l));
   else
-    y = divrr(y, rpowuu(10UL, (ulong)-n, l+1));
-  return gerepileuptoleaf(av, rtor(y, l));
+    y = divrr(y, rpowuu(10UL, (ulong)-n, l));
+  return gerepileuptoleaf(av, rtor(y, prec));
 }
 
 static GEN
@@ -461,7 +491,7 @@ int_read(const char **s)
 {
   int nb;
   GEN y = utoi(number(&nb, s));
-  if (nb == 9) y = int_read_more(y, s);
+  if (nb == MAX_DIGITS) y = int_read_more(y, s);
   return y;
 }
 
@@ -509,6 +539,7 @@ skipconstante(char **lex)
   if (**lex=='.')
   {
     char *old = ++*lex;
+    if (**lex == '.') { --*lex; return KINTEGER; }
     if (isalpha((int)**lex))
     {
       skipexponent(lex);
@@ -569,7 +600,12 @@ pari_lex(union token_value *yylval, struct node_loc *yylloc, char **lex)
   }
   if (**lex == '.')
   {
-    int token=skipconstante(lex);
+    int token;
+    if ((*lex)[1]== '.')
+    {
+      *lex+=2; yylloc->end = *lex; return KDOTDOT;
+    }
+    token=skipconstante(lex);
     if (token==KREAL)
     {
       yylloc->end = *lex;
@@ -654,7 +690,7 @@ pari_lex(union token_value *yylval, struct node_loc *yylloc, char **lex)
       *lex+=2; yylloc->end = *lex; return KSL;
     }
   yylloc->end = *lex+1;
-  return *(*lex)++;
+  return (unsigned char) *(*lex)++;
 }
 
 /********************************************************************/
@@ -710,12 +746,8 @@ static entree *
 findentry(const char *name, long len, entree *ep1)
 {
   entree *ep;
-
   for (ep = ep1; ep; ep = ep->next)
     if (!strncmp(ep->name, name, len) && !(ep->name)[len]) return ep;
-
-  if (foreignAutoload) /* Try to autoload. */
-    return foreignAutoload(name,len);
   return NULL; /* not found */
 }
 
@@ -778,7 +810,7 @@ long pari_var_next_temp(void) { return max_avail; }
 static long
 pari_var_pop(long v)
 {
-  if (v != nvar-1) pari_err(talker,"can't pop user variable %ld", v);
+  if (v != nvar-1) pari_err(e_MISC,"can't pop user variable %ld", v);
   return --nvar;
 }
 void
@@ -787,7 +819,7 @@ pari_var_create(entree *ep)
   GEN p = (GEN)initial_value(ep);
   long v;
   if (*p) return;
-  if (nvar == max_avail) pari_err(talker,"no more variables available");
+  if (nvar == max_avail) pari_err(e_MISC,"no more variables available");
   v = nvar++;
   /* set p = pol_x(v) */
   p[0] = evaltyp(t_POL) | _evallg(4);
@@ -806,7 +838,7 @@ delete_var(void)
 long
 fetch_var(void)
 {
-  if (nvar == max_avail) pari_err(talker,"no more variables available");
+  if (nvar == max_avail) pari_err(e_MISC,"no more variables available");
   return max_avail--;
 }
 
@@ -822,9 +854,9 @@ manage_var(long n, entree *ep)
       case manage_var_delete: return delete_var();
       case manage_var_create:
         pari_var_create(ep);
-        return varn(initial_value(ep));
+        return varn((GEN)initial_value(ep));
   }
-  pari_err(talker, "panic");
+  pari_err(e_MISC, "panic");
   return -1; /* not reached */
 }
 
@@ -838,7 +870,7 @@ fetch_named_var(const char *s)
   {
     case EpVAR: return ep;
     case EpNEW: break;
-    default: pari_err(talker, "%s already exists with incompatible valence", s);
+    default: pari_err(e_MISC, "%s already exists with incompatible valence", s);
   }
   pari_var_create(ep);
   ep->valence=EpVAR;
@@ -849,7 +881,7 @@ fetch_named_var(const char *s)
 long
 fetch_user_var(const char *s)
 {
-  return varn( initial_value(fetch_named_var(s)) );
+  return varn((GEN)initial_value(fetch_named_var(s)) );
 }
 
 GEN
@@ -871,9 +903,9 @@ name_var(long n, const char *s)
   char *u;
 
   if (n < pari_var_next())
-    pari_err(talker, "renaming a GP variable is forbidden");
+    pari_err(e_MISC, "renaming a GP variable is forbidden");
   if (n > (long)MAXVARN)
-    pari_err(talker, "variable number too big");
+    pari_err_OVERFLOW("variable number");
 
   ep = (entree*)pari_malloc(sizeof(entree) + strlen(s) + 1);
   u = (char *)initial_value(ep);
@@ -904,7 +936,7 @@ gpolvar(GEN x)
   }
   if (typ(x)==t_PADIC) return gcopy( gel(x,2) );
   v = gvar(x);
-  if (v==NO_VARIABLE) pari_err(typeer,"gpolvar");
+  if (v==NO_VARIABLE) return gen_0;
   return pol_x(v);
 }
 
@@ -954,7 +986,7 @@ alias0(const char *s, const char *old)
   ep = fetch_entry(old,strlen(old));
   e  = fetch_entry(s,strlen(s));
   if (EpVALENCE(e) != EpALIAS && EpVALENCE(e) != EpNEW)
-    pari_err(talker,"can't replace an existing symbol by an alias");
+    pari_err(e_MISC,"can't replace an existing symbol by an alias");
   freeep(e);
   x = newblock(2); x[0] = evaltyp(t_STR)|_evallg(2); /* for getheap */
   gel(x,1) = (GEN)ep;
@@ -983,6 +1015,18 @@ ifpari_void(GEN g, GEN a/*closure*/, GEN b/*closure*/)
   }
 }
 
+GEN
+ifpari_multi(GEN g, GEN a/*closure*/)
+{
+  long i, nb = lg(a)-1;
+  if (!gequal0(g)) /* false */
+    return closure_evalgen(gel(a,1));
+  for(i=2;i<nb;i+=2)
+    if (!gequal0(closure_evalgen(gel(a,i))))
+      return closure_evalgen(gel(a,i+1));
+  return i<=nb? closure_evalgen(gel(a,i)): gnil;
+}
+
 GEN
 andpari(GEN a, GEN b/*closure*/)
 {
@@ -1005,13 +1049,6 @@ orpari(GEN a, GEN b/*closure*/)
   return gequal0(g)?gen_0:gen_1;
 }
 
-GEN
-gstore(GEN *x, GEN y)
-{
-  *x=y;
-  return *x;
-}
-
 GEN gmule(GEN *x, GEN y)
 {
   *x=gmul(*x,y);
@@ -1098,14 +1135,14 @@ print_all_user_fun(int member)
   pari_sp av = avma;
   long iL = 0, lL = 1024;
   GEN L = cgetg(lL+1, t_VECSMALL);
-  const char *f;
   entree *ep;
   int i;
   for (i = 0; i < functions_tblsz; i++)
     for (ep = functions_hash[i]; ep; ep = ep->next)
     {
+      const char *f;
       int is_member;
-      if (EpVALENCE(ep) != EpVAR || typ(ep->value)!=t_CLOSURE) continue;
+      if (EpVALENCE(ep) != EpVAR || typ((GEN)ep->value)!=t_CLOSURE) continue;
       f = ep->name;
       is_member = (f[0] == '_' && f[1] == '.');
       if (member != is_member) continue;
@@ -1126,8 +1163,7 @@ print_all_user_fun(int member)
     for (i = 1; i <= iL; i++)
     {
       ep = (entree*)L[i];
-      f = ep->name;
-      pari_printf("%s =\n  %Ps\n\n", f, ep->value);
+      pari_printf("%s =\n  %Ps\n\n", ep->name, ep->value);
     }
   }
   avma = av;
