@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -38,12 +36,15 @@ static long
 sturm0(GEN x) {return sturm(x);}
 
 static GEN
-sigmak0(long k, GEN x) {return gsumdivk(x,k);}
+sigmak0(long k, GEN x) {return sumdivk(x,k);}
 
 static GEN
 lseriesell0(GEN a, GEN b, GEN c, GEN d, long prec)
   {(void)c; return elllseries(a,b,d,prec);}
 
+static GEN
+weipell(GEN e, long precdl) { return ellwpseries(e,0,precdl); }
+
 static long
 rounderror(GEN x)
 {
@@ -58,7 +59,7 @@ _factpol(GEN x, long t/*unused*/, long hint/*unused*/) {
 }
 
 static void
-suppressed(void) {pari_err(talker,"this function no longer exists");}
+suppressed(void) {pari_err(e_MISC,"this function no longer exists");}
 
 #define BUCH_PROTO "GDGDGD5,G,D1,G,D4,L,D3,L,p"
 #define B_ARGS GEN g1,GEN g2,GEN g3,GEN g4,GEN g5,long l1,long l2,long prec
@@ -168,7 +169,6 @@ entree oldfonctions[]={
 {"_.tate",0,(void*)member_tate,14,"G","_.tate"},
 {"_.tu",0,(void*)member_tu,14,"G","_.tu"},
 {"_.tufu",0,(void*)member_tufu,14,"G","_.tufu"},
-{"_.w",0,(void*)member_w,14,"G","_.w"},
 {"_.zk",0,(void*)member_zk,14,"G","_.zk"},
 {"_.zkst",0,(void*)member_zkst,14,"G","_.zkst"},
 {"_/=_",0,(void*)gdive,13,"&G","x/=y"},
@@ -178,11 +178,12 @@ entree oldfonctions[]={
 {"_<=_",0,(void*)gle,13,"GG","_<=_"},
 {"_<_",0,(void*)glt,13,"GG","_<_"},
 {"_==_",0,(void*)geq,13,"GG","_==_"},
-{"_=_",0,(void*)gstore,13,"&G","x=y: store value y in variable x."},
 {"_>=_",0,(void*)gge,13,"GG","_>=_"},
 {"_>>=_",0,(void*)gshiftre,13,"&L","_>>=_"},
 {"_>>_",0,(void*)gshift_right,13,"GL","x>>y"},
 {"_>_",0,(void*)ggt,13,"GG","_>_"},
+{"_[_.._,_.._]",0,(void*)matslice0,13,"GD0,L,D0,L,D0,L,D0,L,","x[a..b,c..d] = [x[a,c],  x[a+1,c],  ...,x[b,c];                      x[a,c+1],x[a+1,c+1],...,x[b,c+1];                        ...       ...          ...                      x[a,d],  x[a+1,d]  ,...,x[b,d]]"},
+{"_[_.._]",0,(void*)vecslice0,13,"GD0,L,L","x[a..b] = [x[a],x[a+1],...,x[b]]"},
 {"_\\/=_",0,(void*)gdivrounde,13,"&G","x\\\\/=y"},
 {"_\\/_",0,(void*)gdivround,13,"GG","x\\\\/y: rounded Euclidean quotient of x and y."},
 {"_\\=_",0,(void*)gdivente,13,"&G","x\\\\=y"},
@@ -192,6 +193,7 @@ entree oldfonctions[]={
 {"__",0,NULL,13,NULL,"__"},
 {"_derivfun",0,(void*)derivfun0,15,"GGp","_derivfun(closure,[args]) numerical derivation of closure with respect to the first variable at (args)."},
 {"_eval_mnemonic",0,(void*)eval_mnemonic,15,"lGs","Convert a mnemonic string to a flag."},
+{"_multi_if",0,(void*)ifpari_multi,15,"GE*","internal variant of if() that allows more than 3 arguments."},
 {"_void_if",0,(void*)ifpari_void,11,"vGDIDI","internal variant of if() that does not return a value."},
 {"_||_",0,(void*)orpari,13,"GE","x||y"},
 {"_~",0,(void*)gtrans,13,"G","_~"},
@@ -200,8 +202,8 @@ entree oldfonctions[]={
 {"Str",0,(void*)Str,2,"s*","Str({x}*): concatenates its (string) argument into a single string."},
 {"abs",1,(void*)gabs,3,"Gp","abs(x)=absolute value (or modulus) of x."},
 {"acos",1,(void*)gacos,3,"Gp","acos(x)=inverse cosine of x."},
-{"acosh",1,(void*)gach,3,"Gp","acosh(x)=inverse hyperbolic cosine of x."},
-{"addell",3,(void*)addell,5,"GGG","addell(e,z1,z2)=sum of the points z1 and z2 on elliptic curve e."},
+{"acosh",1,(void*)gacosh,3,"Gp","acosh(x)=inverse hyperbolic cosine of x."},
+{"addell",3,(void*)elladd,5,"GGG","addell(e,z1,z2)=sum of the points z1 and z2 on elliptic curve e."},
 {"addprimes",1,(void*)addprimes,4,"G","addprimes(x)=add primes in the vector x (with at most 20 components) to the prime table."},
 {"adj",1,(void*)adj,8,"G","adj(x)=adjoint matrix of x."},
 {"agm",2,(void*)agm,3,"GGp","agm(x,y)=arithmetic-geometric mean of x and y."},
@@ -215,19 +217,19 @@ entree oldfonctions[]={
 {"apprpadic",2,(void*)padicappr,7,"GG","apprpadic(x,a)=p-adic roots of the polynomial x congruent to a mod p."},
 {"arg",1,(void*)garg,3,"Gp","arg(x)=argument of x,such that -pi<arg(x)<=pi."},
 {"asin",1,(void*)gasin,3,"Gp","asin(x)=inverse sine of x."},
-{"asinh",1,(void*)gash,3,"Gp","asinh(x)=inverse hyperbolic sine of x."},
+{"asinh",1,(void*)gasinh,3,"Gp","asinh(x)=inverse hyperbolic sine of x."},
 {"assmat",1,(void*)matcompanion,8,"G","matcompanion(x)=associated matrix to polynomial x."},
 {"atan",1,(void*)gatan,3,"Gp","atan(x)=inverse tangent of x."},
-{"atanh",1,(void*)gath,3,"Gp","atanh(x)=inverse hyperbolic tangent of x."},
+{"atanh",1,(void*)gatanh,3,"Gp","atanh(x)=inverse hyperbolic tangent of x."},
 {"basis",13,(void*)nfbasis0,6,"GD0,L,DG","basis(x)=integral basis of the field Q[a], where a is a root of the polynomial x, using the round 4 algorithm."},
 {"basis2",13,(void*)nfbasis0,6,"GD2,L,DG","basis2(x)=integral basis of the field Q[a], where a is a root of the polynomial x, using the round 2 algorithm."},
 {"basistoalg",2,(void*)basistoalg,6,"GG","basistoalg(nf,x)=transforms the vertical vector x on the integral basis into an algebraic number."},
 {"bernreal",11,(void*)bernreal,3,"Lp","bernreal(x)=Bernoulli number B_x, as a real number with the current precision."},
 {"bernvec",11,(void*)bernvec,3,"L","bernvec(x)=Vector of rational Bernoulli numbers B_0, B_2,... up to B_(2x)."},
 {"bestappr",2,(void*)bestappr,4,"GGp","bestappr(x,k)=gives the best approximation to the real x with denominator less or equal to k."},
-{"bezout",2,(void*)vecbezout,4,"GG","bezout(x,y)=gives a 3-dimensional row vector [u,v,d] such that d=gcd(x,y) and u*x+v*y=d."},
-{"bezoutres",2,(void*)vecbezoutres,4,"GG","bezoutres(x,y)=gives a 3-dimensional row vector [u,v,d] such that d=resultant(x,y) and u*x+v*y=d, where x and y are polynomials."},
-{"bigomega",1,(void*)gbigomega,4,"G","bigomega(x)=number of repeated prime divisors of x."},
+{"bezout",2,(void*)gcdext0,4,"GG","bezout(x,y)=gives a 3-dimensional row vector [u,v,d] such that d=gcd(x,y) and u*x+v*y=d."},
+{"bezoutres",2,(void*)polresultantext,4,"GG","bezoutres(x,y)=gives a 3-dimensional row vector [u,v,d] such that d=resultant(x,y) and u*x+v*y=d, where x and y are polynomials."},
+{"bigomega",1,(void*)bigomega,4,"lG","bigomega(x)=number of repeated prime divisors of x."},
 {"bilhell",3,(void*)bilhell,5,"GGGp","bilhell(e,z1,z2)=canonical bilinear form for the points z1,z2 on the elliptic curve e. Either z1 or z2 can also be a vector/matrix of points."},
 {"bin",21,(void*)binomial,4,"GL","bin(x,y)=binomial coefficient x*(x-1)...*(x-y+1)/y! defined for y in Z and any x."},
 {"binary",1,(void*)binaire,2,"G","binary(x)=gives the vector formed by the binary digits of x (x C-integer)."},
@@ -276,11 +278,11 @@ entree oldfonctions[]={
 {"content",1,(void*)content,4,"G","content(x)=gcd of all the components of x, when this makes sense."},
 {"convol",2,(void*)convol,7,"GG","convol(x,y)=convolution (or Hadamard product) of two power series."},
 {"core",1,(void*)core,4,"G","core(n)=unique (positive of negative) squarefree integer d dividing n such that n/d is a square."},
-{"core2",1,(void*)core2,4,"G","core2(n)=(long)gen_2-component row vector [d,f], where d is the unique squarefree integer dividing n such that n/d=f^2 is a square."},
+{"core2",1,(void*)core2,4,"G","core2(n)=2-component row vector [d,f], where d is the unique squarefree integer dividing n such that n/d=f^2 is a square."},
 {"coredisc",1,(void*)coredisc,4,"G","coredisc(n)=discriminant of the quadratic field Q(sqrt(n))."},
-{"coredisc2",1,(void*)coredisc2,4,"G","coredisc2(n)=(long)gen_2-component row vector [d,f], where d is the discriminant of the quadratic field Q(sqrt(n)) and n=df^2. f may be a half integer."},
+{"coredisc2",1,(void*)coredisc2,4,"G","coredisc2(n)=2-component row vector [d,f], where d is the discriminant of the quadratic field Q(sqrt(n)) and n=df^2. f may be a half integer."},
 {"cos",1,(void*)gcos,3,"Gp","cos(x)=cosine of x."},
-{"cosh",1,(void*)gch,3,"Gp","cosh(x)=hyperbolic cosine of x."},
+{"cosh",1,(void*)gcosh,3,"Gp","cosh(x)=hyperbolic cosine of x."},
 {"cvtoi",13,(void*)gcvtoi,2,"Gf","cvtoi(x)=truncation of x, without taking into account loss of integer part precision."},
 {"cyclo",11,(void*)polcyclo,7,"LDn","cyclo(n)=n-th cyclotomic polynomial."},
 {"decodefactor",1,(void*)factorback,4,"G","decodefactor(fa)=given a factorisation fa, gives the factored object back."},
@@ -311,7 +313,7 @@ entree oldfonctions[]={
 {"discrayrelcond",62,(void*)bnrdisc0,6,"GD0,G,D0,G,D3,L,","discrayrelcond(bnr,subgroup)=relative [N,R1,rnfdisc] of the subfield of the ray class field bnr given by buchrayinit, defined by the HNF matrix subgroup. Result is zero if module is not the conductor."},
 {"divisors",1,(void*)divisors,4,"G","divisors(x)=gives a vector formed by the divisors of x in increasing order."},
 {"divres",2,(void*)gdiventres,1,"GG","divres(x,y)=euclidean division of x by y giving as a 2-dimensional column vector the quotient and the remainder."},
-{"divsum",22,(void*)divsum,9,"GVE","divsum(n,X,expr)=sum of expression expr, X running over the divisors of n."},
+{"divsum",22,(void*)sumdivexpr,9,"GVE","divsum(n,X,expr)=sum of expression expr, X running over the divisors of n."},
 {"eigen",1,(void*)eigen,8,"Gp","eigen(x)=eigenvectors of the matrix x given as columns of a matrix."},
 {"eint1",1,(void*)eint1,3,"Gp","eint1(x)=exponential integral E1(x)."},
 {"erfc",1,(void*)gerfc,3,"Gp","erfc(x)=complementary error function."},
@@ -348,7 +350,7 @@ entree oldfonctions[]={
 {"galoisconj",1,(void*)galoisconj,6,"GDG","galoisconj(nf)=list of conjugates of a root of the polynomial x=nf[1] in the same number field, using p-adics, LLL on integral basis (not always complete)."},
 {"galoisconj1",0,(void*)suppressed,6,"G","galoisconj1(nf)=list of conjugates of a root of the polynomial x=nf[1] in the same number field nf, using complex numbers, LLL on integral basis (not always complete)."},
 {"galoisconjforce",0,(void*)suppressed,6,"G","galoisconjforce(nf)=list of conjugates of a root of the polynomial x=nf[1] in the Galois number field nf, using p-adics, LLL on integral basis. Guaranteed to be complete if the field is Galois, otherwise there is an infinite loop."},
-{"gamh",1,(void*)ggamd,3,"Gp","gamh(x)=gamma of x+1/2 (x integer)."},
+{"gamh",1,(void*)ggammah,3,"Gp","gamh(x)=gamma of x+1/2 (x integer)."},
 {"gamma",1,(void*)ggamma,3,"Gp","gamma(x)=gamma function at x."},
 {"gauss",2,(void*)gauss,8,"GG","gauss(a,b)=gaussian solution of ax=b (a matrix,b vector)."},
 {"gaussmodulo",3,(void*)gaussmodulo,8,"GGG","gaussmodulo(M,D,Y)=(long)gen_1 solution of system of congruences MX=Y mod D."},
@@ -406,8 +408,8 @@ entree oldfonctions[]={
 {"idealnorm",2,(void*)idealnorm,6,"GG","idealnorm(nf,x)=norm of the ideal x in the number field nf."},
 {"idealpow",3,(void*)idealpow,6,"GGG","idealpow(nf,x,n)=n-th power of the ideal x in HNF in the number field nf."},
 {"idealpowred",3,(void*)idealpowred,6,"GGGp","idealpowred(nf,x,n)=reduced n-th power of the ideal x in HNF in the number field nf."},
-{"idealtwoelt",2,(void*)idealtwoelt,6,"GG","idealtwoelt(nf,x)=(long)gen_2-element representation of an ideal x in the number field nf."},
-{"idealtwoelt2",3,(void*)idealtwoelt2,6,"GGG","idealtwoelt2(nf,x,a)=(long)gen_2-element representation of an ideal x in the number field nf, with the first element equal to a."},
+{"idealtwoelt",2,(void*)idealtwoelt,6,"GG","idealtwoelt(nf,x)=2-element representation of an ideal x in the number field nf."},
+{"idealtwoelt2",3,(void*)idealtwoelt2,6,"GGG","idealtwoelt2(nf,x,a)=2-element representation of an ideal x in the number field nf, with the first element equal to a."},
 {"idealval",30,(void*)idealval,6,"lGGG","idealval(nf,x,p)=valuation at p given in primedec format of the ideal x in the number field nf."},
 {"idmat",11,(void*)matid,8,"L","idmat(n)=identity matrix of order n (n C-integer)."},
 {"if",0,(void*)ifpari,11,"GDEDE","if(a,seq1,seq2): if a is nonzero, seq1 is evaluated, otherwise seq2. seq1 and seq2 are optional, and if seq2 is omitted, the preceding comma can be omitted also."},
@@ -425,7 +427,7 @@ entree oldfonctions[]={
 {"initalg",1,(void*)nfinit,6,"Gp","initalg(x)=x being a nonconstant irreducible polynomial, gives the vector: [x,[r1,r2],nfdisc,index,[M,MC,T2,T,different] (see manual),r1+r2 first roots, integral basis, matrix of power basis in terms of integral basis, multiplication table of basis]."},
 {"initalgred",1,(void*)nfinitred,6,"Gp","initalgred(x)=x being a nonconstant irreducible polynomial, finds (using polred) a simpler polynomial pol defining the same number field, and gives the vector: [pol,[r1,r2],nfdisc,index,[M,MC,T2,T,different] (see manual), r1+r2 first roots, integral basis, matrix of power basis in terms of integral basis, multiplication table of basis]."},
 {"initalgred2",1,(void*)nfinitred2,6,"Gp","initalgred2(P)=P being a nonconstant irreducible polynomial, gives a two-element vector [nf,mod(a,pol)], where nf is as output by nfinitred and mod(a,pol) is a polymod equal to mod(x,P) and pol=nf[1]."},
-{"initell",1,(void*)ellinit,5,"Gp","initell(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector: [a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j,[e1,e2,e3],w1,w2,eta1,eta2,q,area]."},
+{"initell",1,(void*)ellinit,5,"GDGp","initell(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector: [a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j,[e1,e2,e3],w1,w2,eta1,eta2,q,area]."},
 {"initzeta",1,(void*)initzeta,6,"Gp","initzeta(x)=compute number field information necessary to use zetak, where x is an irreducible polynomial."},
 {"integ",14,(void*)integ,7,"Gn","integ(x,y)=formal integration of x with respect to the main variable of y."},
 {"intersect",2,(void*)intersect,8,"GG","intersect(x,y)=intersection of the vector spaces whose bases are the columns of x and y."},
@@ -435,11 +437,11 @@ entree oldfonctions[]={
 {"intopen",99,(void*)intnumromb0,9,"V=GGED3,L,p","intopen(X=a,b,s)=numerical integration of s from a to b with respect to X, where s has only limits at a or b."},
 {"inverseimage",2,(void*)inverseimage,8,"GG","inverseimage(x,y)=an element of the inverse image of the vector y by the matrix x if one exists, the empty vector otherwise."},
 {"isdiagonal",10,(void*)isdiagonal,8,"lG","isdiagonal(x)=true(1) if x is a diagonal matrix, false(0) otherwise."},
-{"isfund",1,(void*)gisfundamental,4,"G","isfund(x)=true(1) if x is a fundamental discriminant (including 1), false(0) if not."},
+{"isfund",1,(void*)isfundamental,4,"lG","isfund(x)=true(1) if x is a fundamental discriminant (including 1), false(0) if not."},
 {"isideal",20,(void*)isideal,6,"lGG","isideal(nf,x)=true(1) if x is an ideal in the number field nf, false(0) if not."},
 {"isincl",2,(void*)nfisincl,6,"GG","isincl(x,y)=tests whether the number field defined by the polynomial x is isomorphic to a subfield of the one defined by y; 0 if not, otherwise all the isomorphisms."},
 {"isinclfast",2,(void*)nfisincl,6,"GG","isinclfast(nf1,nf2)=tests whether the number nf1 is isomorphic to a subfield of nf2 or not. If it gives a non-zero result, this proves that this is the case. However if it gives zero, nf1 may still be isomorphic to a subfield of nf2 so you have to use the much slower isincl to be sure."},
-{"isirreducible",1,(void*)gisirreducible,7,"G","isirreducible(x)=true(1) if x is an irreducible non-constant polynomial, false(0) if x is reducible or constant."},
+{"isirreducible",1,(void*)isirreducible,7,"lG","isirreducible(x)=true(1) if x is an irreducible non-constant polynomial, false(0) if x is reducible or constant."},
 {"isisom",2,(void*)nfisisom,6,"GG","isisom(x,y)=tests whether the number field defined by the polynomial x is isomorphic to the one defined by y; 0 if not, otherwise all the isomorphisms."},
 {"isisomfast",2,(void*)nfisisom,6,"GG","isisomfast(nf1,nf2)=tests whether the number fields nf1 and nf2 are isomorphic or not. If it gives a non-zero result, this proves that they are isomorphic. However if it gives zero, nf1 and nf2 may still be isomorphic so you have to use the much slower isisom to be sure."},
 {"isoncurve",20,(void*)oncurve,5,"iGG","isoncurve(e,x)=true(1) if x is on elliptic curve e, false(0) if not."},
@@ -453,8 +455,8 @@ entree oldfonctions[]={
 {"ispsp",1,(void*)gispseudoprime,4,"GD1,L,","ispsp(x)=true(1) if x is a strong pseudoprime, false(0) if not."},
 {"isqrt",1,(void*)sqrtint,4,"G","isqrt(x)=integer square root of x (x integer)."},
 {"isset",10,(void*)setisset,8,"lG","isset(x)=true(1) if x is a set (row vector with strictly increasing entries), false(0) if not."},
-{"issqfree",1,(void*)gissquarefree,4,"G","issqfree(x)=true(1) if x is squarefree, false(0) if not."},
-{"issquare",1,(void*)gissquare,4,"G","issquare(x)=true(1) if x is a square, false(0) if not."},
+{"issqfree",1,(void*)issquarefree,4,"lG","issqfree(x)=true(1) if x is squarefree, false(0) if not."},
+{"issquare",1,(void*)issquare,4,"lG","issquare(x)=true(1) if x is a square, false(0) if not."},
 {"isunit",2,(void*)bnfisunit,6,"GG","isunit(bnf,x)=bnf being output by buchinit, gives the vector of exponents of x on the fundamental units and the roots of unity if x is a unit, the empty vector otherwise."},
 {"jacobi",1,(void*)jacobi,8,"Gp","jacobi(x)=eigenvalues and orthogonal matrix of eigenvectors of the real symmetric matrix x."},
 {"jbesselh",2,(void*)jbesselh,3,"GGp","jbesselh(n,x)=J-bessel function of index n+1/2 and argument x, where n is a non-negative integer."},
@@ -467,7 +469,7 @@ entree oldfonctions[]={
 {"kerint",1,(void*)matkerint0,8,"GD0,L,","kerint(x)=LLL-reduced Z-basis of the kernel of the matrix x with integral entries using a modified LLL."},
 {"kerint1",1,(void*)matkerint0,8,"GD1,L,","kerint1(x)=LLL-reduced Z-basis of the kernel of the matrix x with rational entries using matrixqz3 and the HNF."},
 {"kerint2",1,(void*)suppressed,8,"G","kerint2(x)=LLL-reduced Z-basis of the kernel of the matrix x with integral entries using a modified LLL."},
-{"kro",2,(void*)gkronecker,4,"GG","kro(x,y)=kronecker symbol (x/y)."},
+{"kro",2,(void*)kronecker,4,"lGG","kro(x,y)=kronecker symbol (x/y)."},
 {"label",0,(void*)suppressed,11,"s*","label(n)=THIS FUNCTION HAS BEEN SUPPRESSED."},
 {"lambdak",2,(void*)glambdak,6,"GGp","lambdak(nfz,s)=Dedekind lambda function of the number field nfz at s, where nfz is the vector computed by initzeta (NOT by nfinit)."},
 {"laplace",1,(void*)laplace,7,"G","laplace(x)=replaces the power series sum of a_n*x^n/n! by sum of a_n*x^n."},
@@ -517,7 +519,7 @@ entree oldfonctions[]={
 {"modp",25,(void*)gmodulo,2,"GG","modp(x,y)=creates the integer x modulo y as a permanent object (on the heap)."},
 {"modreverse",1,(void*)modreverse,6,"G","modreverse(x)=reverse polymod of the polymod x, if it exists."},
 {"modulargcd",2,(void*)QX_gcd,2,"GG","modulargcd(x,y)=gcd of the polynomials x and y using the modular method."},
-{"mu",1,(void*)gmoebius,4,"G","mu(x)=Moebius function of x."},
+{"mu",1,(void*)moebius,4,"lG","mu(x)=Moebius function of x."},
 {"newtonpoly",2,(void*)newtonpoly,6,"GG","newtonpoly(x,p)=Newton polygon of polynomial x with respect to the prime p."},
 {"nextprime",1,(void*)nextprime,4,"G","nextprime(x)=smallest prime number>=x."},
 {"nfdetint",2,(void*)nfdetint,6,"GG","nfdetint(nf,x)=multiple of the ideal determinant of the pseudo generating set x."},
@@ -535,22 +537,22 @@ entree oldfonctions[]={
 {"norm",1,(void*)gnorm,2,"G","norm(x)=norm of x."},
 {"norml2",1,(void*)gnorml2,2,"G","norml2(x)=square of the L2-norm of the vector x."},
 {"nucomp",3,(void*)nucomp,4,"GGG","nucomp(x,y,l)=composite of primitive positive definite quadratic forms x and y using nucomp and nudupl, where l=[|D/4|^(1/4)] is precomputed."},
-{"numdiv",1,(void*)numbdiv,4,"G","numdiv(x)=number of divisors of x."},
+{"numdiv",1,(void*)numdiv,4,"G","numdiv(x)=number of divisors of x."},
 {"numer",1,(void*)numer,2,"G","numer(x)=numerator of x."},
 {"nupow",2,(void*)nupow,4,"GG","nupow(x,n)=n-th power of primitive positive definite quadratic form x using nucomp and nudupl."},
 {"o",0,(void*)ggrando,7,"GD1,L,","o(a^b): p-adic or power series zero with precision given by b."},
-{"omega",1,(void*)gomega,4,"G","omega(x)=number of unrepeated prime divisors of x."},
+{"omega",1,(void*)omega,4,"lG","omega(x)=number of unrepeated prime divisors of x."},
 {"ordell",2,(void*)ellordinate,5,"GGp","ordell(e,x)=y-coordinates corresponding to x-ordinate x on elliptic curve e."},
 {"order",1,(void*)order,4,"G","order(x)=order of the integermod x in (Z/nZ)*."},
 {"orderell",2,(void*)orderell,5,"GG","orderell(e,p)=order of the point p on the elliptic curve e over Q, 0 if non-torsion."},
-{"ordred",1,(void*)ordred,6,"Gp","ordred(x)=reduction of the polynomial x, staying in the same order."},
+{"ordred",1,(void*)polredord,6,"Gp","ordred(x)=reduction of the polynomial x, staying in the same order."},
 {"padicprec",20,(void*)padicprec,2,"lGG","padicprec(x,p)=absolute p-adic precision of object x."},
 {"pascal",99,(void*)matqpascal,8,"LDG","pascal(n)=pascal triangle of order n (n C-integer)."},
 {"perf",10,(void*)perf,8,"G","perf(a)=rank of matrix of xx~ for x minimal vectors of a gram matrix a."},
 {"permutation",24,(void*)numtoperm,2,"LG","permutation(n,k)=permutation number k (mod n!) of n letters (n C-integer)."},
 {"permutation2num",1,(void*)permtonum,2,"G","permutation2num(vect)=ordinal (between 1 and n!) of permutation vect."},
 {"pf",2,(void*)primeform,4,"GGp","pf(x,p)=returns the prime form whose first coefficient is p, of discriminant x."},
-{"phi",1,(void*)geulerphi,4,"G","phi(x)=Euler's totient function of x."},
+{"phi",1,(void*)eulerphi,4,"G","phi(x)=Euler's totient function of x."},
 {"pi",0,(void*)mppi,3,"p","pi=pi()=the constant pi, with current precision."},
 {"pnqn",1,(void*)pnqn,4,"G","pnqn(x)=[p_n,p_{n-1};q_n,q_{n-1}] corresponding to the continued fraction x."},
 {"pointell",2,(void*)pointell,5,"GGp","pointell(e,z)=coordinates of point on the curve e corresponding to the complex number z."},
@@ -571,14 +573,14 @@ entree oldfonctions[]={
 {"polylogp",24,(void*)polylogp,3,"LGp","polylogp(m,x)=P_m-modified m-th polylog of x."},
 {"polyrev",14,(void*)gtopolyrev,2,"Gn","polyrev(x,v)=convert x (usually a vector or a power series) into a polynomial with variable v, starting with the constant term."},
 {"polzag",21,(void*)polzag,9,"LL","polzag(n,m)=Zagier's polynomials of index n,m."},
-{"powell",3,(void*)powell,5,"GGGp","powell(e,x,n)=n times the point x on elliptic curve e (n in Z)."},
+{"powell",3,(void*)ellmul,5,"GGGp","powell(e,x,n)=n times the point x on elliptic curve e (n in Z)."},
 {"powrealraw",23,(void*)qfrpowraw,4,"GL","powrealraw(x,n)=n-th power without reduction of the binary quadratic form x of positive discriminant."},
 {"prec",21,(void*)gprec,2,"GL","prec(x,n)=change the precision of x to be n (n C-integer)."},
 {"precision",1,(void*)precision0,2,"GD0,L,","precision(x)=real precision of object x."},
 {"prime",11,(void*)prime,4,"L","prime(n)=returns the n-th prime (n C-integer)."},
 {"primedec",2,(void*)idealprimedec,6,"GG","primedec(nf,p)=prime ideal decomposition of the prime number p in the number field nf as a vector of 5 component vectors [p,a,e,f,b] representing the prime ideals pZ_K+a.Z_K, e,f as usual, a as vector of components on the  integral basis, b Lenstra's constant."},
 {"primes",11,(void*)primes,4,"L","primes(n)=returns the vector of the first n primes (n C-integer)."},
-{"primroot",1,(void*)znprimroot0,4,"G","primroot(n)=returns a primitive root of n when it exists."},
+{"primroot",1,(void*)znprimroot,4,"G","primroot(n)=returns a primitive root of n when it exists."},
 {"principalideal",2,(void*)principalideal,6,"GG","principalideal(nf,x)=returns the principal ideal generated by the algebraic number x in the number field nf."},
 {"principalidele",2,(void*)suppressed,6,"GG","principalidele(nf,x)=returns the principal idele generated by the algebraic number x in the number field nf."},
 {"prod",48,(void*)prod0,9,"GV=GGE","prod(x,X=a,b,expr)=x times the product (X runs from a to b) of expression."},
@@ -605,7 +607,7 @@ entree oldfonctions[]={
 {"reorder",0,(void*)suppressed,11,"G","reorder(x)=THIS FUNCTION HAS BEEN SUPPRESSED."},
 {"resultant",2,(void*)resultant_all,7,"GGD&","resultant(x,y)=resultant of the polynomials x and y with exact entries."},
 {"resultant2",2,(void*)resultant2,7,"GG","resultant2(x,y)=resultant of the polynomials x and y."},
-{"reverse",1,(void*)recip,7,"G","reverse(x)=reversion of the power series x."},
+{"reverse",1,(void*)serreverse,7,"G","reverse(x)=reversion of the power series x."},
 {"rhoreal",1,(void*)rhoreal,4,"G","rhoreal(x)=single reduction step of the binary quadratic form x of positive discriminant."},
 {"rhorealnod",2,(void*)rhorealnod,4,"GG","rhorealnod(x,sq)=single reduction step of the binary quadratic form x with D>0 without distance function where sq=[sqrt D]."},
 {"rndtoi",13,(void*)grndtoi,2,"Gf","rndtoi(x)=take the nearest integer to all the coefficients of x, without taking into account loss of integer part precision."},
@@ -624,7 +626,7 @@ entree oldfonctions[]={
 {"rootpadic",32,(void*)rootpadic,7,"GGL","rootpadic(x,p,r)=p-adic roots of the polynomial x to precision r."},
 {"roots",1,(void*)roots,7,"Gp","roots(x)=roots of the polynomial x using Schonhage's method modified by Gourdon."},
 {"rootsof1",1,(void*)rootsof1,6,"G","rootsof1(nf)=number of roots of unity and primitive root of unity in the number field nf."},
-{"rootsold",1,(void*)roots0,7,"GD1,L,p","rootsold(x)=roots of the polynomial x using a modified Newton's method."},
+{"rootsold",0,(void*)suppressed,11,"G","rootsold(x)=THIS FUNCTION HAS BEEN SUPPRESSED."},
 {"round",1,(void*)ground,2,"G","round(x)=take the nearest integer to all the coefficients of x."},
 {"rounderror",10,(void*)rounderror,2,"lG","rounderror(x)=maximum error found in rounding x."},
 {"series",14,(void*)gtoser,2,"Gn","series(x,v)=convert x (usually a vector) into a power series with variable v, starting with the constant coefficient."},
@@ -644,13 +646,13 @@ entree oldfonctions[]={
 {"simplefactmod",2,(void*)simplefactmod,4,"GG","simplefactmod(x,p)=same as factmod except that only the degrees of the irreducible factors are given."},
 {"simplify",1,(void*)simplify,2,"G","simplify(x)=simplify the object x as much as possible."},
 {"sin",1,(void*)gsin,3,"Gp","sin(x)=sine of x."},
-{"sinh",1,(void*)gsh,3,"Gp","sinh(x)=hyperbolic sine of x."},
+{"sinh",1,(void*)gsinh,3,"Gp","sinh(x)=hyperbolic sine of x."},
 {"size",10,(void*)sizedigit,2,"lG","size(x)=maximum number of decimal digits minus one of (the coefficients of) x."},
 {"smallbasis",13,(void*)nfbasis0,6,"GD1,L,DG","smallbasis(x)=integral basis of the field Q[a], where a is a root of the polynomial x where one assumes that no square of a prime>primelimit divides the discriminant of x."},
 {"smallbuchinit",93,(void*)smallbuchinit,6,BUCH_PROTO,"smallbuchinit(pol)=small buchinit, which can be converted to a big one using makebigbnf."},
 {"smalldiscf",1,(void*)nfdisc0,6,"GD1,L,DG","smalldiscf(x)=discriminant of the number field defined by the polynomial x where one assumes that no square of a prime>primelimit divides the discriminant of x."},
 {"smallfact",1,(void*)boundfact,4,"GD0,L,","smallfact(x)=partial factorization of the integer x (using only the stored primes)."},
-{"smallinitell",1,(void*)smallellinit,5,"Gp","smallinitell(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector: [a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j]."},
+{"smallinitell",1,(void*)ellinit,5,"GDGp","smallinitell(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector: [a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j]."},
 {"smallpolred",1,(void*)smallpolred,6,"G","smallpolred(x)=partial reduction of the polynomial x (gives minimal polynomials only)."},
 {"smallpolred2",1,(void*)smallpolred2,6,"G","smallpolred2(x)=partial reduction of the polynomial x (gives elements and minimal polynomials)."},
 {"smith",1,(void*)smith,8,"G","smith(x)=Smith normal form (i.e. elementary divisors) of the matrix x, expressed as a vector."},
@@ -666,7 +668,7 @@ entree oldfonctions[]={
 {"sturm",10,(void*)sturm0,7,"lG","sturm(x)=number of real roots of the polynomial x."},
 {"sturmpart",30,(void*)sturmpart,7,"lGGG","sturmpart(x,a,b)=number of real roots of the polynomial x in the interval (a,b]."},
 {"subcyclo",2,(void*)polsubcyclo,6,"LLDn","subcyclo(p,d)=finds an equation for the d-th degree subfield of Q(zeta_p), where p must be a prime power."},
-{"subell",3,(void*)subell,5,"GGGp","subell(e,z1,z2)=difference of the points z1 and z2 on elliptic curve e."},
+{"subell",3,(void*)ellsub,5,"GGGp","subell(e,z1,z2)=difference of the points z1 and z2 on elliptic curve e."},
 {"subst",26,(void*)gsubst,7,"GnG","subst(x,y,z)=in expression x, replace the variable y by the expression z."},
 {"sum",48,(void*)sum0,9,"GV=GGE","sum(x,X=a,b,expr)=x plus the sum (X goes from a to b) of expression expr."},
 {"sumalt",27,(void*)sumalt0,9,"V=GED0,L,p","sumalt(X=a,expr)=Villegas-Zagier's acceleration of alternating series expr, X starting at a."},
@@ -677,7 +679,7 @@ entree oldfonctions[]={
 {"supplement",1,(void*)suppl,8,"G","supplement(x)=supplement the columns of the matrix x to an invertible matrix."},
 {"sylvestermatrix",2,(void*)sylvestermatrix,7,"GG","sylvestermatrix(x,y)=forms the sylvester matrix associated to the two polynomials x and y. Warning: the polynomial coefficients are in columns, not in rows."},
 {"tan",1,(void*)gtan,3,"Gp","tan(x)=tangent of x."},
-{"tanh",1,(void*)gth,3,"Gp","tanh(x)=hyperbolic tangent of x."},
+{"tanh",1,(void*)gtanh,3,"Gp","tanh(x)=hyperbolic tangent of x."},
 {"taniyama",1,(void*)elltaniyama,5,"GP","taniyama(e)=modular parametrization of elliptic curve e."},
 {"taylor",12,(void*)tayl,7,"GnP","taylor(x,y)=taylor expansion of x with respect to the main variable of y."},
 {"tchebi",11,(void*)polchebyshev,7,"LD1,L,Dn","tchebi(n)=Tchebitcheff polynomial of degree n (n C-integer)."},
@@ -694,7 +696,7 @@ entree oldfonctions[]={
 {"twototwo",0,(void*)suppressed,6,"GGG","twototwo(nf,a,b)=returns a 3-component vector [d,e,U] such that U is a unimodular 2x2 matrix with algebraic integer coefficients such that [a,b]*U=[d,e] and d,e are hopefully smaller."},
 {"unit",1,(void*)quadunit,4,"G","unit(x)=fundamental unit of the quadratic field of discriminant x where x must be positive."},
 {"until",0,(void*)untilpari,11,"vEI","until(a,seq): evaluate the expression sequence seq until a is nonzero."},
-{"valuation",20,(void*)ggval,2,"lGG","valuation(x,p)=valuation of x with respect to p."},
+{"valuation",20,(void*)gvaluation,2,"lGG","valuation(x,p)=valuation of x with respect to p."},
 {"vec",1,(void*)gtovec,2,"G","vec(x)=transforms the object x into a vector. Used mainly if x is a polynomial or a power series."},
 {"vecindexsort",2,(void*)indexsort,8,"G","vecindexsort(x): indirect sorting of the vector x."},
 {"veclexsort",2,(void*)lexsort,8,"G","veclexsort(x): sort the elements of the vector x in ascending lexicographic order."},
