@@ -1,5 +1,4 @@
-/*
-Copyright (C) 2006  The PARI group.
+/* Copyright (C) 2006  The PARI group.
 
 This file is part of the PARI package.
 
@@ -19,12 +18,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 /********************************************************************/
 /*                                                                  */
-/*         break/next/return/allocatemem handling                   */
+/*                   break/next/return handling                     */
 /*                                                                  */
 /********************************************************************/
 
 static THREAD long br_status, br_count;
-enum { br_NONE = 0, br_BREAK, br_NEXT, br_MULTINEXT, br_RETURN };
 static THREAD GEN br_res;
 
 long
@@ -53,16 +51,15 @@ GEN
 return0(GEN x)
 {
   GEN y = br_res;
-  br_res = (x && x != gnil)? gclone(x): NULL;
-  if (y) gunclone(y);
+  br_res = (x && x != gnil)? gcloneref(x): NULL;
+  if (y) gunclone_deep(y);
   br_status = br_RETURN; return NULL;
 }
 
 GEN
 next0(long n)
 {
-  if (n < 1)
-    pari_err(talker,"positive integer expected in next");
+  if (n < 1) pari_err_DOMAIN("next", "n", "<", gen_1, stoi(n));
   if (n == 1) br_status = br_NEXT;
   else
   {
@@ -75,8 +72,7 @@ next0(long n)
 GEN
 break0(long n)
 {
-  if (n < 1)
-    pari_err(talker,"positive integer expected in break");
+  if (n < 1) pari_err_DOMAIN("break", "n", "<", gen_1, stoi(n));
   br_count = n;
   br_status = br_BREAK; return NULL;
 }
@@ -140,9 +136,6 @@ pop_val(entree *ep)
 void
 freeep(entree *ep)
 {
-  if (foreignFuncFree && ep->code && (*ep->code == 'x'))
-    (*foreignFuncFree)(ep); /* function created by foreign interpreter */
-
   if (EpSTATIC(ep)) return; /* gp function loaded at init time */
   if (ep->help) {pari_free((void*)ep->help); ep->help=NULL;}
   if (ep->code) {pari_free((void*)ep->code); ep->code=NULL;}
@@ -218,11 +211,13 @@ copyvalue(entree *ep)
 }
 
 INLINE void
-err_var(void) { pari_err(talker, "variable name expected"); }
+err_var(GEN x) { pari_err_TYPE("evaluator [variable name expected]", x); }
 
 INLINE void
 checkvalue(entree *ep)
 {
+  if (MT_IS_THREAD)
+    pari_err(e_MISC,"mt: global variable not supported: %s",ep->name);
   if (ep->valence==EpNEW)
   {
     pari_var_create(ep);
@@ -230,7 +225,7 @@ checkvalue(entree *ep)
     ep->value = initial_value(ep);
   }
   else if (ep->valence!=EpVAR)
-    err_var();
+    err_var(strtoGENstr(ep->name));
 }
 
 /* make GP variables safe for avma = top */
@@ -252,15 +247,48 @@ lvar_make_safe(void)
 }
 
 static void
-check_array_index(long c, long max)
+check_array_index(long c, long l)
 {
-  if (c < 1 || c >= max)
-  {
-    if (max <= 2)
-      pari_err(talker,"array index (%ld) out of allowed range [%s]",c,max==1?"none":"1");
-    else
-      pari_err(talker,"array index (%ld) out of allowed range [1-%ld]",c,max-1);
-  }
+  if (c < 1) pari_err_COMPONENT("", "<", gen_1, stoi(c));
+  if (c >= l) pari_err_COMPONENT("", ">", stoi(l-1), stoi(c));
+}
+
+GEN*
+safegel(GEN x, long l)
+{
+  if (!is_matvec_t(typ(x)))
+    pari_err_TYPE("safegel",x);
+  check_array_index(l, lg(x));
+  return &(gel(x,l));
+}
+
+GEN*
+safelistel(GEN x, long l)
+{
+  GEN d;
+  if (typ(x)!=t_LIST)
+    pari_err_TYPE("safelistel",x);
+  d = list_data(x);
+  check_array_index(l, lg(d));
+  return &(gel(d,l));
+}
+
+long*
+safeel(GEN x, long l)
+{
+  if (typ(x)!=t_VECSMALL)
+    pari_err_TYPE("safeel",x);
+  check_array_index(l, lg(x));
+  return &(x[l]);
+}
+
+GEN*
+safegcoeff(GEN x, long a, long b)
+{
+  if (typ(x)!=t_MAT) pari_err_TYPE("safegcoeff", x);
+  check_array_index(b, lg(x));
+  check_array_index(a, lg(gel(x,b)));
+  return &(gcoeff(x,a,b));
 }
 
 typedef struct matcomp
@@ -273,7 +301,7 @@ typedef struct matcomp
 typedef struct gp_pointer
 {
   matcomp c;
-  GEN x;
+  GEN x, ox;
   entree *ep;
   long vn;
   long sp;
@@ -290,14 +318,14 @@ change_compo(matcomp *c, GEN res)
   if (typ(p) == t_VECSMALL)
   {
     if (typ(res) != t_INT || is_bigint(res))
-      pari_err(talker,"not a suitable VECSMALL component");
+      pari_err_TYPE("t_VECSMALL assignment", res);
     *pt = (GEN)itos(res); return;
   }
   t = typ(res);
   if (c->full_row)
   {
-    if (t != t_VEC || lg(res) != lg(p))
-      pari_err(talker,"incorrect type or length in matrix assignment");
+    if (t != t_VEC) pari_err_TYPE("matrix row assignment", res);
+    if (lg(res) != lg(p)) pari_err_DIM("matrix row assignment");
     for (i=1; i<lg(p); i++)
     {
       GEN p1 = gcoeff(p,c->full_row,i); if (isclone(p1)) gunclone_deep(p1);
@@ -306,8 +334,10 @@ change_compo(matcomp *c, GEN res)
     return;
   }
   if (c->full_col)
-    if (t != t_COL || lg(res) != lg(*pt))
-      pari_err(talker,"incorrect type or length in matrix assignment");
+  {
+    if (t != t_COL) pari_err_TYPE("matrix col assignment", res);
+    if (lg(res) != lg(*pt)) pari_err_DIM("matrix col assignment");
+  }
 
   res = gclone(res);
   gunclone_deep(*pt);
@@ -328,11 +358,11 @@ struct var_lex
 
 struct trace
 {
-  long *pc;
+  long pc;
   GEN closure;
 };
 
-static THREAD long sp, rp;
+static THREAD long sp, rp, dbg_level;
 static THREAD long *st;
 static THREAD gp_pointer *ptrs;
 static THREAD entree **lvars;
@@ -400,25 +430,26 @@ restore_trace(long nbtrace)
   s_trace.n-=nbtrace;
 }
 
-INLINE void
-trace_push(long *pc, GEN C)
+INLINE long
+trace_push(long pc, GEN C)
 {
   long tr;
   BLOCK_SIGINT_START
-  tr = stack_new(&s_trace);
+  tr = pari_stack_new(&s_trace);
   trace[tr].pc = pc;
   trace[tr].closure = C;
   BLOCK_SIGINT_END
+  return tr;
 }
 
 void
 push_lex(GEN a, GEN C)
 {
-  long vn=stack_new(&s_var);
+  long vn=pari_stack_new(&s_var);
   struct var_lex *v=var+vn;
   v->flag  = PUSH_VAL;
   v->value = a;
-  if (C) trace_push(NULL, C);
+  if (C) (void) trace_push(-1, C);
 }
 
 GEN
@@ -446,30 +477,34 @@ pop_lex(long n)
   s_trace.n--;
 }
 
+static pari_stack s_relocs;
+static entree **relocs;
+
 void
 pari_init_evaluator(void)
 {
   sp=0;
-  stack_init(&s_st,sizeof(*st),(void**)&st);
-  stack_alloc(&s_st,32);
+  pari_stack_init(&s_st,sizeof(*st),(void**)&st);
+  pari_stack_alloc(&s_st,32);
   s_st.n=s_st.alloc;
   rp=0;
-  stack_init(&s_ptrs,sizeof(*ptrs),(void**)&ptrs);
-  stack_alloc(&s_ptrs,16);
+  pari_stack_init(&s_ptrs,sizeof(*ptrs),(void**)&ptrs);
+  pari_stack_alloc(&s_ptrs,16);
   s_ptrs.n=s_ptrs.alloc;
-  stack_init(&s_var,sizeof(*var),(void**)&var);
-  stack_init(&s_lvars,sizeof(*lvars),(void**)&lvars);
-  stack_init(&s_trace,sizeof(*trace),(void**)&trace);
+  pari_stack_init(&s_var,sizeof(*var),(void**)&var);
+  pari_stack_init(&s_lvars,sizeof(*lvars),(void**)&lvars);
+  pari_stack_init(&s_trace,sizeof(*trace),(void**)&trace);
   br_res = NULL;
+  pari_stack_init(&s_relocs,sizeof(*relocs),(void**)&relocs);
 }
 void
 pari_close_evaluator(void)
 {
-  stack_delete(&s_st);
-  stack_delete(&s_ptrs);
-  stack_delete(&s_var);
-  stack_delete(&s_lvars);
-  stack_delete(&s_trace);
+  pari_stack_delete(&s_st);
+  pari_stack_delete(&s_ptrs);
+  pari_stack_delete(&s_var);
+  pari_stack_delete(&s_lvars);
+  pari_stack_delete(&s_trace);
 }
 
 static gp_pointer *
@@ -479,7 +514,7 @@ new_ptr(void)
   {
     long i;
     gp_pointer *old = ptrs;
-    (void)stack_new(&s_ptrs);
+    (void)pari_stack_new(&s_ptrs);
     if (old != ptrs)
       for(i=0; i<rp; i++)
       {
@@ -536,7 +571,7 @@ INLINE long
 closure_varn(GEN x)
 {
   if (!x) return -1;
-  if (!gcmpX(x)) err_var();
+  if (!gequalX(x)) err_var(x);
   return varn(x);
 }
 
@@ -557,7 +592,7 @@ closure_castgen(GEN z, long mode)
   case Gvoid:
     break;
   default:
-    pari_err(bugparier,"closure_castgen, type unknown");
+    pari_err_BUG("closure_castgen, type unknown");
   }
 }
 
@@ -573,9 +608,9 @@ closure_castlong(long z, long mode)
     gel(st,sp++)=stoi(z);
     break;
   case Gvar:
-    err_var();
+    err_var(stoi(z));
   default:
-    pari_err(bugparier,"closure_castlong, type unknown");
+    pari_err_BUG("closure_castlong, type unknown");
   }
 }
 
@@ -585,9 +620,9 @@ closure_func_err(void)
   long fun=s_trace.n-1, pc;
   const char *code;
   GEN C, oper;
-  if (fun < 0 || !trace[fun].pc) return NULL;
-  pc = *trace[fun].pc; C  = trace[fun].closure;
-  code = GSTR(gel(C,2))-1; oper = gel(C,3);
+  if (fun < 0 || trace[fun].pc < 0) return NULL;
+  pc = trace[fun].pc; C  = trace[fun].closure;
+  code = closure_codestr(C); oper = closure_get_oper(C);
   if (code[pc]==OCcallgen || code[pc]==OCcallgen2 ||
       code[pc]==OCcallint || code[pc]==OCcalllong || code[pc]==OCcallvoid)
     return ((entree*)oper[pc])->name;
@@ -615,27 +650,35 @@ get_next_label(const char *s, int member, char **next_fun)
   *u++ = 0; return next_label;
 }
 
+static const char *
+get_arg_name(GEN C, long i)
+{
+  GEN e = gmael(closure_get_dbg(C), 3, 1);
+  return ((entree*)e[i])->name;
+}
+
 void
-closure_err(void)
+closure_err(long level)
 {
   GEN base;
-  const long lastfun = s_trace.n - 1;
+  const long lastfun = s_trace.n - 1 - level;
   char *next_label, *next_fun;
   long i = maxss(0, lastfun - 19);
   if (lastfun < 0) return; /*e.g. when called by gp_main_loop's simplify */
   if (i > 0) while (lg(trace[i].closure)==6) i--;
-  base = gel(trace[i].closure,6); /* gcc -Wall*/
+  base = closure_get_text(trace[i].closure); /* gcc -Wall*/
   next_label = pari_strdup(i == 0? "at top-level": "[...] at");
   next_fun = next_label;
   for (; i <= lastfun; i++)
   {
     GEN C = trace[i].closure;
-    if (lg(C) >= 7) base=gel(C,6);
+    if (lg(C) >= 7) base=closure_get_text(C);
     if ((i==lastfun || lg(trace[i+1].closure)>=7))
     {
+      GEN dbg = gel(closure_get_dbg(C),1);
       /* After a SIGINT, pc can be slightly off: ensure 0 <= pc < lg() */
-      long pc = minss(lg(mael(C,5,1))-1, trace[i].pc ? *trace[i].pc: 1);
-      long offset = pc? mael3(C,5,1,pc): 0;
+      long pc = minss(lg(dbg)-1, trace[i].pc>=0 ? trace[i].pc: 1);
+      long offset = pc? dbg[pc]: 0;
       int member;
       const char *s, *sbase;
       if (typ(base)!=t_VEC) sbase = GSTR(base);
@@ -661,18 +704,17 @@ closure_err(void)
 }
 
 long
-closure_context(long start)
+closure_context(long start, long level)
 {
-  const long lastfun = s_trace.n - 1;
+  const long lastfun = s_trace.n - 1 - level;
   long i, fun = lastfun;
   if (fun<0) return lastfun;
   while (fun>start && lg(trace[fun].closure)==6) fun--;
-  for (i=fun; i<=lastfun; i++)
-  {
-    long *pc=trace[i].pc;
-    push_frame(trace[i].closure, pc?*pc:-1);
-  }
-  return lastfun;
+  for (i=fun; i <= lastfun; i++)
+    push_frame(trace[i].closure, trace[i].pc,0);
+  for (  ; i < s_trace.n; i++)
+    push_frame(trace[i].closure, trace[i].pc,1);
+  return s_trace.n-level;
 }
 
 INLINE void
@@ -680,30 +722,42 @@ st_alloc(long n)
 {
   if (sp+n>s_st.n)
   {
-    stack_alloc(&s_st,n+16);
+    pari_stack_alloc(&s_st,n+16);
     s_st.n=s_st.alloc;
     if (DEBUGMEM>=2) pari_warn(warner,"doubling evaluator stack");
   }
 }
 
+INLINE void
+ptr_proplock(gp_pointer *g, GEN C)
+{
+  g->x = C;
+  if (isclone(g->x))
+  {
+    clone_unlock(g->ox);
+    g->ox = g->x;
+    ++bl_refc(g->ox);
+  }
+}
+
 static void
 closure_eval(GEN C)
 {
-  const char *code=GSTR(gel(C,2))-1;
-  GEN oper=gel(C,3);
-  GEN data=gel(C,4);
+  const char *code=closure_codestr(C);
+  GEN oper=closure_get_oper(C);
+  GEN data=closure_get_data(C);
   long loper=lg(oper);
-  long saved_sp=sp-C[1];
+  long saved_sp=sp-closure_arity(C);
   long saved_rp=rp;
   long j, nbmvar=0, nblvar=0;
-  long pc = 0; /* pc need to be defined after a ^C */
-  if (isclone(C)) ++bl_refc(C);
-  trace_push(&pc, C);
+  long pc, t;
+  clone_lock(C);
+  t = trace_push(0, C);
   if (lg(C)==8)
   {
-    GEN z=gel(C,7);
+    GEN z=closure_get_frame(C);
     long l=lg(z)-1;
-    stack_alloc(&s_var,l);
+    pari_stack_alloc(&s_var,l);
     s_var.n+=l;
     nbmvar+=l;
     for(j=1;j<=l;j++)
@@ -717,8 +771,9 @@ closure_eval(GEN C)
   {
     op_code opcode=(op_code) code[pc];
     long operand=oper[pc];
-    if (sp<0) pari_err(bugparier,"closure_eval, stack underflow");
+    if (sp<0) pari_err_BUG("closure_eval, stack underflow");
     st_alloc(16);
+    trace[t].pc = pc;
     CHECK_CTRLC
     switch(opcode)
     {
@@ -761,6 +816,7 @@ closure_eval(GEN C)
         g->ep = (entree*) operand;
         checkvalue(g->ep);
         g->x = (GEN) g->ep->value;
+        g->ox = g->x; clone_lock(g->ox);
         g->sp = sp;
         gel(st,sp++) = (GEN)&(g->x);
         break;
@@ -771,6 +827,7 @@ closure_eval(GEN C)
         g->vn=operand;
         g->ep=(entree *)0x1L;
         g->x = (GEN) var[s_var.n+operand].value;
+        g->ox = g->x; clone_lock(g->ox);
         g->sp = sp;
         gel(st,sp++) = (GEN)&(g->x);
         break;
@@ -783,6 +840,7 @@ closure_eval(GEN C)
         checkvalue(ep);
         g->sp = -1;
         g->x = copyvalue(ep);
+        g->ox = g->x; clone_lock(g->ox);
         g->vn=0;
         g->ep=NULL;
         C=&g->c;
@@ -797,6 +855,7 @@ closure_eval(GEN C)
         matcomp *C;
         g->sp = -1;
         g->x = copylex(operand);
+        g->ox = g->x; clone_lock(g->ox);
         g->vn=0;
         g->ep=NULL;
         C=&g->c;
@@ -824,6 +883,7 @@ closure_eval(GEN C)
             changevalue(g->ep, g->x);
         }
         else change_compo(&(g->c), g->x);
+        clone_unlock(g->ox);
       }
       break;
     case OCstoredyn:
@@ -836,6 +896,13 @@ closure_eval(GEN C)
     case OCstorelex:
       changelex(operand,gel(st,--sp));
       break;
+    case OCstoreptr:
+      {
+        gp_pointer *g = &ptrs[--rp];
+        change_compo(&(g->c), gel(st,--sp));
+        clone_unlock(g->ox);
+        break;
+      }
     case OCstackgen:
       {
         GEN z = gerepileupto(st[sp-2],gel(st,sp-1));
@@ -871,10 +938,8 @@ closure_eval(GEN C)
       break;
     case OCtostr:
       {
-        GEN z=gel(st,sp+operand);
-        if (typ(z)!=t_STR)
-          z = GENtoGENstr(z);
-        st[sp+operand] = (long) GSTR(z);
+        GEN z = gel(st,sp+operand);
+        st[sp+operand] = (long)GENtostr_unquoted(z);
         break;
       }
     case OCvarn:
@@ -928,7 +993,7 @@ closure_eval(GEN C)
           closure_castlong(p[c],operand);
           break;
         default:
-          pari_err(talker,"_[_]: not a vector");
+          pari_err_TYPE("_[_] OCcompo1 [not a vector]", p);
           break;
         }
         break;
@@ -946,7 +1011,7 @@ closure_eval(GEN C)
         case t_VEC: case t_COL:
           check_array_index(c, lg(p));
           C->ptcell = (GEN *) p+c;
-          g->x = *(C->ptcell);
+          ptr_proplock(g, *(C->ptcell));
           break;
         case t_VECSMALL:
           check_array_index(c, lg(p));
@@ -957,10 +1022,10 @@ closure_eval(GEN C)
           p = list_data(p); lx = p? lg(p): 1;
           check_array_index(c,lx);
           C->ptcell = (GEN *) p+c;
-          g->x = *(C->ptcell);
+          ptr_proplock(g, *(C->ptcell));
           break;
         default:
-          pari_err(talker,"_[_]: not a vector");
+          pari_err_TYPE("&_[_] OCcompo1ptr [not a vector]", p);
         }
         C->parent   = p;
         break;
@@ -970,10 +1035,9 @@ closure_eval(GEN C)
         GEN  p=gel(st,sp-3);
         long c=st[sp-2];
         long d=st[sp-1];
-        if (typ(p)!=t_MAT)
-          pari_err(talker,"_[_,_]: not a matrix");
+        if (typ(p)!=t_MAT) pari_err_TYPE("_[_,_] OCcompo2 [not a matrix]", p);
         check_array_index(d, lg(p));
-        check_array_index(c, lg(p[d]));
+        check_array_index(c, lg(gel(p,d)));
         sp-=3;
         closure_castgen(gcoeff(p,c,d),operand);
         break;
@@ -987,12 +1051,12 @@ closure_eval(GEN C)
         GEN p = g->x;
         sp-=2;
         if (typ(p)!=t_MAT)
-          pari_err(talker,"_[_,_]: not a matrix");
+          pari_err_TYPE("&_[_,_] OCcompo2ptr [not a matrix]", p);
         check_array_index(d, lg(p));
-        check_array_index(c, lg(p[d]));
+        check_array_index(c, lg(gel(p,d)));
         C->ptcell = (GEN *) gel(p,d)+c;
         C->parent   = p;
-        g->x = *(C->ptcell);
+        ptr_proplock(g, *(C->ptcell));
         break;
       }
     case OCcompoC:
@@ -1000,7 +1064,7 @@ closure_eval(GEN C)
         GEN  p=gel(st,sp-2);
         long c=st[sp-1];
         if (typ(p)!=t_MAT)
-          pari_err(talker,"_[,_]: not a matrix");
+          pari_err_TYPE("_[,_] OCcompoC [not a matrix]", p);
         check_array_index(c, lg(p));
         sp--;
         gel(st,sp-1) = gel(p,c);
@@ -1014,12 +1078,12 @@ closure_eval(GEN C)
         GEN p = g->x;
         sp--;
         if (typ(p)!=t_MAT)
-          pari_err(talker,"_[,_]: not a matrix");
+          pari_err_TYPE("&_[,_] OCcompoCptr [not a matrix]", p);
         check_array_index(c, lg(p));
         C->ptcell = (GEN *) p+c;
         C->full_col = c;
         C->parent   = p;
-        g->x = *(C->ptcell);
+        ptr_proplock(g, *(C->ptcell));
         break;
       }
     case OCcompoL:
@@ -1028,9 +1092,8 @@ closure_eval(GEN C)
         long r=st[sp-1];
         sp--;
         if (typ(p)!=t_MAT)
-          pari_err(talker,"_[_,]: not a matrix");
-        if (lg(p)==1) pari_err(talker,"a 0x0 matrix has no elements");
-        check_array_index(r,lg(p[1]));
+          pari_err_TYPE("_[_,] OCcompoL [not a matrix]", p);
+        check_array_index(r,lg(p) == 1? 1: lgcols(p));
         gel(st,sp-1) = row(p,r);
         break;
       }
@@ -1042,9 +1105,8 @@ closure_eval(GEN C)
         GEN p = g->x, p2;
         sp--;
         if (typ(p)!=t_MAT)
-          pari_err(talker,"_[_,]: not a matrix");
-        if (lg(p)==1) pari_err(talker,"a 0x0 matrix has no elements");
-        check_array_index(r,lg(p[1]));
+          pari_err_TYPE("&_[_,] OCcompoLptr [not a matrix]", p);
+        check_array_index(r,lg(p) == 1? 1: lgcols(p));
         p2 = rowcopy(p,r);
         C->full_row = r; /* record row number */
         C->ptcell = &p2;
@@ -1055,14 +1117,14 @@ closure_eval(GEN C)
     case OCdefaultarg:
       if (var[s_var.n+operand].flag==DEFAULT_VAL)
       {
-        GEN z = closure_evalnobrk(gel(st,sp-1));
-        pushlex(operand,z);
+        GEN z = gel(st,sp-1);
+        pushlex(operand,typ(z)== t_CLOSURE? closure_evalnobrk(z): z);
       }
       sp--;
       break;
     case OClocalvar:
       {
-        long n = stack_new(&s_lvars);
+        long n = pari_stack_new(&s_lvars);
         entree *ep = (entree *)operand;
         checkvalue(ep);
         lvars[n] = ep;
@@ -1072,7 +1134,7 @@ closure_eval(GEN C)
       }
     case OClocalvar0:
       {
-        long n = stack_new(&s_lvars);
+        long n = pari_stack_new(&s_lvars);
         entree *ep = (entree *)operand;
         checkvalue(ep);
         lvars[n] = ep;
@@ -1106,7 +1168,7 @@ closure_eval(GEN C)
     case 19: sp-=19; f(st[sp],st[sp+1],st[sp+2],st[sp+3],st[sp+4],st[sp+5],st[sp+6],st[sp+7],st[sp+8],st[sp+9],st[sp+10],st[sp+11],st[sp+12],st[sp+13],st[sp+14],st[sp+15],st[sp+16],st[sp+17],st[sp+18]); break; \
     case 20: sp-=20; f(st[sp],st[sp+1],st[sp+2],st[sp+3],st[sp+4],st[sp+5],st[sp+6],st[sp+7],st[sp+8],st[sp+9],st[sp+10],st[sp+11],st[sp+12],st[sp+13],st[sp+14],st[sp+15],st[sp+16],st[sp+17],st[sp+18],st[sp+19]); break; \
     default: \
-      pari_err(impl,"functions with more than 20 parameters");\
+      pari_err_IMPL("functions with more than 20 parameters");\
       goto endeval; /*not reached*/ \
   }
 
@@ -1164,19 +1226,19 @@ closure_eval(GEN C)
         GEN fun = gel(st,sp-1-n);
         long arity;
         GEN z;
-        if (typ(fun)!=t_CLOSURE) pari_err(notfuncer, fun);
-        arity=fun[1];
+        if (typ(fun)!=t_CLOSURE) pari_err(e_NOTFUNC, fun);
+        arity = closure_arity(fun);
         if (n!=arity)
         {
           if (n>arity)
-            pari_err(talker,"too many parameters in user-defined function call");
+            pari_err(e_MISC,"too many parameters in user-defined function call");
           st_alloc(arity-n);
           for (j=n+1;j<=arity;j++)
             gel(st,sp++)=0;
         }
 #ifdef STACK_CHECK
         if (PARI_stack_limit && (void*) &z <= PARI_stack_limit)
-          pari_err(talker, "deep recursion");
+          pari_err(e_MISC, "deep recursion");
 #endif
         z = closure_return(fun);
         if (br_status) goto endeval;
@@ -1184,9 +1246,10 @@ closure_eval(GEN C)
         break;
       }
     case OCnewframe:
-      stack_alloc(&s_var,operand);
+      if (operand>0) nbmvar+=operand;
+      else operand=-operand;
+      pari_stack_alloc(&s_var,operand);
       s_var.n+=operand;
-      nbmvar+=operand;
       for(j=1;j<=operand;j++)
       {
         var[s_var.n-j].flag=PUSH_VAL;
@@ -1208,7 +1271,7 @@ closure_eval(GEN C)
       }
       break;
     case OCgetargs:
-      stack_alloc(&s_var,operand);
+      pari_stack_alloc(&s_var,operand);
       s_var.n+=operand;
       nbmvar+=operand;
       sp-=operand;
@@ -1223,15 +1286,21 @@ closure_eval(GEN C)
         }
       }
       break;
+    case OCcheckuserargs:
+      for (j=0; j<operand; j++)
+        if (var[s_var.n-operand+j].flag==DEFAULT_VAL)
+          pari_err(e_MISC,"missing mandatory argument"
+                   " '%s' in user function",get_arg_name(C,j+1));
+      break;
     case OCcheckargs:
       for (j=sp-1;operand;operand>>=1UL,j--)
         if ((operand&1L) && gel(st,j)==NULL)
-          pari_err(talker,"missing mandatory argument");
+          pari_err(e_MISC,"missing mandatory argument");
       break;
     case OCcheckargs0:
       for (j=sp-1;operand;operand>>=1UL,j--)
         if ((operand&1L) && gel(st,j))
-          pari_err(talker,"argument type not implemented");
+          pari_err(e_MISC,"argument type not implemented");
       break;
     case OCdefaultlong:
       sp--;
@@ -1267,17 +1336,29 @@ closure_eval(GEN C)
     case OCpop:
       sp-=operand;
       break;
+    case OCdup:
+      {
+        long i, s=st[sp-1];
+        st_alloc(operand);
+        for(i=1;i<=operand;i++)
+          st[sp++]=s;
+      }
+      break;
     }
   }
   if (0)
   {
 endeval:
     sp = saved_sp;
-    rp = saved_rp;
+    for(  ; rp>saved_rp ;  )
+    {
+      gp_pointer *g = &ptrs[--rp];
+      clone_unlock(g->ox);
+    }
   }
   s_trace.n--;
   restore_vars(nbmvar, nblvar);
-  if (isclone(C)) gunclone(C);
+  clone_unlock(C);
 }
 
 GEN
@@ -1299,12 +1380,14 @@ evalstate_save(struct pari_evalstate *state)
   state->lvars= s_lvars.n;
   state->trace= s_trace.n;
   compilestate_save(&state->comp);
+  mtstate_save(&state->pending_threads);
 }
 
 void
 evalstate_restore(struct pari_evalstate *state)
 {
   avma = state->avma;
+  mtstate_restore(&state->pending_threads);
   sp = state->sp;
   rp = state->rp;
   restore_vars(s_var.n-state->var,s_lvars.n-state->lvars);
@@ -1316,8 +1399,10 @@ evalstate_restore(struct pari_evalstate *state)
 void
 evalstate_reset(void)
 {
+  mtstate_reset();
   sp = 0;
   rp = 0;
+  dbg_level = 0;
   restore_vars(s_var.n, s_lvars.n);
   s_trace.n = 0;
   reset_break();
@@ -1345,8 +1430,8 @@ closure_trapgen(GEN C, long numerr)
   VOLATILE GEN x;
   struct pari_evalstate state;
   evalstate_save(&state);
-  CATCH(numerr) { x = (GEN)1L; }
-  TRY { x = closure_evalgen(C); } ENDCATCH;
+  pari_CATCH(numerr) { x = (GEN)1L; }
+  pari_TRY { x = closure_evalgen(C); } pari_ENDCATCH;
   if (x == (GEN)1L) evalstate_restore(&state);
   return x;
 }
@@ -1356,7 +1441,7 @@ closure_evalnobrk(GEN C)
 {
   pari_sp ltop=avma;
   closure_eval(C);
-  if (br_status) pari_err(talker, "break not allowed here");
+  if (br_status) pari_err(e_MISC, "break not allowed here");
   return gerepileupto(ltop,gel(st,--sp));
 }
 
@@ -1384,39 +1469,217 @@ closure_returnupto(GEN C)
 void
 closure_callvoid1(GEN C, GEN x)
 {
-  long i;
-  gel(st,sp++)=x;
-  for(i=2; i<=C[1]; i++) gel(st,sp++) = NULL;
+  long i, ar = closure_arity(C);
+  gel(st,sp++) = x;
+  for(i=2; i <= ar; i++) gel(st,sp++) = NULL;
   closure_evalvoid(C);
 }
 
 GEN
 closure_callgen1(GEN C, GEN x)
 {
-  long i;
-  gel(st,sp++)=x;
-  for(i=2; i<=C[1]; i++) gel(st,sp++) = NULL;
+  long i, ar = closure_arity(C);
+  gel(st,sp++) = x;
+  for(i=2; i<= ar; i++) gel(st,sp++) = NULL;
   return closure_returnupto(C);
 }
 
+GEN
+pareval_worker(GEN C)
+{
+  return closure_callgenall(C, 0);
+}
+
+GEN
+pareval(GEN C)
+{
+  pari_sp av = avma;
+  long l = lg(C), i, pending = 0, workid;
+  struct pari_mt pt;
+  GEN worker, V, done;
+  if (!is_vec_t(typ(C))) pari_err_TYPE("pareval",C);
+  for (i=1; i<l; i++)
+    if (typ(gel(C,i))!=t_CLOSURE)
+      pari_err_TYPE("pareval",gel(C,i));
+  worker = snm_closure(is_entry("_pareval_worker"), NULL);
+  V = cgetg(l, t_VEC);
+  mt_queue_start(&pt, worker);
+  for (i=1; i<l || pending; i++)
+  {
+    mt_queue_submit(&pt, i, i<l? mkvec(gel(C,i)): NULL);
+    done = mt_queue_get(&pt, &workid, &pending);
+    if (done) gel(V,workid) = done;
+  }
+  mt_queue_end(&pt);
+  return gerepilecopy(av, V);
+}
+
+GEN
+parvector_worker(GEN i, GEN C)
+{
+  return closure_callgen1(C, i);
+}
+
+GEN
+parfor_worker(GEN i, GEN C)
+{
+  retmkvec2(gcopy(i), closure_callgen1(C, i));
+}
+
+GEN
+parvector(long n, GEN code)
+{
+  long i, pending = 0, workid;
+  GEN worker = snm_closure(is_entry("_parvector_worker"), mkvec(code));
+  GEN a, V, done;
+  struct pari_mt pt;
+  mt_queue_start(&pt, worker);
+  a = mkvec(cgetipos(3)); /* left on the stack */
+  V = cgetg(n+1, t_VEC);
+  for (i=1; i<=n || pending; i++)
+  {
+    mael(a,1,2) = i;
+    mt_queue_submit(&pt, i, i<=n? a: NULL);
+    done = mt_queue_get(&pt, &workid, &pending);
+    if (done) gel(V,workid) = done;
+  }
+  mt_queue_end(&pt);
+  return V;
+}
+
+GEN
+parsum(GEN a, GEN b, GEN code, GEN x)
+{
+  pari_sp av = avma, av2, lim;
+  long pending = 0;
+  GEN worker = snm_closure(is_entry("_parvector_worker"), mkvec(code));
+  GEN done;
+  struct pari_mt pt;
+  if (typ(a) != t_INT) pari_err_TYPE("parsum",a);
+  if (!x) x = gen_0;
+  if (gcmp(b,a) < 0) return gcopy(x);
+
+  mt_queue_start(&pt, worker);
+  b = gfloor(b);
+  a = mkvec(setloop(a));
+  av2=avma; lim = stack_lim(av2,1);
+  for (; cmpii(gel(a,1),b) <= 0 || pending; gel(a,1) = incloop(gel(a,1)))
+  {
+    mt_queue_submit(&pt, 0, cmpii(gel(a,1),b) <= 0? a: NULL);
+    done = mt_queue_get(&pt, NULL, &pending);
+    if (done)
+    {
+      x = gadd(x, done);
+      if (low_stack(lim, stack_lim(av2,1)))
+      {
+        if (DEBUGMEM>1) pari_warn(warnmem,"sum");
+        x = gerepileupto(av2,x);
+      }
+    }
+  }
+  mt_queue_end(&pt);
+  return gerepilecopy(av, x);
+}
+
+void
+parfor(GEN a, GEN b, GEN code, GEN code2)
+{
+  pari_sp av = avma, av2;
+  long running, pending = 0;
+  long status = br_NONE;
+  GEN worker = snm_closure(is_entry("_parfor_worker"), mkvec(code));
+  GEN done, stop = NULL;
+  struct pari_mt pt;
+  if (typ(a) != t_INT) pari_err_TYPE("parfor",a);
+  if (b && gcmp(b,a) < 0) return;
+
+  mt_queue_start(&pt, worker);
+  b = b ? gfloor(b): NULL;
+  a = mkvec(setloop(a));
+  av2 = avma;
+  while ((running = (!stop && (!b || cmpii(gel(a,1),b) <= 0))) || pending)
+  {
+    mt_queue_submit(&pt, 0, running ? a: NULL);
+    done = mt_queue_get(&pt, NULL, &pending);
+    if (code2 && done && (!stop || cmpii(gel(done,1),stop) < 0))
+    {
+      push_lex(gel(done,1), code2);
+      push_lex(gel(done,2), NULL);
+      closure_evalvoid(code2);
+      pop_lex(2);
+      if (loop_break())
+      {
+        status = br_status;
+        br_status = br_NONE;
+        stop = gerepileuptoint(av2, gel(done,1));
+      }
+    }
+    gel(a,1) = incloop(gel(a,1));
+    if (!stop) avma = av2;
+  }
+  avma = av2;
+  mt_queue_end(&pt);
+  br_status = status;
+  avma = av;
+}
+
+void
+parforprime(GEN a, GEN b, GEN code, GEN code2)
+{
+  pari_sp av = avma, av2;
+  long running, pending = 0;
+  long status = br_NONE;
+  GEN worker = snm_closure(is_entry("_parfor_worker"), mkvec(code));
+  GEN done, stop = NULL;
+  struct pari_mt pt;
+  forprime_t T;
+
+  if (!forprime_init(&T, a,b)) { avma = av; return; }
+  mt_queue_start(&pt, worker);
+  av2 = avma;
+  while ((running = (!stop && forprime_next(&T))) || pending)
+  {
+    mt_queue_submit(&pt, 0, running ? mkvec(T.pp): NULL);
+    done = mt_queue_get(&pt, NULL, &pending);
+    if (code2 && done && (!stop || cmpii(gel(done,1),stop) < 0))
+    {
+      push_lex(gel(done,1), code2);
+      push_lex(gel(done,2), NULL);
+      closure_evalvoid(code2);
+      pop_lex(2);
+      if (loop_break())
+      {
+        status = br_status;
+        br_status = br_NONE;
+        stop = gerepileuptoint(av2, gel(done,1));
+      }
+    }
+    if (!stop) avma = av2;
+  }
+  avma = av2;
+  mt_queue_end(&pt);
+  br_status = status;
+  avma = av;
+}
+
 GEN
 closure_callgen2(GEN C, GEN x, GEN y)
 {
-  long i;
-  st_alloc(C[1]);
-  gel(st,sp++)=x;
-  gel(st,sp++)=y;
-  for(i=3; i<=C[1]; i++) gel(st,sp++) = NULL;
+  long i, ar = closure_arity(C);
+  st_alloc(ar);
+  gel(st,sp++) = x;
+  gel(st,sp++) = y;
+  for(i=3; i<=ar; i++) gel(st,sp++) = NULL;
   return closure_returnupto(C);
 }
 
 GEN
 closure_callgenvec(GEN C, GEN args)
 {
-  long i, l = lg(args);
-  st_alloc(C[1]);
+  long i, l = lg(args), ar = closure_arity(C);
+  st_alloc(ar);
   for (i = 1; i < l;   i++) gel(st,sp++) = gel(args,i);
-  for(      ; i<=C[1]; i++) gel(st,sp++) = NULL;
+  for(      ; i <= ar; i++) gel(st,sp++) = NULL;
   return closure_returnupto(C);
 }
 
@@ -1424,11 +1687,11 @@ GEN
 closure_callgenall(GEN C, long n, ...)
 {
   va_list ap;
-  long i;
+  long i, ar = closure_arity(C);
   va_start(ap,n);
-  st_alloc(C[1]);
-  for (i = 1; i <=n;   i++) gel(st,sp++) = va_arg(ap, GEN);
-  for(      ; i<=C[1]; i++) gel(st,sp++) = NULL;
+  st_alloc(ar);
+  for (i = 1; i <=n;  i++) gel(st,sp++) = va_arg(ap, GEN);
+  for(      ; i <=ar; i++) gel(st,sp++) = NULL;
   va_end(ap);
   return closure_returnupto(C);
 }
@@ -1441,6 +1704,21 @@ gp_eval(void *E, GEN x)
   return closure_evalnobrk(code);
 }
 
+GEN
+gp_evalupto(void *E, GEN x)
+{
+  pari_sp av = avma;
+  return copyupto(gp_eval(E,x), (GEN)av);
+}
+
+long
+gp_evalbool(void *E, GEN x)
+{
+  pari_sp av = avma;
+  long res  = !gequal0(gp_eval(E,x));
+  avma = av; return res;
+}
+
 long
 gp_evalvoid(void *E, GEN x)
 {
@@ -1495,13 +1773,12 @@ disassemble_cast(long mode)
 void
 closure_disassemble(GEN C)
 {
-  char * code;
+  const char * code;
   GEN oper;
   long i;
-  if (typ(C)!=t_CLOSURE)
-    pari_err(typeer,"disassemble");
-  code=GSTR(gel(C,2))-1;
-  oper=gel(C,3);
+  if (typ(C)!=t_CLOSURE) pari_err_TYPE("disassemble",C);
+  code=closure_codestr(C);
+  oper=closure_get_oper(C);
   for(i=1;i<lg(oper);i++)
   {
     op_code opcode=(op_code) code[i];
@@ -1548,6 +1825,9 @@ closure_disassemble(GEN C)
     case OCstorelex:
       pari_printf("storelex\t%ld\n",operand);
       break;
+    case OCstoreptr:
+      pari_printf("storeptr\n");
+      break;
     case OCsimpleptrdyn:
       {
         entree *ep = (entree *)operand;
@@ -1629,6 +1909,9 @@ closure_disassemble(GEN C)
     case OCcheckargs0:
       pari_printf("checkargs0\t0x%lx\n",operand);
       break;
+    case OCcheckuserargs:
+      pari_printf("checkuserargs\t%ld\n",operand);
+      break;
     case OCdefaultlong:
       pari_printf("defaultlong\t%ld\n",operand);
       break;
@@ -1704,6 +1987,9 @@ closure_disassemble(GEN C)
     case OCpop:
       pari_printf("pop\t\t%ld\n",operand);
       break;
+    case OCdup:
+      pari_printf("dup\t\t%ld\n",operand);
+      break;
     case OCavma:
       pari_printf("avma\n",operand);
       break;
@@ -1722,3 +2008,218 @@ closure_disassemble(GEN C)
     }
   }
 }
+
+static int
+opcode_need_relink(op_code opcode)
+{
+  switch(opcode)
+  {
+  case OCpushlong:
+  case OCpushgen:
+  case OCpushgnil:
+  case OCpushreal:
+  case OCpushstoi:
+  case OCpushlex:
+  case OCstorelex:
+  case OCstoreptr:
+  case OCsimpleptrlex:
+  case OCnewptrlex:
+  case OCpushptr:
+  case OCstackgen:
+  case OCendptr:
+  case OCprecreal:
+  case OCprecdl:
+  case OCstoi:
+  case OCitos:
+  case OCtostr:
+  case OCvarn:
+  case OCcopy:
+  case OCcopyifclone:
+  case OCcompo1:
+  case OCcompo1ptr:
+  case OCcompo2:
+  case OCcompo2ptr:
+  case OCcompoC:
+  case OCcompoCptr:
+  case OCcompoL:
+  case OCcompoLptr:
+  case OCcheckargs:
+  case OCcheckargs0:
+  case OCcheckuserargs:
+  case OCgetargs:
+  case OCdefaultarg:
+  case OCdefaultgen:
+  case OCdefaultlong:
+  case OCcalluser:
+  case OCvec:
+  case OCcol:
+  case OCmat:
+  case OCnewframe:
+  case OCsaveframe:
+  case OCdup:
+  case OCpop:
+  case OCavma:
+  case OCgerepile:
+  case OCcowvarlex:
+    break;
+  case OCpushvar:
+  case OCpushdyn:
+  case OCstoredyn:
+  case OCsimpleptrdyn:
+  case OCnewptrdyn:
+  case OClocalvar:
+  case OClocalvar0:
+  case OCcallgen:
+  case OCcallgen2:
+  case OCcalllong:
+  case OCcallint:
+  case OCcallvoid:
+  case OCcowvardyn:
+    return 1;
+  }
+  return 0;
+}
+
+static void
+closure_relink(GEN C, hashtable *table)
+{
+  const char *code = closure_codestr(C);
+  GEN oper = closure_get_oper(C);
+  GEN fram = gel(closure_get_dbg(C),3);
+  long i, j;
+  for(i=1;i<lg(oper);i++)
+    if (oper[i] && opcode_need_relink((op_code)code[i]))
+      oper[i] = (long) hash_search(table,(void*) oper[i])->val;
+  for (i=1;i<lg(fram);i++)
+    for (j=1;j<lg(gel(fram,i));j++)
+      if (mael(fram,i,j))
+        mael(fram,i,j) = (long) hash_search(table,(void*) mael(fram,i,j))->val;
+}
+
+void
+gen_relink(GEN x, hashtable *table)
+{
+  long i, lx, tx = typ(x);
+  switch(tx)
+  {
+    case t_CLOSURE:
+      closure_relink(x, table);
+      gen_relink(closure_get_data(x), table);
+      if (lg(x)==8) gen_relink(closure_get_frame(x), table);
+      break;
+    case t_LIST:
+      gen_relink(list_data(x), table);
+      break;
+    case t_VEC: case t_COL: case t_MAT: case t_ERROR:
+      lx = lg(x);
+      for (i=lontyp[tx]; i<lx; i++) gen_relink(gel(x,i), table);
+  }
+}
+
+static void
+closure_unlink(GEN C)
+{
+  const char *code = closure_codestr(C);
+  GEN oper = closure_get_oper(C);
+  GEN fram = gel(closure_get_dbg(C),3);
+  long i, j;
+  for(i=1;i<lg(oper);i++)
+    if (oper[i] && opcode_need_relink((op_code) code[i]))
+    {
+      long n = pari_stack_new(&s_relocs);
+      relocs[n] = (entree *) oper[i];
+    }
+  for (i=1;i<lg(fram);i++)
+    for (j=1;j<lg(gel(fram,i));j++)
+      if (mael(fram,i,j))
+      {
+        long n = pari_stack_new(&s_relocs);
+        relocs[n] = (entree *) mael(fram,i,j);
+      }
+}
+
+static void
+gen_unlink(GEN x)
+{
+  long i, lx, tx = typ(x);
+  switch(tx)
+  {
+    case t_CLOSURE:
+      closure_unlink(x);
+      gen_unlink(closure_get_data(x));
+      if (lg(x)==8) gen_unlink(closure_get_frame(x));
+      break;
+    case t_LIST:
+      gen_unlink(list_data(x));
+      break;
+    case t_VEC: case t_COL: case t_MAT: case t_ERROR:
+      lx = lg(x);
+      for (i = lontyp[tx]; i<lx; i++) gen_unlink(gel(x,i));
+  }
+}
+
+GEN
+copybin_unlink(GEN C)
+{
+  long i, l , n, nold = s_relocs.n;
+  GEN v, w, V, res;
+  if (C)
+    gen_unlink(C);
+  else
+  { /* contents of all variables */
+    long v, maxv = pari_var_next();
+    for (v=0; v<maxv; v++)
+    {
+      entree *ep = varentries[v];
+      if (!ep || !ep->value) continue;
+      gen_unlink((GEN)ep->value);
+    }
+  }
+  n = s_relocs.n-nold;
+  v = cgetg(n+1, t_VECSMALL);
+  for(i=0; i<n; i++)
+    v[i+1] = (long) relocs[i];
+  s_relocs.n = nold;
+  w = vecsmall_uniq(v); l = lg(w);
+  res = cgetg(3,t_VEC);
+  V = cgetg(l, t_VEC);
+  for(i=1; i<l; i++)
+  {
+    entree *ep = (entree*) w[i];
+    gel(V,i) = strtoGENstr(ep->name);
+  }
+  gel(res,1) = vecsmall_copy(w);
+  gel(res,2) = V;
+  return res;
+}
+
+static ulong
+hash_id(void *x) { return (ulong)x; }
+static int
+eq_id(void *x, void *y) { return x == y; }
+
+/* e = t_VECSMALL of entree *ep [ addresses ],
+ * names = t_VEC of strtoGENstr(ep.names),
+ * Return hashtable : ep => is_entry(ep.name) */
+hashtable *
+hash_from_link(GEN e, GEN names, int use_stack)
+{
+  long i, l = lg(e);
+  hashtable *h = hash_create(l-1, &hash_id, &eq_id, use_stack);
+  if (lg(names) != l) pari_err_DIM("hash_from_link");
+  for (i = 1; i < l; i++)
+  {
+    char *s = GSTR(gel(names,i));
+    hash_insert(h, (void*)e[i], (void*)fetch_entry(s, strlen(s)));
+  }
+  return h;
+}
+
+void
+bincopy_relink(GEN C, GEN V)
+{
+  pari_sp av = avma;
+  hashtable *table = hash_from_link(gel(V,1),gel(V,2),1);
+  gen_relink(C, table);
+  avma = av;
+}
