@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -85,7 +83,7 @@ qrom3(void *E, GEN (*eval)(void *, GEN), GEN a, GEN b, long prec)
     gel(s,j) = gerepileupto(av, gmul2n(gadd(gel(s,j-1), sum), -1));
     if (DEBUGLEVEL>3) err_printf("qrom3: iteration %ld: %Ps\n", j,s[j]);
 
-    if (j >= KLOC && (ss = interp(h, s, j, bit_accuracy(prec)-j-6, KLOC)))
+    if (j >= KLOC && (ss = interp(h, s, j, prec2nbits(prec)-j-6, KLOC)))
       return gmulsg(sig,ss);
   }
   return NULL;
@@ -127,7 +125,7 @@ qrom2(void *E, GEN (*eval)(void *, GEN), GEN a, GEN b, long prec)
     gel(s,j) = gerepileupto(av, gadd(p1,sum));
     if (DEBUGLEVEL>3) err_printf("qrom2: iteration %ld: %Ps\n", j,s[j]);
 
-    if (j >= KLOC && (ss = interp(h, s, j, bit_accuracy(prec)-(3*j/2)-6, KLOC)))
+    if (j >= KLOC && (ss = interp(h, s, j, prec2nbits(prec)-(3*j/2)-6, KLOC)))
       return gmulsg(sig, ss);
   }
   return NULL;
@@ -273,7 +271,7 @@ findmforinit(long m, long prec)
 
   if (m <= 0)
   {
-    p = (long)bit_accuracy_mul(prec, 0.3);
+    p = (long)prec2nbits_mul(prec, 0.3);
     m = 2; r = 4;
     while (r < p) { m++; r <<= 1; }
   }
@@ -289,7 +287,7 @@ intinit_start(intdata *D, long m0, long flext, long prec)
   long m = findmforinit(m0, prec), lim = 20L<<m;
   if (flext > 0) lim = lim << (2*flext);
   D->m = m;
-  D->eps = bit_accuracy(prec);
+  D->eps = prec2nbits(prec);
   D->tabxp = cgetg(lim+1, t_VEC);
   D->tabwp = cgetg(lim+1, t_VEC);
   D->tabxm = cgetg(lim+1, t_VEC);
@@ -300,7 +298,7 @@ static GEN
 intinit_end(intdata *D, long pnt, long mnt)
 {
   GEN v = cgetg(8, t_VEC);
-  if (pnt < 0) pari_err(talker,"incorrect table length in intnum initialization");
+  if (pnt < 0) pari_err_DOMAIN("intnuminit","table length","<",gen_0,stoi(pnt));
   gel(v,1) = stoi(D->m);
   TABx0(v) = D->tabx0;
   TABw0(v) = D->tabw0;
@@ -310,9 +308,16 @@ intinit_end(intdata *D, long pnt, long mnt)
   TABwm(v) = D->tabwm; setlg(D->tabwm, mnt+1); return v;
 }
 
+static const long EXTRAPREC =
+#ifdef LONG_IS_64BIT
+  1;
+#else
+  2;
+#endif
+
 /* divide by 2 in place */
 static GEN
-divr2_ip(GEN x) { setexpo(x, expo(x)-1); return x; }
+divr2_ip(GEN x) { shiftr_inplace(x, -1); return x; }
 
 /* phi(t)=tanh((3/2)sinh(t)) : from -1 to 1, hence also from a to b compact
  * interval. */
@@ -331,12 +336,12 @@ inittanhsinh(long m, long prec)
   et = ex = mpexp(h);
   for (k = 1; k <= lim; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec+1);
-    gel(D.tabwp,k) = cgetr(prec+1); av = avma;
+    gel(D.tabxp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwp,k) = cgetr(prec+EXTRAPREC); av = avma;
     ct = divr2_ip(addrr(et, invr(et)));
     st = subrr(et, ct);
     ext = invr( addrs(mpexp(mulur(3, st)), 1) );
-    setexpo(ext, expo(ext)+1);
+    shiftr_inplace(ext, 1);
     xp = subsr(1, ext);
     wp = divr2_ip(mulur(3, mulrr(ct, mulrr(ext, addsr(1, xp)))));
     if (expo(wp) < -D.eps) { nt = k-1; break; }
@@ -363,8 +368,8 @@ initsinhsinh(long m, long prec)
   et = ex = mpexp(h);
   for (k = 1; k <= lim; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec+1);
-    gel(D.tabwp,k) = cgetr(prec+1); av = avma;
+    gel(D.tabxp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwp,k) = cgetr(prec+EXTRAPREC); av = avma;
     ct = divr2_ip(addrr(et, invr(et)));
     st = subrr(et, ct);
     ext = mpexp(st);
@@ -395,8 +400,8 @@ initsinh(long m, long prec)
   et = ex = mpexp(h);
   for (k = 1; k <= lim; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec+1);
-    gel(D.tabwp,k) = cgetr(prec+1); av = avma;
+    gel(D.tabxp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwp,k) = cgetr(prec+EXTRAPREC); av = avma;
     eti = invr(et);
     xp = subrr(et, eti);
     wp = addrr(et, eti);
@@ -433,7 +438,7 @@ initexpsinh(long m, long prec)
     gel(D.tabwp,k) = mulrr(xp, t);
     gel(D.tabxm,k) = invr(xp);
     gel(D.tabwm,k) = mulrr(gel(D.tabxm,k), t);
-    if (expo(D.tabxm[k]) < -D.eps) { nt = k-1; break; }
+    if (expo(gel(D.tabxm,k)) < -D.eps) { nt = k-1; break; }
   }
   return gerepilecopy(ltop, intinit_end(&D, nt, nt));
 }
@@ -454,10 +459,10 @@ initexpexp(long m, long prec)
   et = ex = mpexp(negr(h));
   for (k = 1; k <= lim; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec+1);
-    gel(D.tabwp,k) = cgetr(prec+1);
-    gel(D.tabxm,k) = cgetr(prec+1);
-    gel(D.tabwm,k) = cgetr(prec+1); av = avma;
+    gel(D.tabxp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabxm,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwm,k) = cgetr(prec+EXTRAPREC); av = avma;
     eti = invr(et); kh = mulur(k,h);
     xp = mpexp(subrr(kh, et));
     xm = mpexp(negr(addrr(kh, eti)));
@@ -489,10 +494,10 @@ initnumsine(long m, long prec)
   et = ex = mpexp(h);
   for (k = 1; k <= lim; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec+1);
-    gel(D.tabwp,k) = cgetr(prec+1);
-    gel(D.tabxm,k) = cgetr(prec+1);
-    gel(D.tabwm,k) = cgetr(prec+1); av = avma;
+    gel(D.tabxp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwp,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabxm,k) = cgetr(prec+EXTRAPREC);
+    gel(D.tabwm,k) = cgetr(prec+EXTRAPREC); av = avma;
     eti = invr(et); /* exp(-kh) */
     ct = divr2_ip(addrr(et, eti));
     st = divr2_ip(subrr(et, eti));
@@ -500,8 +505,8 @@ initnumsine(long m, long prec)
     extm = invr(extp); extm1 = subsr(1, extm); extm2 = invr(extm1);
     kpi = mulur(k, pi);
     kct = mulur(k, ct);
-    setexpo(extm1, expo(extm1) + D.m);
-    setexpo(extp1, expo(extp1) + D.m);
+    shiftr_inplace(extm1, D.m);
+    shiftr_inplace(extp1, D.m);
     xp = mulrr(kpi, extm2);
     wp = mulrr(subrr(extm1, mulrr(kct, extm)), mulrr(pi, sqrr(extm2)));
     xm = mulrr(negr(kpi), extp2);
@@ -522,13 +527,14 @@ suminit_start(GEN sig)
 
   if (typ(sig) == t_VEC)
   {
-    if (lg(sig) != 3) pari_err(typeer,"sumnum");
+    if (lg(sig) != 3) pari_err_TYPE("sumnum",sig);
     sig2 = gel(sig,2);
     sig  = gel(sig,1);
+    if (!isinR(sig2)) pari_err_TYPE("sumnum",sig2);
+    if (gsigne(sig2) > 0) sig2 = mulcxmI(sig2);
   }
   else sig2 = gen_0;
-  if (!isinR(sig) || !isinR(sig2)) pari_err(talker, "incorrect abscissa in sumnum");
-  if (gsigne(sig2) > 0) sig2 = mulcxmI(sig2);
+  if (!isinR(sig)) pari_err_TYPE("sumnum",sig);
   return mkvec2(mkvec(gen_1), sig2);
 }
 
@@ -549,10 +555,10 @@ sumnuminit(GEN sig, long m, long sgn, long prec)
     tab = intnuminit(mkvec(gen_m1), mkvec(gen_1), m, prec);
   else
     tab = intnuminit(gen_0, b, m, prec);
-  eps = bit_accuracy(prec);
+  eps = prec2nbits(prec);
   t = gmul(pi, TABx0(tab));
-  if (sgn < 0) TABw0(tab) = gdiv(TABw0(tab), gch(t, prec));
-  else         TABw0(tab) = gmul(TABw0(tab), gth(t, prec));
+  if (sgn < 0) TABw0(tab) = gdiv(TABw0(tab), gcosh(t, prec));
+  else         TABw0(tab) = gmul(TABw0(tab), gtanh(t, prec));
   tabxp = TABxp(tab); L = lg(tabxp);
   tabwp = TABwp(tab);
   tabxm = TABxm(tab);
@@ -562,16 +568,16 @@ sumnuminit(GEN sig, long m, long sgn, long prec)
     if (cmprs(gel(tabxp,k), eps) < 0)
     {
       t = mulrr(pi, gel(tabxp,k));
-      gel(tabwp,k) = (sgn < 0)? divrr(gel(tabwp,k), gch(t, prec))
-                              : mulrr(gel(tabwp,k), gth(t, prec));
+      gel(tabwp,k) = (sgn < 0)? divrr(gel(tabwp,k), gcosh(t, prec))
+                              : mulrr(gel(tabwp,k), gtanh(t, prec));
     }
     else
       if (sgn < 0) gel(tabwp,k) = real_0_bit(-eps);
     if (!flii)
     {
       t = mulrr(pi, gel(tabxm,k));
-      gel(tabwm,k) = (sgn < 0)? divrr(gel(tabwm,k), gch(t, prec))
-                              : mulrr(gel(tabwm,k), gth(t, prec));
+      gel(tabwm,k) = (sgn < 0)? divrr(gel(tabwm,k), gcosh(t, prec))
+                              : mulrr(gel(tabwm,k), gtanh(t, prec));
     }
   }
   return gerepilecopy(ltop, tab);
@@ -605,8 +611,9 @@ intn(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab)
   long m, k, L, i;
   pari_sp ltop = avma, av;
 
-  if (!checktabsimp(tab)) pari_err(typeer,"intnum");
-  if (!isinC(a) || !isinC(b)) pari_err(typeer,"intnum");
+  if (!checktabsimp(tab)) pari_err_TYPE("intnum",tab);
+  if (!isinC(a)) pari_err_TYPE("intnum",a);
+  if (!isinC(b)) pari_err_TYPE("intnum",b);
   m = itos(TABm(tab));
   tabx0 = TABx0(tab); tabw0 = TABw0(tab);
   tabxp = TABxp(tab); tabwp = TABwp(tab); L = lg(tabxp);
@@ -642,7 +649,7 @@ intnsing(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab, long prec)
   long m, k, L, i;
   pari_sp ltop = avma, av;
 
-  if (!checktabsimp(tab)) pari_err(typeer,"intnum");
+  if (!checktabsimp(tab)) pari_err_TYPE("intnum",tab);
   m = itos(TABm(tab));
   tabx0 = TABx0(tab); tabw0 = TABw0(tab);
   tabxp = TABxp(tab); tabwp = TABwp(tab); L = lg(tabxp);
@@ -683,7 +690,7 @@ intninfpm(void *E, GEN (*eval)(void*, GEN), GEN a, long si, GEN tab)
   long m, L, k, h = 0, pas, i;
   pari_sp ltop = avma, av;
 
-  if (!checktabdoub(tab)) pari_err(typeer,"intnum");
+  if (!checktabdoub(tab)) pari_err_TYPE("intnum",tab);
   m = itos(TABm(tab));
   tabx0 = TABx0(tab); tabw0 = TABw0(tab);
   tabxp = TABxp(tab); tabwp = TABwp(tab); L = lg(tabxp);
@@ -720,7 +727,7 @@ intninfinfintern(void *E, GEN (*eval)(void*, GEN), GEN tab, long flag)
   long m, L, k, i, spf;
   pari_sp ltop = avma;
 
-  if (!checktabsimp(tab)) pari_err(typeer,"intnum");
+  if (!checktabsimp(tab)) pari_err_TYPE("intnum",tab);
   m = itos(TABm(tab));
   tabx0 = TABx0(tab); tabw0 = TABw0(tab);
   tabxp = TABxp(tab); tabwp = TABwp(tab); L = lg(tabxp);
@@ -777,10 +784,12 @@ enum {
   f_YOSCS  = 5, /* +\infty, sine oscillating */
   f_YOSCC  = 6  /* +\infty, cosine oscillating */
 };
-
-#define is_fin_f(c) ((c) == f_REG || (c) == f_SING) /* is c finite */
-#define is_slow_f(c) ((c) == f_YSLOW || (c) == f_YVSLO) /* slow case */
-#define is_osc_f(c) ((c) == f_YOSCS || (c) == f_YOSCC) /* oscillating case */
+/* is c finite */
+static int
+is_fin_f(long c) { return c == f_REG || c == f_SING; }
+/* oscillating case: valid for +oo (c > 0) or -oo (c < 0) */
+static int
+is_osc_f(long c) { c = labs(c); return c == f_YOSCS || c == f_YOSCC; }
 
 static GEN
 f_getycplx(GEN a, long prec)
@@ -797,43 +806,55 @@ f_getycplx(GEN a, long prec)
   return tmp;
 }
 
-static long
-code_aux(GEN a2, int warn)
+static void
+err_code(GEN a, const char *name)
 {
-  GEN a2R = real_i(a2), a2I = imag_i(a2);
-  long s = gsigne(a2I);
+  char *s = stack_sprintf("intnum [incorrect %s]", name);
+  pari_err_TYPE(s, a);
+}
+
+/* a = [[+/-1], alpha]*/
+static long
+code_aux(GEN a, const char *name)
+{
+  GEN re, im, alpha = gel(a,2);
+  long s;
+  if (!isinC(alpha)) err_code(a, name);
+  re = real_i(alpha);
+  im = imag_i(alpha);
+  s = gsigne(im);
   if (s)
   {
-    if(warn && !gequal0(a2R))
-      pari_warn(warner,"both nonzero real and imag. part in coding, real ignored");
+    if(!gequal0(re))
+      pari_warn(warner,"real(z)*imag(z)!=0 in endpoint code, real(z) ignored");
     return s > 0 ? f_YOSCC : f_YOSCS;
   }
-  if (gequal0(a2R) || gcmpgs(a2R, -2)<=0) return f_YSLOW;
-  if (gsigne(a2R) > 0) return f_YFAST;
-  if (gcmpgs(a2R, -1) >= 0) pari_err(talker,"incorrect a or b in intnum");
+  if (gequal0(re) || gcmpgs(re, -2)<=0) return f_YSLOW;
+  if (gsigne(re) > 0) return f_YFAST;
+  if (gcmpgs(re, -1) >= 0) err_code(a, name);
   return f_YVSLO;
 }
 
 static long
-transcode(GEN a, long warn)
+transcode(GEN a, const char *name)
 {
   GEN a1, a2;
-  long la;
-
   if (typ(a) != t_VEC) return f_REG;
-  la = lg(a);
-  if (la == 1 || la > 3) pari_err(talker,"incorrect a or b in intnum");
-  if (la == 2) return gsigne(gel(a,1)) > 0 ? f_YSLOW : -f_YSLOW;
+  switch(lg(a))
+  {
+    case 2: return gsigne(gel(a,1)) > 0 ? f_YSLOW : -f_YSLOW;
+    case 3: a1 = gel(a,1); a2 = gel(a,2); break;
+    default: err_code(a,name);
+  }
   a1 = gel(a,1);
   a2 = gel(a,2);
   if (typ(a1) != t_VEC)
   {
-    if (!isinC(a1) || !isinR(a2) || gcmpgs(a2, -1) <= 0)
-      pari_err(talker,"incorrect a or b in intnum");
+    if (!isinC(a1) || !isinR(a2) || gcmpgs(a2, -1) <= 0) err_code(a,name);
     return gsigne(a2) < 0 ? f_SING : f_REG;
   }
-  if (lg(a1) != 2 || !isinC(a2)) pari_err(talker,"incorrect a or b in intnum");
-  return gsigne(gel(a1,1)) * code_aux(a2, warn);
+  if (lg(a1) != 2) err_code(a,name);
+  return gsigne(gel(a1,1)) * code_aux(a, name);
 }
 
 /* computes the necessary tabs, knowing a, b and m */
@@ -903,10 +924,10 @@ intnuminit(GEN a, GEN b, long m, long prec)
   long codea, codeb, l;
   GEN T, U, km, kma, kmb, tmp;
 
-  if (m > 30) pari_err(talker,"m too large in intnuminit");
-  l = prec + 1;
-  codea = transcode(a, 1);
-  codeb = transcode(b, 1);
+  if (m > 30) pari_err_OVERFLOW("intnuminit [m]");
+  l = prec+EXTRAPREC;
+  codea = transcode(a, "a");
+  codeb = transcode(b, "b");
   if (is_fin_f(codea) && is_fin_f(codeb)) return inittanhsinh(m, l);
   if (labs(codea) > labs(codeb)) { swap(a, b); lswap(codea, codeb); }
   if (codea == f_REG)
@@ -1002,7 +1023,7 @@ intnuminit0(GEN a, GEN b, GEN tab, long prec)
   if (!tab) m = 0;
   else if (typ(tab) != t_INT)
   {
-    if (!checktab(tab)) pari_err(typeer,"intnuminit0");
+    if (!checktab(tab)) pari_err_TYPE("intnuminit0",tab);
     return tab;
   }
   else
@@ -1016,7 +1037,7 @@ sumnuminit0(GEN a, GEN tab, long sgn, long prec)
   if (!tab) m = 0;
   else if (typ(tab) != t_INT)
   {
-    if (!checktab(tab)) pari_err(typeer,"sumnuminit0");
+    if (!checktab(tab)) pari_err_TYPE("sumnuminit0",tab);
     return tab;
   }
   else
@@ -1024,23 +1045,13 @@ sumnuminit0(GEN a, GEN tab, long sgn, long prec)
   return sumnuminit(a, m, sgn, prec);
 }
 
-/* here always eps = 2^(-k). */
-static GEN
-myderiv_num(void *E, GEN (*eval)(void*, GEN), GEN a, GEN eps, long k, long prec)
-{
-  GEN tmp = gmul2n(gsub(eval(E, gadd(a,eps)), eval(E, gsub(a,eps))), k - 1);
-  return gprec_w(tmp, prec);
-}
-
-/* User-defined change of variable phi(t) = f(t), where t always goes from
- * -\infty to +\infty, and a and b are as in intnuminit. If [a,b] compact,
- * assume phi(t) odd, otherwise assume nothing. */
+/* User-defined change of variable phi(t) = f(t), where t goes from -oo to +oo,
+ * and a and b are as in intnuminit. If [a,b] compact, assume phi(t) odd. */
 static int
-condfin(long code, GEN xw, GEN xwmod, long eps, long m, long k)
+condfin(long code, GEN xw, long eps, long m, long k)
 {
   GEN x, w;
   eps -= 8; /* for safety. Lose 8 bits, but took 1 whole word extra. */
-  if (!is_osc_f(labs(code))) xw = xwmod;
   x = gel(xw,1);
   w = gel(xw,2);
   switch(labs(code))
@@ -1052,123 +1063,118 @@ condfin(long code, GEN xw, GEN xwmod, long eps, long m, long k)
     case f_YFAST:
       return cmprs(x, (long)(LOG2 * (gexpo(w) + eps) + 1)) > 0;
     case f_YOSCS: case f_YOSCC:
-      return gexpo(x) + m + expi(stoi(10 * k)) < - eps;
+      return gexpo(x) + m + expu(10 * k) < - eps;
     default: return 0;
   }
 }
 
-/* Do not change the numbers below unless you understand what you are doing. */
-enum {
-  f_COMP = -1, /* [a,b] */
-  f_SEMI =  0, /* [a,+-\infty[, no oscillation */
-  f_OSC1 =  1, /* [a,+-\infty[, oscillation */
-  f_INF  =  2, /* ]-\infty,+\infty[, no oscillation */
-  f_OSC2 =  3  /* ]-\infty,+\infty[, oscillation */
-};
-
-#define not_osc(fl) ((fl) == f_COMP || (fl) == f_SEMI || (fl) == f_INF)
-#define not_odd(fl) ((fl) == f_SEMI || (fl) == f_OSC1)
-
+/* eps = 2^(-k). Return f'(a) ~ (f(a+eps) - f(a-eps)) / 2eps*/
 static GEN
-ffprime(void *E, GEN (*eval)(void*, GEN), GEN xpr, GEN xprn, GEN eps, long h, long precl)
+myderiv_num(void *E, GEN (*eval)(void*, GEN), GEN a, GEN eps, long k, long prec)
 {
-  GEN z = cgetg(3, t_VEC);
-  gel(z,1) = eval(E, xpr);
-  gel(z,2) = myderiv_num(E, eval, xprn, eps, h, precl);
-  return z;
+  GEN d = gmul2n(gsub(eval(E, gadd(a,eps)), eval(E, gsub(a,eps))), k-1);
+  return gprec_w(d, prec);
 }
-
+/* zp = z to a higher accuracy (enough to evaluate numerical derivative) */
 static GEN
-ffmodify(GEN tmp, GEN ab, long flag)
+ffprime(void *E, GEN (*eval)(void*, GEN), GEN z, GEN zp, GEN eps, long h, long precl)
 {
-  GEN z, t;
-
-  if (not_osc(flag)) return tmp;
-  t = ginv(gsubsg(1, gel(tmp,1)));
-  z = cgetg(3, t_VEC);
-  gel(z,1) = gmul(ab, t);
-  gel(z,2) = gadd(t, gmul(gsqr(t), gmul(ab, gel(tmp,2))));
-  return z;
+  GEN f = eval(E, z), fp = myderiv_num(E, eval, zp, eps, h, precl);
+  return mkvec2(f, fp);
+}
+/* v = [f(z), f'(z)]. Return h := z/(1-f(z)), h + h^2 zf'(z) */
+static GEN
+ffmodify(GEN v, GEN z)
+{
+  GEN f = gel(v,1), fp = gel(v,2), h = ginv(gsubsg(1, f));
+  return mkvec2(gmul(z, h), gadd(h, gmul(gsqr(h), gmul(z,fp))));
 }
-
 GEN
 intnuminitgen(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, long m,
               long flext, long prec)
 {
+  enum {
+    f_COMP, /* [a,b] */
+    f_SEMI, /* [a,+-oo[, no oscillation */
+    f_OSC1, /* [a,+-oo[, oscillation */
+    f_INF , /* ]-oo,+oo[, no oscillation */
+    f_OSC2  /* ]-oo,+oo[, oscillation */
+  };
   pari_sp ltop = avma;
-  GEN hpr, hnpr, eps, pisurh = gen_0, tmpxw, tmpxwmodp, tmpxwmodm = gen_0, ab;
-  long k, h, newprec, nt, lim, ntn, precl = prec + 1;
-  long flag = f_SEMI, codea = transcode(a, 1), codeb = transcode(b, 1);
+  GEN hnpr, eps, v;
+  long k, h, newprec, lim, precl = prec+EXTRAPREC;
+  long flag, codea = transcode(a, "a"), codeb = transcode(b, "b");
+  int NOT_OSC, NOT_ODD;
   intdata D; intinit_start(&D, m, flext, precl);
 
-  if (flag < 0 || flag >= 3) pari_err(flagerr,"intnuminitgen");
-  lim = lg(D.tabxp) - 1;
-  if (is_osc_f(labs(codea)) || is_osc_f(labs(codeb)))
-    { pisurh = Pi2n(D.m, precl); flag = f_OSC1; }
+  flag = f_SEMI;
+  if (is_osc_f(codea) || is_osc_f(codeb)) flag = f_OSC1;
   if (is_fin_f(codea) && is_fin_f(codeb)) flag = f_COMP;
   else if (!is_fin_f(codea) && !is_fin_f(codeb))
   {
-    if (codea * codeb > 0)
-      pari_err(talker,"infinities of the same sign in intnuminitgen");
-    if (labs(codea) != labs(codeb))
-      pari_err(talker,"infinities of different type in intnuminitgen");
+    if (codea * codeb > 0) return gen_0;
+    if (codea != -codeb)
+      pari_err_TYPE("intnuminitgen [infinities of different types]",
+                    mkvec2(a,b));
     flag = (flag == f_SEMI) ? f_INF : f_OSC2;
   }
+  NOT_OSC = (flag == f_COMP || flag == f_SEMI || flag == f_INF);
+  NOT_ODD = (flag == f_SEMI || flag == f_OSC1);
+
   newprec = (3*precl - 1)>>1;
-  h = bit_accuracy(precl)/2;
+  h = prec2nbits(precl)/2;
   eps = real2n(-h, newprec);
 
-  if (not_osc(flag) || !gequal1(eval(E, gen_0)))
+  if (NOT_OSC || !gequal1(eval(E, gen_0)))
   {
-    ab = real_0(precl);
-    tmpxw = ffprime(E, eval, ab, real_0(newprec), eps, h, precl);
-    tmpxwmodp = ffmodify(tmpxw, ab, flag);
-    D.tabx0 = gel(tmpxwmodp,1);
-    D.tabw0 = gel(tmpxwmodp,2);
+    GEN a0 = real_0(precl), a0n = real_0(newprec), xw, xwmod;
+    xw = ffprime(E, eval, a0, a0n, eps, h, precl);
+    xwmod = NOT_OSC? xw: ffmodify(xw, a0);
+    D.tabx0 = gel(xwmod,1);
+    D.tabw0 = gel(xwmod,2);
   }
   else
   {
-    tmpxw = gdiv(pol_x(0), gsubsg(1, eval(E, gadd(pol_x(0), zeroser(0, 4)))));
-    D.tabx0 = gprec_w(polcoeff0(tmpxw, 0, 0), precl);
-    D.tabw0 = gprec_w(polcoeff0(tmpxw, 1, 0), precl);
+    GEN xw = gdiv(pol_x(0), gsubsg(1, eval(E, gadd(pol_x(0), zeroser(0, 4)))));
+    D.tabx0 = gprec_w(polcoeff0(xw, 0, 0), precl);
+    D.tabw0 = gprec_w(polcoeff0(xw, 1, 0), precl);
   }
-  hpr = real2n(-D.m, precl);
-  hnpr= real2n(-D.m, newprec);
+  /* precl <= newprec */
+  hnpr = real2n(-D.m, newprec);
+  lim = lg(D.tabxp) - 1;
   for (k = 1; k <= lim; k++)
   {
+    GEN akn = mulur(k, hnpr), ak = rtor(akn, precl), xw, xwmod;
     int finb;
-    ab = mulur(k, hpr);
-    tmpxw = ffprime(E, eval, ab, mulur(k, hnpr), eps, h, precl);
-    tmpxwmodp = ffmodify(tmpxw, ab, flag);
-    D.tabxp[k] = tmpxwmodp[1];
-    D.tabwp[k] = tmpxwmodp[2];
-    finb = condfin(codeb, tmpxw, tmpxwmodp, D.eps, D.m, k);
-    if (not_odd(flag))
+    xw = ffprime(E, eval, ak, akn, eps, h, precl);
+    xwmod = NOT_OSC? xw: ffmodify(xw, ak);
+    gel(D.tabxp,k) = gel(xwmod,1);
+    gel(D.tabwp,k) = gel(xwmod,2);
+    finb = condfin(codeb, is_osc_f(codeb)? xw: xwmod, D.eps, D.m, k);
+    if (NOT_ODD)
     {
-      ab = negr(ab);
-      tmpxw = ffprime(E, eval, ab, mulsr(-k, hnpr), eps, h, precl);
-      tmpxwmodm = ffmodify(tmpxw, ab, flag);
-      D.tabxm[k] = tmpxwmodm[1];
-      D.tabwm[k] = tmpxwmodm[2];
-      if (finb && condfin(codea, tmpxw, tmpxwmodm, D.eps, D.m, k)) break;
+      ak = negr(ak); akn = negr(akn);
+      xw = ffprime(E, eval, ak, akn, eps, h, precl);
+      xwmod = NOT_OSC? xw: ffmodify(xw, ak);
+      gel(D.tabxm,k) = gel(xwmod,1);
+      gel(D.tabwm,k) = gel(xwmod,2);
+      if (finb && condfin(codea, is_osc_f(codeb)? xw: xwmod, D.eps, D.m, k))
+        break;
     }
     else if (finb) break;
   }
-  nt = k - 1;
-  if (!not_osc(flag))
+  v = intinit_end(&D, k-1, NOT_ODD? k-1: 0);
+  if (!NOT_OSC)
   {
-    D.tabx0 = gmul(D.tabx0, pisurh);
-    D.tabw0 = gmul(D.tabw0, pisurh);
-    setlg(D.tabxp, nt + 1); D.tabxp = gmul(D.tabxp, pisurh);
-    setlg(D.tabwp, nt + 1); D.tabwp = gmul(D.tabwp, pisurh);
+    GEN C = Pi2n(D.m, precl);
+    TABx0(v) = gmul(TABx0(v), C);
+    TABw0(v) = gmul(TABw0(v), C);
+    TABxp(v) = RgV_Rg_mul(TABxp(v), C);
+    TABwp(v) = RgV_Rg_mul(TABwp(v), C);
+    TABxm(v) = RgV_Rg_mul(TABxm(v), C);
+    TABwm(v) = RgV_Rg_mul(TABwm(v), C);
   }
-  if (flag == f_OSC1)
-  {
-    setlg(D.tabxm, nt + 1); D.tabxm = gmul(D.tabxm, pisurh);
-    setlg(D.tabwm, nt + 1); D.tabwm = gmul(D.tabwm, pisurh);
-  }
-  ntn = not_odd(flag) ? nt : 0;
-  return gerepilecopy(ltop, intinit_end(&D, nt, ntn));
+  return gerepilecopy(ltop, v);
 }
 
 /* Assigns the values of the function weighted by w[k] at quadrature points x[k]
@@ -1236,7 +1242,7 @@ intnum_i(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab, long prec)
 {
   GEN tmp, S = gen_0, res1, res2, tm, pi2, pi2p, pis2, pis2p, kma, kmb;
   GEN SP, SN;
-  long tmpi, sgns = 1, codea = transcode(a, 0), codeb = transcode(b, 0);
+  long tmpi, sgns = 1, codea = transcode(a, "a"), codeb = transcode(b, "b");
 
   if (codea == f_REG && typ(a) == t_VEC) a = gel(a,1);
   if (codeb == f_REG && typ(b) == t_VEC) b = gel(b,1);
@@ -1310,7 +1316,7 @@ intnum_i(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab, long prec)
       SP = intninfpm(E, eval, coupeb, 1, gel(tab,2));
     else
     {
-      if (codeb != f_YOSCC) pari_err(bugparier, "code error in intnum");
+      if (codeb != f_YOSCC) pari_err_BUG("code error in intnum");
       if (gequal(kma, kmb))
         SP = intninfpm(E, eval, coupeb, 1, gel(tab,2));
       else
@@ -1330,11 +1336,10 @@ GEN
 intnum(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab, long prec)
 {
   pari_sp ltop = avma;
-  long l = prec + 1;
+  long l = prec+EXTRAPREC;
   GEN S;
 
-  tab = intnuminit0(a, b, tab, prec); /* prec + 1 is done in intnuminit0 */
-
+  tab = intnuminit0(a, b, tab, prec);
   S = intnum_i(E, eval, gprec_w(a, l), gprec_w(b, l), tab, l);
   return gerepilecopy(ltop, gprec_wtrunc(S, prec));
 }
@@ -1342,6 +1347,7 @@ intnum(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN tab, long prec)
 typedef struct auxint_s {
   GEN a, R, pi;
   GEN (*f)(void*, GEN);
+  GEN (*w)(GEN, long);
   long prec;
   void *E;
 } auxint_t;
@@ -1376,39 +1382,24 @@ gettmpP(GEN x) { return mkvec2(mkvec(gen_1), x); }
 static GEN
 gettmpN(GEN tmpP) { return mkvec2(gneg(gel(tmpP,1)), gel(tmpP,2)); }
 
+/* w(Rt) f(a+it) */
 static GEN
-auxinvcos(void *E, GEN t)
+auxinv(void *E, GEN t)
 {
   auxint_t *D = (auxint_t*) E;
-  GEN tmp = gcos(gmul(D->R, t), D->prec);
+  GEN tmp = D->w(gmul(D->R, t), D->prec);
   return gmul(tmp, D->f(D->E, gadd(D->a, mulcxI(t))));
 }
 static GEN
-auxinvsin(void *E, GEN t)
-{
-  auxint_t *D = (auxint_t*) E;
-  GEN tmp = gsin(gmul(D->R, t), D->prec);
-  return gmul(tmp, D->f(D->E, gadd(D->a, mulcxI(t))));
-}
-static GEN
-auxinvexp(void *E, GEN t)
-{
-  auxint_t *D = (auxint_t*) E;
-  GEN tmp = gexp(gmul(D->R, t), D->prec);
-  return gmul(tmp, D->f(D->E, gadd(D->a, mulcxI(t))));
-}
-
-static GEN
-intinvintern(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long flag, long prec)
+intinvintern(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long prec)
 {
   auxint_t D;
   GEN z, zR, zI, tmpP, tmpN;
 
-  if (typ(sig) != t_VEC) sig = mkvec2(sig, stoi(flag));
   if (lg(sig) != 3 || !isinR(gel(sig,1)) || !isinR(gel(sig,2)))
-    pari_err(typeer,"integral transform");
+    pari_err_TYPE("integral transform",sig);
   if (gsigne(gel(sig,2)) < 0)
-    pari_err(talker,"exponential increase in integral transform");
+    pari_err_OVERFLOW("integral transform [exponential increase]");
   D.a = gel(sig,1);
   D.prec = prec;
   D.f = eval;
@@ -1419,16 +1410,19 @@ intinvintern(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long fla
     tmpP = gettmpP(mulcxI(gabs(x, prec)));
     tmpN = gettmpN(tmpP);
     tab = intnuminit0(tmpN, tmpP, tab, prec);
-    zR = intnum_i(&D, &auxinvcos, tmpN, tmpP, tab, prec);
+    D.w = gcos;
+    zR = intnum_i(&D, &auxinv, tmpN, tmpP, tab, prec);
     gel(tmpP,2) = gneg(gel(tmpP,2));
-    zI = intnum_i(&D, &auxinvsin, gettmpN(tmpP), tmpP, tab, prec);
+    D.w = gsin;
+    zI = intnum_i(&D, &auxinv, gettmpN(tmpP), tmpP, tab, prec);
     z = gadd(zR, mulcxI(zI));
   }
   else
   {
     D.R = mulcxI(x);
     tmpP = gettmpP(gel(sig,2));
-    z = intnum(&D, &auxinvexp, gettmpN(tmpP), tmpP, tab, prec);
+    D.w = gexp;
+    z = intnum(&D, &auxinv, gettmpN(tmpP), tmpP, tab, prec);
   }
   return gdiv(gmul(gexp(gmul(gel(sig,1), x), prec), z), Pi2n(1, prec));
 }
@@ -1438,7 +1432,8 @@ intinvintern(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long fla
 GEN
 intmellininv(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long prec)
 {
-  return intinvintern(E, eval, sig, gneg(glog(x, prec)), tab, 1, prec);
+  if (typ(sig) != t_VEC) sig = mkvec2(sig, gen_1);
+  return intinvintern(E, eval, sig, gneg(glog(x, prec)), tab, prec);
 }
 
 /* If sig = [sigR, e]: if e = 0, slowly decreasing, if e > 0, exponentially
@@ -1446,7 +1441,8 @@ intmellininv(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long pre
 GEN
 intlaplaceinv(void *E, GEN (*eval)(void*, GEN), GEN sig, GEN x, GEN tab, long prec)
 {
-  return intinvintern(E, eval, sig, x, tab, 0, prec);
+  if (typ(sig) != t_VEC) sig = mkvec2(sig, gen_0);
+  return intinvintern(E, eval, sig, x, tab, prec);
 }
 
 /* assume tab computed with additional weights f(sig + I*T) */
@@ -1470,9 +1466,9 @@ intmellininvshort(GEN sig, GEN x, GEN tab, long prec)
 
   if (typ(sig) != t_VEC) sig = mkvec2(sig, gen_1);
   if (lg(sig) != 3 || !isinR(gel(sig,1)) || !isinR(gel(sig,2)))
-    pari_err(typeer,"intmellininvshort");
+    pari_err_TYPE("intmellininvshort",sig);
   if (gsigne(gel(sig,2)) <= 0)
-    pari_err(talker,"need exponential decrease in intinvmellinshort");
+    pari_err_OVERFLOW("intinvmellinshort [need exponential decrease]");
   D.L = mulcxI(LX);
   D.prec = prec;
   tmpP = gettmpP(gel(sig,2));
@@ -1482,47 +1478,41 @@ intmellininvshort(GEN sig, GEN x, GEN tab, long prec)
 
 /* a as in intnum. flag = 0 for sin, flag = 1 for cos. */
 static GEN
-mytra(GEN a, GEN x, long flag)
+mytra(GEN a, GEN x, long flag, const char *name)
 {
   GEN b, xa;
-  long s, codea = transcode(a, 1);
+  long s, codea = transcode(a, name);
 
   switch (labs(codea))
   {
     case f_REG: case f_SING: case f_YFAST: return a;
     case f_YSLOW: case f_YVSLO:
       xa = real_i(x); s = gsigne(xa);
-      if (!s) pari_err(talker,"x = 0 in Fourier");
+      if (!s) pari_err_DOMAIN("Fourier transform","Re(x)","=",gen_0,x);
       if (s < 0) xa = gneg(xa);
       b = cgetg(3, t_VEC);
       gel(b,1) = mkvec( codea > 0 ? gen_1 : gen_m1 );
       gel(b,2) = (flag? mulcxI(xa): mulcxmI(xa));
       return b;
     case f_YOSCS: case f_YOSCC:
-      pari_err(impl,"Fourier transform of oscillating functions");
+      pari_err_IMPL("Fourier transform of oscillating functions");
   }
   return NULL;
 }
 
+/* w(ta) f(t) */
 static GEN
-auxfoursin(void *E, GEN t)
+auxfour(void *E, GEN t)
 {
   auxint_t *D = (auxint_t*) E;
-  return gmul(gsin(gmul(t, D->a), D->prec), D->f(D->E, t));
-}
-
-static GEN
-auxfourcos(void *E, GEN t)
-{
-  auxint_t *D = (auxint_t*) E;
-  return gmul(gcos(gmul(t, D->a), D->prec), D->f(D->E, t));
+  return gmul(D->w(gmul(t, D->a), D->prec), D->f(D->E, t));
 }
 
 GEN
 intfouriersin(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN x, GEN tab, long prec)
 {
   auxint_t D;
-  GEN z, tmp;
+  GEN tmp;
 
   if (gequal0(x)) return gcopy(x);
   tmp = gmul(x, Pi2n(1, prec));
@@ -1531,15 +1521,17 @@ intfouriersin(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN x, GEN tab, lo
   D.prec = prec;
   D.f = eval;
   D.E = E;
-  z = intnum(&D, &auxfoursin, mytra(a, tmp, 0), mytra(b, tmp, 0), tab, prec);
-  return z;
+  a = mytra(a,tmp,0,"a");
+  b = mytra(b,tmp,0,"b");
+  D.w = gsin;
+  return intnum(&D, &auxfour, a, b, tab, prec);
 }
 
 GEN
 intfouriercos(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN x, GEN tab, long prec)
 {
   auxint_t D;
-  GEN z, tmp;
+  GEN tmp;
 
   if (gequal0(x)) return intnum(E, eval, a, b, tab, prec);
   tmp = gmul(x, Pi2n(1, prec));
@@ -1548,8 +1540,10 @@ intfouriercos(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, GEN x, GEN tab, lo
   D.prec = prec;
   D.f = eval;
   D.E = E;
-  z = intnum(&D, &auxfourcos, mytra(a, tmp, 1), mytra(b, tmp, 1), tab, prec);
-  return z;
+  a = mytra(a,tmp,1,"a");
+  b = mytra(b,tmp,1,"b");
+  D.w = gcos;
+  return intnum(&D, &auxfour, a, b, tab, prec);
 }
 
 GEN
@@ -1573,9 +1567,9 @@ intnumromb(void *E, GEN (*eval)(void *, GEN), GEN a, GEN b, long flag, long prec
     case 1: z = rombint(E, eval, a, b, prec); break;
     case 2: z = qromi  (E, eval, a, b, prec); break;
     case 3: z = qrom2  (E, eval, a, b, prec); break;
-    default: pari_err(flagerr); return NULL; /* not reached */
+    default: pari_err_FLAG("intnumromb"); return NULL; /* not reached */
   }
-  if (!z) pari_err(talker, "too many iterations in intnumromb");
+  if (!z) pari_err_IMPL("intnumromb recovery [too many iterations]");
   return gerepileupto(av, z);
 }
 
@@ -1734,7 +1728,7 @@ sumnumall(void *E, GEN (*eval)(void*, GEN), GEN a, GEN sig, GEN tab, long flag,
 
   b = suminit_start(sig);
   flii = gequal0(gel(b,2));
-  if (!is_scalar_t(typ(a))) pari_err(talker, "incorrect beginning value in sumnum");
+  if (!is_scalar_t(typ(a))) pari_err_TYPE("sumnum",a);
   tab = sumnuminit0(sig, tab, sgn, prec);
 
   signew = (typ(sig) == t_VEC) ? gel(sig,1) : sig;
