@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -31,15 +29,15 @@ checktnf(GEN tnf)
 {
   long l = lg(tnf);
   if (typ(tnf)!=t_VEC || (l!=8 && l!=3)) return 0;
-  if (typ(tnf[1]) != t_VEC) return 0;
+  if (typ(gel(tnf,1)) != t_VEC) return 0;
   if (l != 8) return 1; /* S=0 */
 
   (void)checkbnf(gel(tnf,2));
-  return (typ(tnf[3]) == t_COL
-       && typ(tnf[4]) == t_COL
-       && typ(tnf[5]) == t_MAT
-       && typ(tnf[6]) == t_MAT
-       && typ(tnf[7]) == t_VEC);
+  return (typ(gel(tnf,3)) == t_COL
+       && typ(gel(tnf,4)) == t_COL
+       && typ(gel(tnf,5)) == t_MAT
+       && typ(gel(tnf,6)) == t_MAT
+       && typ(gel(tnf,7)) == t_VEC);
 }
 
 static GEN
@@ -58,28 +56,14 @@ myround(GEN x, long dir)
   return gmul(x, gadd(gen_1, eps));
 }
 
-/* Returns the index of the largest element in a vector */
+/* v a t_VEC/t_VEC */
 static GEN
-_Vecmax(GEN Vec, long *ind)
-{
-  long k, tno = 1, l = lg(Vec);
-  GEN tmax = gel(Vec,1);
-  for (k = 2; k < l; k++)
-    if (gcmp(gel(Vec,k),tmax) > 0) { tmax = gel(Vec,k); tno = k; }
-  if (ind) *ind = tno;
-  return tmax;
-}
-
-static GEN
-Vecmax(GEN v) { return _Vecmax(v, NULL); }
-
-static long
-Vecmaxind(GEN v) { long i; (void)_Vecmax(v, &i); return i; }
+vecmax_shallow(GEN v) { return gel(v, vecindexmax(v)); }
 
 static GEN
 tnf_get_roots(GEN poly, long prec, long S, long T)
 {
-  GEN R0 = cleanroots(poly, prec), R = cgetg(lg(R0), t_COL);
+  GEN R0 = QX_complex_roots(poly, prec), R = cgetg(lg(R0), t_COL);
   long k;
 
   for (k=1; k<=S; k++) gel(R,k) = gel(R0,k);
@@ -149,26 +133,26 @@ static GEN
 T_A_Matrices(GEN MatFU, long r, GEN *eps5, long prec)
 {
   GEN A, p1, m1, IntM, nia, eps3, eps2;
-  long e = bit_accuracy(prec);
+  long e = prec2nbits(prec);
 
   m1 = rowslice(vecslice(MatFU, 1,r), 1,r); /* minor order r */
   m1 = logabs(m1, 3);
 
-  A = RgM_inv(m1); if (!A) pari_err(precer,"thue");
+  A = RgM_inv(m1); if (!A) pari_err_PREC("thue");
   IntM = RgM_Rg_add(RgM_mul(A,m1), gen_m1);
 
-  eps2 = gadd(vecmax(gabs(IntM, 3)), real2n(-e, 3)); /* t_REAL */
+  eps2 = gadd(vecmax(gabs(IntM, 3)), real2n(-e, LOWDEFAULTPREC)); /* t_REAL */
   nia = vecmax(gabs(A, 3));
-  if (typ(nia) != t_REAL) nia = gtofp(nia, 3);
+  if (typ(nia) != t_REAL) nia = gtofp(nia, LOWDEFAULTPREC);
 
   /* Check for the precision in matrix inversion. See paper, Lemma 2.4.2. */
   p1 = addrr(mulsr(r, gmul2n(nia, e)), eps2); /* t_REAL */
-  if (expo(p1) < -2*r) pari_err(precer,"thue");
+  if (expo(p1) < -2*r) pari_err_PREC("thue");
 
   p1 = addrr(mulsr(r, gmul2n(nia,-e)), eps2);
   eps3 = mulrr(mulsr(2*r*r,nia), p1);
   if (!signe(eps3))
-    eps3 = real2n(expo(eps3), 3);
+    eps3 = real2n(expo(eps3), LOWDEFAULTPREC);
   else
     eps3 = myround(eps3, 1);
 
@@ -194,7 +178,6 @@ inithue(GEN P, GEN bnf, long flag, long prec)
 
   if (!bnf)
   {
-    if (!gequal1(leading_term(P))) pari_err(talker,"non-monic polynomial in thue");
     bnf = Buchall(P, nf_FORCE, DEFAULTPREC);
     if (flag) (void)bnfcertify(bnf);
     else
@@ -208,7 +191,7 @@ inithue(GEN P, GEN bnf, long flag, long prec)
     ro = tnf_get_roots(P, prec_roots, s, t);
     MatFU = Conj_LH(bnf_get_fu(bnf), &ALH, ro, prec);
     if (MatFU) break;
-    prec_roots = (prec_roots << 1) - 2;
+    prec_roots = precdbl(prec_roots);
     if (DEBUGLEVEL>1) pari_warn(warnprec, "inithue", prec_roots);
   }
 
@@ -231,7 +214,8 @@ inithue(GEN P, GEN bnf, long flag, long prec)
     }
   c2 = gprec_w(myround(c2, -1), DEFAULTPREC);
 
-  if (t==0) x0 = gen_1;
+  if (t==0)
+    x0 = real_1(DEFAULTPREC);
   else
   {
     gpmin = NULL; /* min |P'(r_i)|, i > s */
@@ -243,7 +227,7 @@ inithue(GEN P, GEN bnf, long flag, long prec)
     gpmin = gprec_w(gpmin, DEFAULTPREC);
 
     /* Compute x0. See paper, Prop. 2.2.1 */
-    x0 = gmul(gpmin, Vecmax(gabs(imag_i(ro), prec)));
+    x0 = gmul(gpmin, vecmax_shallow(gabs(imag_i(ro), prec)));
     x0 = sqrtnr(gdiv(int2n(n-1), x0), n);
   }
   if (DEBUGLEVEL>1)
@@ -419,20 +403,25 @@ new_sol(GEN z, GEN S)
   return 1;
 }
 
+/* add solution (x,y) if not already known */
 static void
 add_sol(GEN *pS, GEN x, GEN y)
 {
   GEN u = mkvec2(x,y);
   if (new_sol(u, *pS)) *pS = shallowconcat(*pS, mkvec(u));
 }
-
+/* z = P(p,q), d = deg P, |z| = |rhs|. Check signs and (possibly)
+ * add solutions (p,q), (-p,-q) */
 static void
-check_sol(GEN x, GEN y, GEN P, GEN rhs, GEN *pS)
+add_pm(GEN *pS, GEN p, GEN q, GEN z, long d, GEN rhs)
 {
-  if (gequal0(y))
-  { if (equalii(powiu(x,degpol(P)), rhs)) add_sol(pS, x, gen_0); }
+  if (signe(z) == signe(rhs))
+  {
+    add_sol(pS, p, q);
+    if (!odd(d)) add_sol(pS, negi(p), negi(q));
+  }
   else
-  { if (equalii(poleval(RgX_rescale(P,y),x), rhs)) add_sol(pS, x, y); }
+    if (odd(d))  add_sol(pS, negi(p), negi(q));
 }
 
 /* Check whether a potential solution is a true solution. Return 0 if
@@ -445,13 +434,14 @@ CheckSol(GEN *pS, GEN z1, GEN z2, GEN P, GEN rhs, GEN ro)
 
   y = grndtoi(real_i(gdiv(gsub(z2,z1), gsub(ro1,ro2))), &e);
   if (e > 0) return 0;
+  if (!signe(y)) return 1; /* y = 0 taken care of in SmallSols */
   x = gadd(z1, gmul(ro1, y));
   x = grndtoi(real_i(x), &e);
   if (e > 0) return 0;
   if (e <= -13)
-  {
-    check_sol(     x ,      y,  P,rhs,pS);
-    check_sol(negi(x), negi(y), P,rhs,pS);
+  { /* y != 0 and rhs != 0; check whether P(x,y) = rhs or P(-x,-y) = rhs */
+    GEN z = poleval(RgX_rescale(P,y),x);
+    if (absi_equal(z, rhs)) add_pm(pS, x,y, z, degpol(P), rhs);
   }
   return 1;
 }
@@ -475,7 +465,11 @@ GuessQi(GEN b, GEN c, GEN *eps)
   *eps = mpabs(*eps); return Q;
 }
 
-/* Check for not-so-small solutions */
+/* x a t_REAL */
+static GEN
+myfloor(GEN x) { return expo(x) > 30 ? ceil_safe(x): floorr(x); }
+
+/* Check for not-so-small solutions. Return a t_REAL or NULL */
 static GEN
 MiddleSols(GEN *pS, GEN bound, GEN roo, GEN poly, GEN rhs, long s, GEN c1)
 {
@@ -491,7 +485,7 @@ MiddleSols(GEN *pS, GEN bound, GEN roo, GEN poly, GEN rhs, long s, GEN c1)
    * 2 + ==> continued fraction is normalized if last entry is 1
    * 3 + ==> start at a0, not a1 */
   nmax = 3 + (long)(gtodouble(logr_abs(bound)) / 0.4812118250596);
-  bound = (expo(bound) > 30)? ceil_safe(bound): floorr(bound);
+  bound = myfloor(bound);
 
   for (k = 1; k <= s; k++)
   {
@@ -509,26 +503,17 @@ MiddleSols(GEN *pS, GEN bound, GEN roo, GEN poly, GEN rhs, long s, GEN c1)
       if (cmpii(q, bound) > 0) break;
       if (DEBUGLEVEL >= 2) err_printf("Checking (+/- %Ps, +/- %Ps)\n",p, q);
 
-      z = poleval(RgX_rescale(poly,q), p); /* = P(p/q) q^dep(P) */
+      z = poleval(ZX_rescale(poly,q), p); /* = P(p/q) q^dep(P) */
       Q = dvmdii(rhs, z, &R);
       if (R != gen_0) continue;
       setabssign(Q);
       if (Z_ispowerall(Q, d, &Q))
       {
-        if (!is_pm1(Q)) {
-          p = mulii(p, Q);
-          q = mulii(q, Q);
-        }
-        if (signe(z) == signe(rhs))
-        {
-          add_sol(pS, p, q);
-          if (!odd(d)) add_sol(pS, negi(p), negi(q));
-        }
-        else
-          if (odd(d))  add_sol(pS, negi(p), negi(q));
+        if (!is_pm1(Q)) { p = mulii(p, Q); q = mulii(q, Q); }
+        add_pm(pS, p, q, z, d, rhs);
       }
     }
-    if (j == lg(t)) pari_err(bugparier, "Short continued fraction in thue");
+    if (j == lg(t)) pari_err_BUG("Short continued fraction in thue");
   }
   return bndcf;
 }
@@ -564,9 +549,14 @@ SmallSols(GEN S, GEN x3, GEN poly, GEN rhs)
   pari_sp av = avma, lim = stack_lim(av, 1);
   GEN X, P, rhs2;
   long j, l = lg(poly), n = degpol(poly);
-  ulong y, By = itou(floorr(x3));
+  ulong y, By;
 
-  if (DEBUGLEVEL>1) err_printf("* Checking for small solutions <= %lu\n", By);
+  x3 = myfloor(x3);
+
+  if (DEBUGLEVEL>1) err_printf("* Checking for small solutions <= %Ps\n", x3);
+  if (lgefint(x3) > 3)
+    pari_err_OVERFLOW(stack_sprintf("thue (SmallSols): y <= %Ps", x3));
+  By = itou(x3);
   /* y = 0 first: solve X^n = rhs */
   if (odd(n))
   {
@@ -597,7 +587,7 @@ SmallSols(GEN S, GEN x3, GEN poly, GEN rhs)
     if (low_stack(lim,stack_lim(av,1)))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"SmallSols");
-      S = gerepilecopy(av, S);
+      gerepileall(av, 2, &S, &rhs2);
       P = cgetg(l, t_POL); P[1] = poly[1];
     }
   }
@@ -632,9 +622,9 @@ thueinit(GEN pol, long flag, long prec)
   long k, s, lfa, dpol;
 
   if (checktnf(pol)) { bnf = checkbnf(gel(pol,2)); pol = gel(pol,1); }
-  if (typ(pol)!=t_POL) pari_err(notpoler,"thueinit");
+  if (typ(pol)!=t_POL) pari_err_TYPE("thueinit",pol);
   dpol = degpol(pol);
-  if (dpol <= 0) pari_err(constpoler,"thueinit");
+  if (dpol <= 0) pari_err_CONSTPOL("thueinit");
   RgX_check_ZX(pol, "thueinit");
   if (varn(pol)) { pol = leafcopy(pol); setvarn(pol, 0); }
   /* POL monic: POL(x) = C pol(x/L), L integer */
@@ -643,18 +633,19 @@ thueinit(GEN pol, long flag, long prec)
   pol = POL;
 
   fa = ZX_factor(pol);
-  lfa = lg(gel(fa,1));
+  lfa = lgcols(fa);
   if (lfa > 2 || itos(gcoeff(fa,1,2)) > 1)
   { /* reducible polynomial */
     GEN P, Q, R, g, f = gcoeff(fa,1,1), E = gcoeff(fa,1,2);
     long e = itos(E);
-    long vy = fetch_user_var("y_");
-    long va = fetch_user_var("a_");
-    long vb = fetch_user_var("b_");
+    long vy = fetch_var();
+    long va = fetch_var();
+    long vb = fetch_var();
     if (e != 1)
     {
       if (lfa == 2) {
         tnf = mkvec2(mkvec3(pol,C,L), mkvec2(thueinit(f, flag, prec), E));
+        delete_var(); delete_var(); delete_var();
         return gerepilecopy(av, tnf);
       }
       P = gpowgs(f,e);
@@ -666,10 +657,11 @@ thueinit(GEN pol, long flag, long prec)
     Q = RgX_Rg_sub(RgX_homogenize(g, vy), pol_x(vb));
     R = polresultant0(P, Q, -1, 0);
     tnf = mkvec2(mkvec3(pol,C,L), mkvec2(mkvecsmall4(degpol(f), e, va,vb),  R));
+    delete_var(); delete_var(); delete_var();
     return gerepilecopy(av, tnf);
   }
 
-  if (dpol <= 2) pari_err(talker,"invalid polynomial in thue (need deg>2)");
+  if (dpol <= 2) pari_err_DOMAIN("thue", "degree","<=",gen_2,pol);
   s = sturm(pol);
   if (s)
   {
@@ -681,9 +673,9 @@ thueinit(GEN pol, long flag, long prec)
     /* Guess precision by approximating Baker's bound. The guess is most of
      * the time not sharp, ie 10 to 30 decimal digits above what is _really_
      * necessary. Note that the limiting step is the reduction. See paper. */
-    PREC = 3 + (long)((5.83 + (dr+4)*5 + log(fact(dr+3)) + (dr+3)*log(dr+2) +
+    PREC = nbits2prec((long)((5.83 + (dr+4)*5 + log(fact(dr+3)) + (dr+3)*log(dr+2) +
                      (dr+3)*log(d) + log(log(2*d*(dr+2))) + (dr+1))
-                     / ((sizeof(long)/4)* 10.));
+                     /10.)*32+32);
 
     if (flag == 0) PREC = (long)(2.2 * PREC); /* Lazy, to be improved */
     if (PREC < prec) PREC = prec;
@@ -692,7 +684,7 @@ thueinit(GEN pol, long flag, long prec)
     for (;;)
     {
       if (( tnf = inithue(pol, bnf, flag, PREC) )) break;
-      PREC = (PREC<<1)-2;
+      PREC = precdbl(PREC);
       if (DEBUGLEVEL>1) pari_warn(warnprec,"thueinit",PREC);
       bnf = NULL; avma = av;
     }
@@ -734,7 +726,7 @@ init_get_B(long i1, long i2, GEN Delta, GEN Lambda, GEN eps5, baker_s *BS,
               gdiv(gel(BS->NE,3), gel(BS->NE,2)));
     lambda = divrr(garg(p1,prec), Pi2);
 
-    inverrdelta = shiftr(gabs(gel(fu,2),prec), bit_accuracy(prec)-1);
+    inverrdelta = shiftr(gabs(gel(fu,2),prec), prec2nbits(prec)-1);
   }
   if (DEBUGLEVEL>1) err_printf("  inverrdelta = %Ps\n",inverrdelta);
   BS->delta = delta;
@@ -785,7 +777,7 @@ get_B0(long i1, GEN Delta, GEN Lambda, GEN eps5, long prec, baker_s *BS)
     i2++; if (i2 == i1) i2++;
     if (i2 > BS->r) break;
   }
-  pari_err(bugparier,"thue (totally rational case)");
+  pari_err_BUG("thue (totally rational case)");
   return NULL; /* not reached */
 }
 
@@ -802,9 +794,10 @@ get_Bx_LLL(long i1, GEN Delta, GEN Lambda, GEN eps5, long prec, baker_s *BS)
     for (;;)
     {
       GEN oldBx = Bx, kappa = utoipos(10);
+      const long cfMAX = 10;
       long cf;
 
-      for (cf = 0; cf < 10; cf++, kappa = muliu(kappa,10))
+      for (cf = 0; cf < cfMAX; cf++, kappa = muliu(kappa,10))
       {
         int res = LLL_1stPass(&B0, kappa, BS, &Bx);
         if (res) break;
@@ -812,7 +805,7 @@ get_Bx_LLL(long i1, GEN Delta, GEN Lambda, GEN eps5, long prec, baker_s *BS)
       }
 
       /* FIXME: TO BE COMPLETED */
-      if (!step && cf == 10)
+      if (!step && cf == cfMAX)
       { /* Semirational or totally rational case */
         GEN Q, ep, q, l0, denbound;
 
@@ -835,7 +828,7 @@ get_Bx_LLL(long i1, GEN Delta, GEN Lambda, GEN eps5, long prec, baker_s *BS)
     i2++; if (i2 == i1) i2++;
     if (i2 > BS->r) break;
   }
-  pari_err(bugparier,"thue (totally rational case)");
+  pari_err_BUG("thue (totally rational case)");
   return NULL; /* not reached */
 }
 
@@ -857,7 +850,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
       if (!is_pm1(gel(csts, 7)) && !is_pm1(bnf_get_no(bnf)) && !is_pm1(rhs))
         pari_warn(warner, "The result returned by 'thue' is conditional on the GRH");
   }
-  else if (typ(ne) != t_VEC) pari_err(typeer, "thue");
+  else if (typ(ne) != t_VEC) pari_err_TYPE("thue",ne);
   if (lg(ne)==1) return NULL;
 
   nf_get_sign(bnf_get_nf(bnf), &s, &t);
@@ -884,7 +877,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
   x1 = gmax(x0, sqrtnr(shiftr(tmp,1),n));
 
   Vect = gmul(gabs(A,DEFAULTPREC), const_col(r, gen_1));
-  c14 = mulrr(c4, Vecmax(Vect));
+  c14 = mulrr(c4, vecmax_shallow(Vect));
   x2 = gmax(x1, sqrtnr(mulur(10,c14), n));
   if (DEBUGLEVEL>1) {
     err_printf("x1 -> %Ps\n",x1);
@@ -907,7 +900,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
     if (iroot <= r) gel(Vect,iroot) = stoi(1-n);
     Delta = RgM_RgC_mul(A,Vect);
 
-    c5 = Vecmax(gabs(Delta,Prec));
+    c5 = vecmax_shallow(gabs(Delta,Prec));
     c5  = myround(gprec_w(c5,DEFAULTPREC), 1);
     c7  = mulur(r,c5);
     BS.c10 = divur(n,c7);
@@ -922,7 +915,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
     for (;;)
     {
       if (( MatNE = Conj_LH(ne, &Hmu, ro, prec) )) break;
-      prec = (prec<<1)-2;
+      prec = precdbl(prec);
       if (DEBUGLEVEL>1) pari_warn(warnprec,"thue",prec);
       ro = tnf_get_roots(P, prec, s, t);
     }
@@ -947,7 +940,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
       }
       Lambda = RgM_RgC_mul(A,Vect2);
 
-      c6 = addrr(dbltor(0.1), Vecmax(gabs(Lambda,DEFAULTPREC)));
+      c6 = addrr(dbltor(0.1), vecmax_shallow(gabs(Lambda,DEFAULTPREC)));
       c6 = myround(c6, 1);
       c8 = addrr(dbltor(1.23), mulur(r,c6));
       c11= mulrr(shiftr(c3,1) , mpexp(divrr(mulur(n,c8),c7)));
@@ -964,7 +957,7 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
       BS.NE = NE;
       BS.Hmu = gel(Hmu,ine);
 
-      i1 = Vecmaxind(gabs(Delta,prec));
+      i1 = vecindexmax(gabs(Delta,prec));
       if (is_pm1(BS.Ind))
       {
         if (! (B0 = get_B0(i1, Delta, Lambda, eps5, prec, &BS)) ) goto PRECPB;
@@ -1043,8 +1036,9 @@ thue(GEN tnf, GEN rhs, GEN ne)
   pari_sp av = avma;
   GEN POL, C, L, x3, S;
 
-  if (!checktnf(tnf)) pari_err(talker,"not a tnf in thue");
-  if (typ(rhs) != t_INT) pari_err(typeer,"thue");
+  if (typ(tnf) == t_POL) tnf = thueinit(tnf, 0, DEFAULTPREC);
+  if (!checktnf(tnf)) pari_err_TYPE("thue [please apply thueinit()]", tnf);
+  if (typ(rhs) != t_INT) pari_err_TYPE("thue",rhs);
 
   /* solve P(x,y) = rhs <=> POL(L x, y) = C rhs, with POL monic in Z[X] */
   POL = gel(tnf,1);
@@ -1086,7 +1080,7 @@ thue(GEN tnf, GEN rhs, GEN ne)
     GEN P, D, v = gmael(tnf, 2, 1), R = gmael(tnf, 2, 2);
     long i, l, degf = v[1], e = v[2], va = v[3], vb = v[4];
     if (!signe(rhs)) {
-      if (degf == 1) pari_err(talker,"infinitely many solutions in thue");
+      if (degf == 1) pari_err_DOMAIN("thue","#sols","=",strtoGENstr("oo"),rhs);
       avma = av; return cgetg(1, t_VEC);
     }
     P = cgetg(lg(POL), t_POL); P[1] = POL[1];
@@ -1103,7 +1097,7 @@ thue(GEN tnf, GEN rhs, GEN ne)
       ry = nfrootsQ(Rab);
       for (k = 1; k < lg(ry); k++)
         if (typ(gel(ry,k)) == t_INT) check_y(&S, P, POL, gel(ry,k), rhs);
-      if (!odd(e)) {
+      if (odd(e)) {
         Rab = gsubst(gsubst(R, va, negi(df)), vb, negi(dg));
         ry = nfrootsQ(Rab);
         for (k = 1; k < lg(ry); k++)
@@ -1236,7 +1230,7 @@ static int
 get_sol_abs(struct sol_abs *T, GEN bnf, GEN a, GEN *ptPR)
 {
   GEN nf = bnf_get_nf(bnf);
-  GEN fact = Z_factor(a), P = gel(fact,1), E = gel(fact,2), PR;
+  GEN fact = absi_factor(a), P = gel(fact,1), E = gel(fact,2), PR;
   long N = nf_get_degree(nf), nP = lg(P)-1, Ngen, max, nPR, i, j;
 
   max = nP*N; /* upper bound for T->nPR */
@@ -1337,12 +1331,12 @@ bnfisintnormabs(GEN bnf, GEN a)
   GEN nf, res, PR;
   long i;
 
-  if (typ(a) != t_INT) pari_err(typeer,"bnfisintnormabs");
+  if (typ(a) != t_INT) pari_err_TYPE("bnfisintnormabs",a);
   bnf = checkbnf(bnf); nf = bnf_get_nf(bnf);
   if (!signe(a)) return mkvec(gen_0);
   if (is_pm1(a)) return mkvec(gen_1);
 
-  if (!get_sol_abs(&T, bnf, absi(a), &PR)) return cgetg(1, t_VEC);
+  if (!get_sol_abs(&T, bnf, a, &PR)) return cgetg(1, t_VEC);
   /* |a| > 1 => T.nPR > 0 */
   res = cgetg(T.sindex+1, t_VEC);
   for (i=1; i<=T.sindex; i++)
